// MacTools.cpp: implementation.
///////////////////////////////////////////////////////////////////////////////////////////////////
#include "StdAfx.h"
///////////////////////////////////////////////////////////////////////////////////////////////////
//
///////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef _MAC_
///////////////////////////////////////////////////////////////////////////////////////////////////
#define STRTERMINATOR   2
///////////////////////////////////////////////////////////////////////////////////////////////////
static  TCHAR           _DCNullString[]      = __T("");
const   DCStringData    _InternalStringData  = { -1, 0, 0, 0 };
const   DCStringData*   _DCNullStringData    = &_InternalStringData;
///////////////////////////////////////////////////////////////////////////////////////////////////
//
///////////////////////////////////////////////////////////////////////////////////////////////////
DCString::~DCString()   //  FREE ANY ATTACHED DATA
{
    Release();
}
///////////////////////////////////////////////////////////////////////////////////////////////////
void DCString::Init()
{
    m_StringData    = (DCStringData*)_DCNullStringData;
    m_pchData       = m_StringData->Content;
}
///////////////////////////////////////////////////////////////////////////////////////////////////
// CONSTRUTORS...
///////////////////////////////////////////////////////////////////////////////////////////////////
DCString::DCString()
{
    Init();
}
#ifndef __LINUX__
///////////////////////////////////////////////////////////////////////////////////////////////////
DCString::DCString (RESID ParmID)
{
    Init();

    LoadString(ParmID);
}
#endif
///////////////////////////////////////////////////////////////////////////////////////////////////
DCString::DCString(PCTSTR theStr)
{
int theLen     = SafeStrlen(theStr);

    Init();

    if((theStr) && (theLen > 0)){
       if(AllocBuffer(theLen)){
          memcpy(m_StringData->Content, theStr, theLen * sizeof(TCHAR));
       }
    }
}
///////////////////////////////////////////////////////////////////////////////////////////////////
DCString::DCString(PCTSTR theStr, int theLen)
{
    Init();
    if((theStr) && (theLen > 0)){
       if(AllocBuffer(theLen)){
          memcpy(m_StringData->Content, theStr, theLen * sizeof(TCHAR));
       }
    }
}
///////////////////////////////////////////////////////////////////////////////////////////////////
DCString::DCString(TCHAR theChar, int theLen)
{
    Init();
    if(theLen > 0){
       if(AllocBuffer(theLen)){
          memset(m_StringData->Content, theChar, (theLen * sizeof(TCHAR)));
       }
    }
}
///////////////////////////////////////////////////////////////////////////////////////////////////
DCString::DCString(const DCString& StringSrc)
{
    if((StringSrc.m_StringData->Refs     > 0)
    ){
       m_StringData    = StringSrc.m_StringData;
       m_pchData       = m_StringData->Content;
       InterlockedIncrement(&m_StringData->Refs);
    } else {
       Init();
       *this = StringSrc;
    }
}
///////////////////////////////////////////////////////////////////////////////////////////////////
//
///////////////////////////////////////////////////////////////////////////////////////////////////
PDCStringData DCString::DetachData()
{
PDCStringData OldData = m_StringData;

    Init();
    return OldData;
}
///////////////////////////////////////////////////////////////////////////////////////////////////
//
///////////////////////////////////////////////////////////////////////////////////////////////////
int  DCString::GetLength() const                 { return m_StringData->DataLength; }
int  DCString::GetAllocLength() const            { return m_StringData->AllocLength; }
BOOL DCString::IsEmpty() const                   { return m_StringData->DataLength == 0; }

int  DCString::SafeStrlen(PCTSTR theStr)         { return (theStr == NULL) ? 0 : TStrLen(theStr); }
///////////////////////////////////////////////////////////////////////////////////////////////////
BOOL DCString::AllocBuffer(int theStrLen) // ALWAYS ALLOCATE ONE EXTRA CHARACTER FOR '\0' TERMINATION
{
int theMAllocLen;

    if(theStrLen == 0){
       Init();
    } else {
       theMAllocLen                          = sizeof(DCStringData) + ((theStrLen + STRTERMINATOR) * sizeof(TCHAR));
       m_StringData                          = (DCStringData*) new BYTE [ theMAllocLen ];
       if(m_StringData                      != NULL){
          // ZeroMemory(m_StringData, theMAllocLen);
          m_StringData->AllocLength          = theStrLen;
          m_StringData->Refs                 = 1;
          m_StringData->Content[theStrLen]   = '\0';
          m_StringData->DataLength           = theStrLen;
          m_pchData                          = m_StringData->Content;
       } else {
          return FALSE;
       }
    }
    return TRUE;
}
///////////////////////////////////////////////////////////////////////////////////////////////////
void DCString::FreeData(PDCStringData StringData)
{
    if((StringData)
    && (StringData != _DCNullStringData)
    ){
       delete[] (BYTE*)StringData;
    }
    StringData     = (DCStringData*)_DCNullStringData;
}
///////////////////////////////////////////////////////////////////////////////////////////////////
void DCString::Release()
{
    if(m_StringData != _DCNullStringData){
       if(InterlockedDecrement(&m_StringData->Refs) <= 0){
          FreeData(m_StringData);
       }
       Init();
    }
}
///////////////////////////////////////////////////////////////////////////////////////////////////
void DCString::Release(PDCStringData StringData)
{
    if((StringData)
    && (StringData != _DCNullStringData)
    ){
       if(InterlockedDecrement(&StringData->Refs) <= 0)
          FreeData(StringData);
    }
}
///////////////////////////////////////////////////////////////////////////////////////////////////
void DCString::Empty()
{
    if((m_StringData             == NULL)
    || (m_StringData->DataLength == 0)
    ){
       return;
    }
    Release();
}
///////////////////////////////////////////////////////////////////////////////////////////////////
BOOL DCString::CopyBeforeWrite()
{
BOOL bRet = FALSE;

    if((m_StringData)
    &&(m_StringData->Refs    > 1)
    ){
       DCStringData* OldData = DetachData();
       if(AllocBuffer(OldData->DataLength)){
          memcpy(m_StringData->Content, OldData->Content, (OldData->DataLength + 1) * sizeof(TCHAR));
          bRet = TRUE;
       }
       Release();
       return bRet;
    }
    return TRUE;
}
///////////////////////////////////////////////////////////////////////////////////////////////////
BOOL DCString::AllocBeforeWrite(int theLen)
{
    if((m_StringData->Refs <= 1)
    || (theLen              > m_StringData->AllocLength)
    ){
       Release();
       return AllocBuffer(theLen);
    }
    return TRUE;
}
///////////////////////////////////////////////////////////////////////////////////////////////////
// ADVANCED DIRECT BUFFER ACCESS
///////////////////////////////////////////////////////////////////////////////////////////////////
PTSTR DCString::GetBuffer(int MinBufLength)
{
PTSTR OutBuffer = m_StringData->Content;

    if(MinBufLength > m_StringData->AllocLength)
    {
        // WE HAVE TO GROW THE BUFFER
        int           OldLen              = m_StringData->DataLength;   // Alloc Buffer will tromp it
        DCStringData* OldData             = DetachData();
        if(MinBufLength                   < OldLen){
           MinBufLength                   = OldLen;
        }
        if(AllocBuffer(MinBufLength)){
           memcpy(m_StringData->Content, OldData->Content, (OldLen + 1) * sizeof(TCHAR));
           m_StringData->DataLength    = OldLen;
           OutBuffer                   = m_StringData->Content;
        } else {
           OutBuffer                   = NULL;
        }
        DCString::Release(OldData);
    }
    return OutBuffer;
}
///////////////////////////////////////////////////////////////////////////////////////////////////
PTSTR DCString::GetBufferSetLength (int NewLength)
{
    if(NewLength){
       if(GetBuffer(NewLength)){
          m_StringData->DataLength         = NewLength;
          m_StringData->Content[NewLength] = '\0';
       } else {
          return NULL;
       }
    }
    return m_StringData->Content;
}
///////////////////////////////////////////////////////////////////////////////////////////////////
void DCString::ReleaseBuffer(int NewLength)
{
    if(CopyBeforeWrite() == FALSE){  // JUST IN CASE GET BUFFER WAS NOT CALLED
       return;
    }
    if(NewLength   == -1){
       NewLength    = SafeStrlen(m_StringData->Content); // ZERO TERMINATED
    }
    if(NewLength    > 0){
       m_StringData->DataLength         = NewLength;
       m_StringData->Content[NewLength] = '\0';
    }
}
///////////////////////////////////////////////////////////////////////////////////////////////////
void DCString::FreeExtra()
{
    if(m_StringData->DataLength   != m_StringData->AllocLength){
       DCStringData* OldData       = DetachData();
       if(AllocBuffer(m_StringData->DataLength)){
          memcpy(m_StringData->Content, OldData->Content, (OldData->DataLength) * sizeof(TCHAR));
       }
       DCString::Release(OldData);
    }
}


///////////////////////////////////////////////////////////////////////////////////////////////////
//
///////////////////////////////////////////////////////////////////////////////////////////////////
void DCString::AllocCopy(DCString& DestStr, int CopyLen, int CopyIndex, int ExtraLen) const
{
int NewLen     = CopyLen + ExtraLen;
    // will clone the data attached to this string allocating 'nExtraLen' characters
    // Places results in uninitialized string 'DestStr'
    // Will copy the part or all of original data to start of new string

    if(NewLen  == 0){
       DestStr.Init();
    } else {
       if(DestStr.AllocBuffer(NewLen)){
          memcpy(DestStr.m_StringData->Content, m_StringData->Content + CopyIndex, CopyLen * sizeof(TCHAR));
       }
    }
}
///////////////////////////////////////////////////////////////////////////////////////////////////
// Assignment operators
//  All assign a new value to the string
//      (a) first see if the buffer is big enough
//      (b) if enough room, copy on top of old buffer, set size and type
//      (c) otherwise free old string data, and create a new one
//
//  All routines return the new string (but as a 'const DCString&' so that
//      assigning it again will cause a copy, eg: s1 = s2 = "hi there".
//
///////////////////////////////////////////////////////////////////////////////////////////////////
void DCString::AssignCopy(int SrcLen, PCTSTR theSrcData)
{
    if((theSrcData) && (SrcLen) && (AllocBeforeWrite(SrcLen))
    ){
       memcpy(m_StringData->Content,    theSrcData, SrcLen * sizeof(TCHAR));
       m_StringData->DataLength       = SrcLen;
       m_StringData->Content[SrcLen]  = '\0';
    } else {
       Release();
    }
}
///////////////////////////////////////////////////////////////////////////////////////////////////
void DCString::AssignCopy(const DCString& StringSrc)
{
    if(m_StringData->Content                != StringSrc.m_StringData->Content){
        if((m_StringData->Refs               < 0  && (m_StringData != _DCNullStringData))
        || (StringSrc.m_StringData->Refs     < 0)
        ){ // ACTUAL COPY NECESSARY SINCE ONE OF THE STRINGS IS LOCKED
           AssignCopy(StringSrc.m_StringData->DataLength, StringSrc.m_StringData->Content);
        } else {// CAN JUST COPY REFERENCES AROUND
           Release();
           m_StringData    = StringSrc.m_StringData;
           m_pchData       = m_StringData->Content;
           InterlockedIncrement(&m_StringData->Refs);
        }
    }
}
#ifndef __LINUX__
///////////////////////////////////////////////////////////////////////////////////////////////////
void DCString::AssignCopy(RESID           ParmID)
{
    LoadString(ParmID);
}
#endif
///////////////////////////////////////////////////////////////////////////////////////////////////
// -- master concatenation routine Concatenate two sources
// NOTE: "operator+" is done as friend functions for simplicity There are three variants:
//      DCString = DCString + DCString
// and for ?     = DCString + ?
//                 ?        + DCString
///////////////////////////////////////////////////////////////////////////////////////////////////
void DCString::ConcatCopy(int Src1Len, PCTSTR Src1Data, int Src2Len, PCTSTR Src2Data)
{   // -- assume that 'this' is a new DCString object
int NewLen    = Src1Len + Src2Len;

    if((Src1Data) && (Src2Data) ){
       if(NewLen      > 0){
          if(AllocBuffer(NewLen)){
             memcpy(m_StringData->Content,  Src1Data, Src1Len * sizeof(TCHAR));
             memcpy(m_StringData->Content + Src1Len,  Src2Data, Src2Len * sizeof(TCHAR));
          }
       }
    }
}
///////////////////////////////////////////////////////////////////////////////////////////////////
// CONCATENATE IN PLACE //  -- THE MAIN ROUTINE FOR += OPERATORS
///////////////////////////////////////////////////////////////////////////////////////////////////
void DCString::ConcatInPlace(int SrcLen, PCTSTR SrcData)
{
    if((SrcData  == NULL)
    || (SrcLen   == 0)
    ){ // concatenating an empty string is a no-op!
       return;
    }
    // IF THE BUFFER IS TOO SMALL, OR WE HAVE A WIDTH MIS-MATCH, JUST ALLOCATE A NEW BUFFER (SLOW BUT SURE)
    if(m_StringData->Refs > 1 || m_StringData->DataLength + SrcLen >= m_StringData->AllocLength)
    {
       // WE HAVE TO GROW THE BUFFER, USE THE CONCATCOPY ROUTINE
       DCStringData* OldData   = DetachData();
       ConcatCopy(OldData->DataLength, OldData->Content, SrcLen, SrcData);
       DCString::Release(OldData);
    } else {
       // FAST CONCATENATION WHEN BUFFER BIG ENOUGH
       memcpy(m_StringData->Content  + m_StringData->DataLength, SrcData, SrcLen * sizeof(TCHAR));
       m_StringData->DataLength     += SrcLen;
       m_StringData->Content[m_StringData->DataLength] = '\0';
    }
}

///////////////////////////////////////////////////////////////////////////////////////////////////
// THE OPERATORS Assignment operators
///////////////////////////////////////////////////////////////////////////////////////////////////
const DCString& DCString::operator =(const DCString& StringSrc)
{
    AssignCopy(StringSrc);
    return *this;
}
///////////////////////////////////////////////////////////////////////////////////////////////////
const DCString& DCString::operator =(PCTSTR theStr)
{
    AssignCopy(SafeStrlen(theStr), theStr);
    return *this;
}
///////////////////////////////////////////////////////////////////////////////////////////////////
const DCString& DCString::operator =(TCHAR theChar)
{
    AssignCopy(1, &theChar);
    return *this;
}
#ifndef __LINUX__
///////////////////////////////////////////////////////////////////////////////////////////////////
const DCString& DCString::operator =(RESID           ParmID)
{
    AssignCopy(ParmID);
    return *this;
}
#endif

///////////////////////////////////////////////////////////////////////////////////////////////////
//
///////////////////////////////////////////////////////////////////////////////////////////////////
const DCString& DCString::operator+=(PCTSTR theStr)
{
    ConcatInPlace(SafeStrlen(theStr), theStr);
    return *this;
}
const DCString& DCString::operator+=(TCHAR theChar)
{
    ConcatInPlace(1, &theChar);
    return *this;
}
const DCString& DCString::operator+=(const DCString& StringSrc)
{
    ConcatInPlace(StringSrc.m_StringData->DataLength, StringSrc.m_StringData->Content);
    return *this;
}
#ifndef __LINUX__
///////////////////////////////////////////////////////////////////////////////////////////////////
const DCString& DCString::operator+=(RESID   ParmID)
{
DCString s;

    s.LoadString(ParmID);
    ConcatInPlace(s.m_StringData->DataLength, s.m_StringData->Content);
    return *this;
}
#endif
///////////////////////////////////////////////////////////////////////////////////////////////////
DCString  operator +(const DCString& string1, TCHAR theChar)
{
    DCString s;
    s.ConcatCopy(string1.m_StringData->DataLength, string1.m_StringData->Content, 1, &theChar);
    return s;
}
DCString  operator +(TCHAR theChar, const DCString& StringSrc)
{
    DCString s;
    s.ConcatCopy(1, &theChar, StringSrc.m_StringData->DataLength, StringSrc.m_StringData->Content);
    return s;
}
///////////////////////////////////////////////////////////////////////////////////////////////////
DCString  operator +(const DCString& string, PCTSTR theStr)
{
    DCString s;
    s.ConcatCopy(string.m_StringData->DataLength, string.m_StringData->Content, DCString::SafeStrlen(theStr), theStr);
    return s;
}
DCString  operator +(PCTSTR theStr, const DCString& string)
{
    DCString s;
    s.ConcatCopy(DCString::SafeStrlen(theStr), theStr, string.m_StringData->DataLength, string.m_StringData->Content);
    return s;
}
///////////////////////////////////////////////////////////////////////////////////////////////////
DCString  operator +(const DCString& string1, const DCString& string2)
{
DCString s;

    s.ConcatCopy(string1.m_StringData->DataLength, string1.m_StringData->Content, string2.m_StringData->DataLength, string2.m_StringData->Content);
    return s;
}

///////////////////////////////////////////////////////////////////////////////////////////////////
// COMMONLY USED ROUTINES
///////////////////////////////////////////////////////////////////////////////////////////////////
void DCString::SetAt(int Index, TCHAR theChar)
{
    if(CopyBeforeWrite()){
       m_StringData->Content[Index] = theChar;
    }
}
TCHAR DCString::GetAt(int Index) const
{
    return m_StringData->Content[Index];
}
TCHAR DCString::operator[](int Index) const // SAME AS GETAT
{
    return m_StringData->Content[Index];
}

///////////////////////////////////////////////////////////////////////////////////////////////////
// COMMONLY USED ROUTINES
///////////////////////////////////////////////////////////////////////////////////////////////////
void DCString::MakeUpper()
{
    if(CopyBeforeWrite()){
       TStrToUpper(m_pchData);
    }
}
void DCString::MakeLower()
{
    if(CopyBeforeWrite()){
       TStrToLower(m_pchData);
    }
}
void DCString::MakeReverse()
{
    if(CopyBeforeWrite()){
       TStrReverse(m_pchData);
    }
}

///////////////////////////////////////////////////////////////////////////////////////////////////
//
///////////////////////////////////////////////////////////////////////////////////////////////////
int DCString::Find(TCHAR theChar) const
{
    return Find(theChar, 0);
}
int DCString::Find(TCHAR theChar, int Start) const  // FIND FIRST SINGLE CHARACTER
{
    int theLen      = m_StringData->DataLength;
    if(Start       >= theLen){
       return -1;
    }
    PTSTR theStr    = TStrChr(m_StringData->Content + Start, (BYTE)theChar);

    // return -1 if not found and index otherwise
    return (theStr == NULL) ? -1 : (int)(theStr - m_StringData->Content);
}
int DCString::FindOneOf(PCTSTR CharSetStr) const
{
    if(CharSetStr){
       PTSTR theStr    = TStrSrchChrSet (m_StringData->Content, CharSetStr);
       return (theStr == NULL) ? -1 : (int)(theStr - m_StringData->Content);
    }
    return -1;
}
///////////////////////////////////////////////////////////////////////////////////////////////////
// FINDING
///////////////////////////////////////////////////////////////////////////////////////////////////
int DCString::ReverseFind(TCHAR theChar) const // FIND LAST SINGLE CHARACTER
{
    PTSTR theStr    = TStrSrchLastChar (m_StringData->Content, (BYTE) theChar);

    // return -1 if not found, distance from beginning otherwise
    return (theStr == NULL) ? -1 : (int)(theStr - m_StringData->Content);
}
///////////////////////////////////////////////////////////////////////////////////////////////////
// FIND A SUB-STRING (LIKE STRSTR)
///////////////////////////////////////////////////////////////////////////////////////////////////
int DCString::Find(PCTSTR StrSub) const
{
    return Find(StrSub, 0);
}
///////////////////////////////////////////////////////////////////////////////////////////////////
int DCString::Find(PCTSTR StrSub, int Start) const
{
int theLen       = m_StringData->DataLength;

    if((StrSub  == NULL)   // CANNOT BE FOUND!
    || (Start    > theLen)
    ){
       return -1;
    }
    // FIND FIRST MATCHING SUBSTRING
    PTSTR theStr  = TStrSrchSubStr (m_StringData->Content + Start, StrSub);

    // return -1 for not found, distance from beginning otherwise
    return (theStr == NULL) ? -1 : (int)(theStr - m_StringData->Content);
}
///////////////////////////////////////////////////////////////////////////////////////////////////
// DCString support (windows specific)    // MBCS/Unicode aware
///////////////////////////////////////////////////////////////////////////////////////////////////
int DCString::Compare(PCTSTR theStr) const
{
    if(theStr){
       return TStrCmp(m_StringData->Content, theStr);
    }
    return -1;
}
///////////////////////////////////////////////////////////////////////////////////////////////////
int DCString::CompareNoCase(PCTSTR theStr) const
{
    if(theStr){
       return TStrICmp(m_StringData->Content, theStr);
    }
    return -1;
}
///////////////////////////////////////////////////////////////////////////////////////////////////
// Collate is often slower than Compare but is MBSC/Unicode  aware as well as locale-sensitive with respect to sort order.
///////////////////////////////////////////////////////////////////////////////////////////////////
int DCString::Collate(PCTSTR theStr) const           // locale sensitive
{
    if(theStr){
       return TStrColl(m_StringData->Content, theStr);
    }
    return -1;
}
///////////////////////////////////////////////////////////////////////////////////////////////////
int DCString::CollateNoCase(PCTSTR theStr) const     // locale sensitive
{
    if(theStr){
       return TStrColl(m_StringData->Content, theStr);
    }
    return -1;
}
///////////////////////////////////////////////////////////////////////////////////////////////////
// STRSPN EQUIVALENT
///////////////////////////////////////////////////////////////////////////////////////////////////
DCString DCString::SpanIncluding(PCTSTR CharSetStr) const
{
    if(CharSetStr){
       return Left(TStrSpan(m_StringData->Content, CharSetStr));
    }
    return Left(0);
}
///////////////////////////////////////////////////////////////////////////////////////////////////
DCString DCString::SpanExcluding(PCTSTR CharSetStr) const
{
    if(CharSetStr){
       return Left(TStrCSpan(m_StringData->Content, CharSetStr));
    }
    return Left(0);
}



///////////////////////////////////////////////////////////////////////////////////////////////////
//
///////////////////////////////////////////////////////////////////////////////////////////////////
int DCString::Delete(int Start, int CharCount /* = 1 */)
{
    if(Start                 < 0){
       Start                 = 0;
    }
    int NewLength            = m_StringData->DataLength;
    if(CharCount             > 0  && Start < NewLength){
       if(CopyBeforeWrite()){
          int BytesToCopy    = NewLength - (Start + CharCount) + 1;

          memcpy(m_StringData->Content + Start, m_StringData->Content + Start + CharCount, BytesToCopy * sizeof(TCHAR));
          m_StringData->DataLength = NewLength - CharCount;
       }
    }
    return NewLength;
}
///////////////////////////////////////////////////////////////////////////////////////////////////
int DCString::Insert(int Start, TCHAR theChar)
{
BOOL bRet          = FALSE;

    if(Start       < 0){
       Start       = 0;
    }
    int NewLength  = m_StringData->DataLength;
    if(Start       > NewLength){
       Start       = NewLength;
    }
    NewLength++;

    if(CopyBeforeWrite()){
       if(m_StringData->AllocLength  < NewLength + STRTERMINATOR){
          DCStringData* OldData      = DetachData();
          PTSTR         StrSub       = m_StringData->Content;
          if(AllocBuffer(NewLength)){
             memcpy(m_StringData->Content, StrSub, (OldData->DataLength + 1) * sizeof(TCHAR));
             bRet     = TRUE;
          }
          Release(OldData);
       } else {
          bRet       = TRUE;
       }
    }
    if(bRet         == FALSE){
       return 0;
    }
    // MAKING ROOM FOR THE CHAR
    memcpy(m_StringData->Content + Start + 1, m_StringData->Content + Start, (NewLength - Start) * sizeof(TCHAR));
    m_StringData->Content[Start] = theChar;
    m_StringData->DataLength     = NewLength;

    return NewLength;
}
///////////////////////////////////////////////////////////////////////////////////////////////////
int DCString::Insert(int Start, PCTSTR StrSub)
{
BOOL bRet            = FALSE;

    if((StrSub      == NULL)
    ){
        return 0;
    }
    if(Start         < 0){
       Start         = 0;
    }
    int InsertLength = SafeStrlen(StrSub);
    int NewLength    = m_StringData->DataLength;
    if(Start         > NewLength){
       Start         = NewLength;
    }
    NewLength       += InsertLength;

    if((InsertLength > 0)){
       if(CopyBeforeWrite()){

          if(m_StringData->AllocLength  < NewLength  + STRTERMINATOR){
             DCStringData* OldData      = DetachData();
             PTSTR         StrTmp       = m_StringData->Content;
             if(AllocBuffer(NewLength)){
                memcpy(m_StringData->Content, StrTmp, (OldData->DataLength + 1) * sizeof(TCHAR));
                bRet     = TRUE;
             }
             Release(OldData);
          } else {
             bRet        = TRUE;
          }
       }
       if(bRet          == FALSE){
          return 0;
       }
       // MAKING ROOM FOR THE STRING
       memcpy(m_StringData->Content + Start + InsertLength, m_StringData->Content + Start, (NewLength - Start - InsertLength + 1) * sizeof(TCHAR));
       memcpy(m_StringData->Content + Start,  StrSub, InsertLength * sizeof(TCHAR));
       m_StringData->DataLength = NewLength;
    }
    return NewLength;
}
///////////////////////////////////////////////////////////////////////////////////////////////////
int DCString::Replace(TCHAR chOld, TCHAR chNew)
{
int CharCount           = 0;

    if(chOld              != chNew){   // SHORT-CIRCUIT THE NOP CASE

       if(CopyBeforeWrite()){       // OTHERWISE MODIFY EACH CHARACTER THAT MATCHES IN THE STRING
          PTSTR StrSub     = m_StringData->Content;
          PTSTR StrEnd     = StrSub + m_StringData->DataLength;
          while(StrSub     < StrEnd){
             if(*StrSub   == chOld){   // REPLACE INSTANCES OF THE SPECIFIED CHARACTER ONLY
                *StrSub    = chNew;
                CharCount++;
             }
             StrSub        = _XNextCharPtr(StrSub);
          }
       }
    }
    return CharCount;
}
///////////////////////////////////////////////////////////////////////////////////////////////////
int DCString::Replace(PCTSTR StrOld, PCTSTR StrNew)
{
BOOL bRet                   = FALSE;
int  SourceLen              = SafeStrlen(StrOld);

    if((StrNew             == NULL)
    || (SourceLen          == 0)
    ){
        return 0;
    }
    int CharCount           = 0;
    int ReplacementLen      = SafeStrlen(StrNew);

    // LOOP ONCE TO FIGURE OUT THE SIZE OF THE RESULT STRING
    PTSTR StrStart          = m_StringData->Content;
    PTSTR StrEnd            = m_StringData->Content + m_StringData->DataLength;
    PTSTR StrTarget;
    while(StrStart          < StrEnd){
       while((StrTarget     = TStrSrchSubStr (StrStart, StrOld)) != NULL){
          CharCount++;
          StrStart          = StrTarget + SourceLen;
       }
       StrStart            += SafeStrlen(StrStart) + 1;
    }
    if(CharCount            > 0){// IF ANY CHANGES WERE MADE, MAKE THEM
       if(CopyBeforeWrite()){

          // IF THE BUFFER IS TOO SMALL, JUST ALLOCATE A NEW BUFFER (SLOW BUT SURE)
          int OldLength     = m_StringData->DataLength;
          int NewLength     = OldLength    + (ReplacementLen - SourceLen) * CharCount;
          if((m_StringData->AllocLength    < NewLength)
          || (m_StringData->Refs           > 1)
          ){
             DCStringData* OldData     = DetachData();
             PTSTR         StrSub      = m_StringData->Content;
             if(AllocBuffer(NewLength)){
                memcpy(m_StringData->Content, StrSub, OldData->DataLength * sizeof(TCHAR));
                bRet                   = TRUE;
             }
             Release(OldData);
          } else {
             bRet                      = TRUE;
          }
          if(bRet){
          // ELSE, WE JUST DO IT IN-PLACE
          StrStart                     = m_StringData->Content;
          StrEnd                       = m_StringData->Content + m_StringData->DataLength;

             // LOOP AGAIN TO ACTUALLY DO THE WORK
             while(StrStart            < StrEnd){
                while((StrTarget       = TStrSrchSubStr (StrStart, StrOld)) != NULL){
                   int Balance         = OldLength - (StrTarget - m_StringData->Content + SourceLen);
                   memmove(StrTarget   + ReplacementLen, StrTarget + SourceLen, Balance * sizeof(TCHAR));
                   memcpy(StrTarget,     StrNew, ReplacementLen * sizeof(TCHAR));
                   StrStart            = StrTarget +  ReplacementLen;
                   StrStart[Balance]   = '\0';
                   OldLength          += (ReplacementLen - SourceLen);
                }
                StrStart              += SafeStrlen(StrStart) + 1;
             }
             m_StringData->DataLength  = NewLength;
          } else {
             CharCount                 = 0;
          }
       } else {
          CharCount                    = 0;
       }
    }
    return CharCount;
}
///////////////////////////////////////////////////////////////////////////////////////////////////
int DCString::Remove(TCHAR chRemove)
{
int CharCount                   = 0;

    if(CopyBeforeWrite()){

       PTSTR StrSource          = m_StringData->Content;
       PTSTR StrDest            = m_StringData->Content;
       PTSTR StrEnd             = m_StringData->Content + m_StringData->DataLength;

       while(StrSource           < StrEnd){
          if(*StrSource         != chRemove){
             *StrDest            = *StrSource;
             StrDest             = _XNextCharPtr(StrDest);
          }
          StrSource              = _XNextCharPtr(StrSource);
       }
       *StrDest                  = '\0';
       CharCount                 = StrSource - StrDest;
       m_StringData->DataLength -= CharCount;
    }
    return CharCount;
}
///////////////////////////////////////////////////////////////////////////////////////////////////
// SUB-STRING EXTRACTION
///////////////////////////////////////////////////////////////////////////////////////////////////
DCString DCString::Mid(int First) const
{
    return Mid(First, m_StringData->DataLength - First);
}
DCString DCString::Mid(int First, int CharCount) const
{
    // OUT-OF-BOUNDS REQUESTS RETURN SENSIBLE THINGS
    if(First         < 0){
       First         = 0;
    }
    if(CharCount     < 0){
       CharCount     = 0;
    }
    if(First         + CharCount > m_StringData->DataLength){
       CharCount     = m_StringData->DataLength - First;
    }
    if(First         > m_StringData->DataLength){
       CharCount     = 0;
    }
    // OPTIMIZE CASE OF RETURNING ENTIRE STRING
    if(First        == 0 && (First + CharCount == m_StringData->DataLength) ){
       return *this;
    }
    DCString DestStr;
    AllocCopy(DestStr, CharCount, First, 0);
    return DestStr;
}
///////////////////////////////////////////////////////////////////////////////////////////////////
DCString DCString::Right(int CharCount) const
{
    if(CharCount     < 0){
       CharCount     = 0;
    }
    if(CharCount    >= m_StringData->DataLength){
       return *this;
    }
    DCString DestStr;
    AllocCopy(DestStr, CharCount, m_StringData->DataLength-CharCount, 0);
    return DestStr;
}
///////////////////////////////////////////////////////////////////////////////////////////////////
DCString DCString::Left(int CharCount) const
{
    if(CharCount     < 0){
       CharCount     = 0;
    }
    if(CharCount    >= m_StringData->DataLength){
       return *this;
    }
    DCString DestStr;
    AllocCopy(DestStr, CharCount, 0, 0);
    return DestStr;
}


///////////////////////////////////////////////////////////////////////////////////////////////////
// FIND BEGINNING OF TRAILING MATCHES BY STARTING AT BEGINNING (DBCS AWARE)
///////////////////////////////////////////////////////////////////////////////////////////////////
void DCString::TrimRight(TCHAR chTarget)
{
    if(CopyBeforeWrite()){
       PTSTR theStr       = m_StringData->Content;
       PTSTR StrLast      = NULL;

       while(*theStr     != '\0'){
           if(*theStr    == chTarget){
              if(StrLast == NULL){
                 StrLast  = theStr;
              }
           } else {
              StrLast     = NULL;
           }
           theStr         = _XNextCharPtr(theStr);
       }
       if(StrLast        != NULL){       // TRUNCATE AT LEFT-MOST MATCHING CHARACTER
          *StrLast        = '\0';
          m_StringData->DataLength = StrLast - m_StringData->Content;
       }
    }
}
///////////////////////////////////////////////////////////////////////////////////////////////////
void DCString::TrimRight(PCTSTR StrTargetList)
{
    if(SafeStrlen(StrTargetList) == 0){
        return;
    }
    if(CopyBeforeWrite()){
       PTSTR theStr       = m_StringData->Content;
       PTSTR StrLast      = NULL;

       while(*theStr     != '\0'){
           if(TStrChr(StrTargetList, *theStr) != NULL){
              if(StrLast == NULL){
                 StrLast  = theStr;
              }
           } else {
              StrLast     = NULL;
           }
           theStr = _XNextCharPtr(theStr);
       }
       if(StrLast != NULL){       // TRUNCATE AT LEFT-MOST MATCHING CHARACTER
          *StrLast = '\0';
          m_StringData->DataLength = StrLast - m_StringData->Content;
       }
    }
}
///////////////////////////////////////////////////////////////////////////////////////////////////
void DCString::TrimRight()
{
    if(CopyBeforeWrite()){
       PTSTR theStr       = m_StringData->Content;
       PTSTR StrLast      = NULL;

       while(*theStr     != '\0'){
           if(isspace(*theStr)){
              if(StrLast == NULL){
                 StrLast  = theStr;
              }
           } else {
              StrLast     = NULL;
           }
           theStr         = _XNextCharPtr(theStr);
       }
       if(StrLast        != NULL){
          *StrLast        = '\0';  // TRUNCATE AT TRAILING SPACE START
          m_StringData->DataLength  = StrLast - m_StringData->Content;
       }
    }
}
///////////////////////////////////////////////////////////////////////////////////////////////////
//
///////////////////////////////////////////////////////////////////////////////////////////////////
void DCString::TrimLeft(TCHAR chTarget)    // FIND FIRST NON-MATCHING CHARACTER
{
    if(CopyBeforeWrite()){
       PCTSTR theStr      = m_StringData->Content;

       while(chTarget    == *theStr){
           theStr         = _XNextCharPtr(theStr);
       }
       if(theStr         != m_StringData->Content){ // FIX UP DATA AND LENGTH
          int DataLength  = m_StringData->DataLength - (theStr - m_StringData->Content);
          memmove(m_StringData->Content, theStr, (DataLength + 1) * sizeof(TCHAR));
          m_StringData->DataLength = DataLength;
       }
    }
}
///////////////////////////////////////////////////////////////////////////////////////////////////
void DCString::TrimLeft(PCTSTR StrTargets)// IF WE'RE NOT TRIMMING ANYTHING, WE'RE NOT DOING ANY WORK
{
    if(SafeStrlen(StrTargets) == 0){
        return;
    }
    if(CopyBeforeWrite()){
       PCTSTR theStr      = m_StringData->Content;

       while(*theStr     != '\0'){
           if(TStrChr(StrTargets, *theStr) == NULL){
              break;
           }
           theStr         = _XNextCharPtr(theStr);
       }
       if(theStr         != m_StringData->Content){ // FIX UP DATA AND LENGTH
          int DataLength  = m_StringData->DataLength - (theStr - m_StringData->Content);
          memmove(m_StringData->Content, theStr, (DataLength + 1) * sizeof(TCHAR));
          m_StringData->DataLength = DataLength;
       }
    }
}
///////////////////////////////////////////////////////////////////////////////////////////////////
void DCString::TrimLeft()    // FIND FIRST NON-SPACE CHARACTER
{
    if(CopyBeforeWrite()){
       PCTSTR theStr          = m_StringData->Content;

       while(isspace(*theStr)){
           theStr             = _XNextCharPtr(theStr);
       }
       if(theStr             != m_StringData->Content){ // FIX UP DATA AND LENGTH
          int DataLength      = m_StringData->DataLength - (theStr - m_StringData->Content);
          memmove(m_StringData->Content, theStr, (DataLength + 1) * sizeof(TCHAR));
          m_StringData->DataLength = DataLength;
       }
    }
}



///////////////////////////////////////////////////////////////////////////////////////////////////
// DCSTRING FORMATTING
///////////////////////////////////////////////////////////////////////////////////////////////////
void DCString::FormatV(PCTSTR StrFormat, va_list argList)
{
ASSERT(AfxIsValidString(StrFormat));
int MaxLen        = 0;

    va_list argListSave = argList;

    // MAKE A GUESS AT THE MAXIMUM LENGTH OF THE RESULTING STRING
    for(PCTSTR theStr = StrFormat; *theStr != '\0'; theStr = _XNextCharPtr(theStr)){
        // HANDLE '%' CHARACTER, BUT WATCH OUT FOR '%%'
        if(*theStr    != '%' || *(theStr    = _XNextCharPtr(theStr)) == '%'){
           MaxLen     += 1;  // _tclen(theStr);
           continue;
        }
        int ItemLen    = 0;

        // HANDLE '%' CHARACTER WITH FORMAT
        int Width              = 0;
        for(; *theStr         != '\0'; theStr = _XNextCharPtr(theStr)){

            if(*theStr        == '#'){    // check for valid flags
               MaxLen         += 2;       // for '0x'
            } else if(*theStr == '*'){
               Width           = va_arg(argList, int);
            } else if(*theStr == '-' || *theStr == '+' || *theStr == '0' ||
               *theStr        == ' '){ NULL;
            } else { // HIT NON-FLAG CHARACTER
               break;
            }
        }
        // GET WIDTH AND SKIP IT
        if(Width == 0){
           Width  = AToI(theStr);   // WIDTH INDICATED BY
           for(; *theStr != '\0' && isdigit(*theStr); theStr = _XNextCharPtr(theStr)){
               ;
           }
        }
        int Precision   = 0;
        if(*theStr     == '.'){  // SKIP PAST '.' SEPARATOR (WIDTH.PRECISION)
           theStr       = _XNextCharPtr(theStr);

           if(*theStr   == '*'){           // GET PRECISION AND SKIP IT
              Precision  = va_arg(argList, int);
              theStr     = _XNextCharPtr(theStr);
           } else {
              Precision  = _ttoi(theStr);
              for(; *theStr != '\0' && isdigit(*theStr); theStr = _XNextCharPtr(theStr)){
                 ;
              }
           }
        }
        // SHOULD BE ON TYPE MODIFIER OR SPECIFIER
        int Modifier    = 0;

        switch (*theStr | Modifier){
        case 'C': case 'c':// SINGLE CHARACTERS
#ifndef __LINUX__
             ItemLen = 2; va_arg(argList, TCHAR);  break;
#else 
             ItemLen = 2; va_arg(argList, int);    break;
#endif
        case 'S': case 's':
             {
                 PCTSTR StrNextArg  = va_arg(argList, PCTSTR);
                 if(StrNextArg     == NULL){
                    ItemLen         = 8;  // "(null)"
                 } else {
                    ItemLen         = SafeStrlen(StrNextArg);
                    ItemLen         = max(4, ItemLen);
                 }
             }
             break;
        }
        if(ItemLen       != 0){          // ADJUST ITEMLEN FOR STRINGS
            if(Precision != 0) ItemLen    = min(ItemLen, Precision);
            ItemLen       =    max(ItemLen, Width);
        } else {
            switch (*theStr){
            case 'd': case 'i': case 'u': case 'x': case 'X': case 'o': // INTEGERS
                 va_arg(argList, int); ItemLen = 32;  ItemLen = max(ItemLen, Width + Precision);
                 break;

            case 'e': case 'g': case 'G':
                 va_arg(argList, double); ItemLen = 128;  ItemLen = max(ItemLen, Width + Precision);
                 break;

            case 'f':
                 {
                 double theFloat;
                 PTSTR  StrTemp;

                     // 312 == SafeStrlen("-1+(309 zeroes).") 309 zeroes == max precision of a double
                     //   6 == adjustment in case precision is not specified, which means that the precision defaults to 6
                     StrTemp      = (PTSTR)malloc(max(Width, 312 + Precision + 8));
                     if(StrTemp){
                         theFloat = va_arg(argList, double);
#ifdef __LINUX__
                         TFormatStr(StrTemp,  Width, __T( "%*.*f" ), Precision + 6, theFloat );
#else
                         TFormatStr(StrTemp, __T( "%*.*f" ), Width, Precision + 6, theFloat );
#endif
                         ItemLen   = SafeStrlen(StrTemp) + 6;
                        free(StrTemp);
                     } else {
                        ItemLen   = 256; // SAFETY FIRST...
                     }
                 }
                 break;

            case 'p':
                 va_arg(argList, void*);  ItemLen = 32; ItemLen = max(ItemLen, Width + Precision);
                 break;

            case 'n':   // NO OUTPUT
                 va_arg(argList, int*);
                 break;

            default:
                ASSERT(FALSE);  // UNKNOWN FORMATTING OPTION
            }
        }
        MaxLen += ItemLen;      // ADJUST MAXLEN FOR OUTPUT ITEMLEN
    }
    if(GetBuffer(MaxLen)){
       //VERIFY(_vstprintf(m_StringData->Content, StrFormat, argListSave) <= GetAllocLength());
#ifdef __LINUX__
    	TFormatStrArg(m_StringData->Content,MaxLen,StrFormat, argListSave);
#else
        TFormatStrArg(m_StringData->Content, StrFormat, argListSave);
#endif

       ReleaseBuffer();
    } else {
       va_end(argListSave);
    }
}
///////////////////////////////////////////////////////////////////////////////////////////////////
// FORMATTING (USING WSPRINTF STYLE FORMATTING)
///////////////////////////////////////////////////////////////////////////////////////////////////
void DCString::Format(PCTSTR StrFormat, ...)
{
va_list argList = (va_list)((&StrFormat) + 1);

    va_start(argList, StrFormat );
    FormatV(StrFormat, argList);
}
///////////////////////////////////////////////////////////////////////////////////////////////////
//
///////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef __LINUX__
BOOL DCString::LoadString(RESID ResID)
{
TCHAR     StringBuf         [32*KB]; // DOES NOT SUPPORT STRING LONGER FROM 32 KB
int       theSize         = 0;       // IN THAT SPECIAL CASE PROGRAMMER HAVE TO MAKE IT'S OWN ROUTINE.
int       theLen          = 0;
HRSRC     hResInfo        = 0;

    // LoadString without annoying warning from the Debug kernel if the segment containing the string is not present
    if((hResInfo          = ::FindResource(AfxGetResourceHandle(), MAKEINTRESOURCE((ResID >> 4) + 1), RT_STRING)) == NULL)
    {
        return FALSE;
    }
    theSize              = SizeofResource(AfxGetResourceHandle(), hResInfo);

    if((theSize          > 0)
    ){
       ZeroMemory(StringBuf, sizeof(StringBuf));
       int theLen        = ::LoadString(AfxGetResourceHandle(), ResID, StringBuf, sizeof(StringBuf));
       if(theLen        == 0){
          return 0;
       }
       *this             = StringBuf;
    }
    return (theLen       > 0);
}
#endif //not LINUX
///////////////////////////////////////////////////////////////////////////////////////////////////
// NEED CONVERSION...
///////////////////////////////////////////////////////////////////////////////////////////////////
#ifdef _UNICODE

    DCString::DCString(PCSTR ParmStringPtr)
    {
        Init();
        int theLen   = ParmStringPtr != NULL ? lstrlenA(ParmStringPtr) : 0;
        if(theLen   != 0)
        {
            if(AllocBuffer(theLen)){
               _mbstowcsz(m_pchData, ParmStringPtr, theLen + 1);
               ReleaseBuffer();
            }
        }
    }
    ///////////////////////////////////////////////////////////////////////////////////////////////
    const  DCString& DCString::operator =(PCSTR   ParmStringPtr)
    {
    DCString s(ParmStringPtr);

        AssignCopy(s);
        return *this;
    }
    ///////////////////////////////////////////////////////////////////////////////////////////////
    const  DCString& DCString::operator+=(PCSTR   ParmStringPtr)
    {
    DCString s(ParmStringPtr);

        ConcatInPlace(s.m_StringData->DataLength, s.m_StringData->Content);
        return *this;
    }
    ///////////////////////////////////////////////////////////////////////////////////////////////
    int  DCString::SafeStrlen(PCSTR theStr)          { return (theStr == NULL) ? 0 : strlen(theStr); }

#else

    ///////////////////////////////////////////////////////////////////////////////////////////////
    DCString::DCString(PCWSTR ParmStringPtr)
    {
        Init();
        int theLen   = ParmStringPtr != NULL ? wcslen(ParmStringPtr) : 0;
        if(theLen   != 0)
        {
            if(AllocBuffer(theLen * sizeof(WCHAR))){
               _wcstombsz(m_pchData, ParmStringPtr, (theLen * sizeof(WCHAR)) + 1);
               ReleaseBuffer();
            }
        }
    }
    ///////////////////////////////////////////////////////////////////////////////////////////////
    const  DCString& DCString::operator =(PCWSTR  ParmStringPtr)
    {
    DCString s(ParmStringPtr);

        AssignCopy(s);
        return *this;
    }
    ///////////////////////////////////////////////////////////////////////////////////////////////
    const  DCString& DCString::operator+=(PCWSTR  ParmStringPtr)
    {
    DCString s(ParmStringPtr);

        ConcatInPlace(s.m_StringData->DataLength, s.m_StringData->Content);
        return *this;
    }
    ///////////////////////////////////////////////////////////////////////////////////////////////
    int  DCString::SafeStrlen(PCWSTR theStr)          { return (theStr == NULL) ? 0 : wcslen(theStr); }

#endif //!_UNICODE


///////////////////////////////////////////////////////////////////////////////////////////////////
#endif // #ifndef _MAC_
///////////////////////////////////////////////////////////////////////////////////////////////////



