// DevUnzip.h
////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef _DevUnzip_H
#define _DevUnzip_H
////////////////////////////////////////////////////////////////////////////////////////////////////
#ifdef __cplusplus
extern "C" {
#endif

////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef ATMEL_PLATFORM
#define ATMEL_PLATFORM 0
#endif
////////////////////////////////////////////////////////////////////////////////////////////////////
#define ZLIB_VERSION "1.1.4"
#ifndef _WINDOWS_
   #define STRICT
#endif
///////////////////////////////////////////////////////////////////////////////////////////////////
#include "UnzipLib.h"
///////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////
#if MAX_MEM_LEVEL >= 8
#  define DEF_MEM_LEVEL 8
#else
#  define DEF_MEM_LEVEL  MAX_MEM_LEVEL
#endif
///////////////////////////////////////////////////////////////////////////////////////////////////
#define MAX_WBITS   15 /* 32K LZ77 window */
///////////////////////////////////////////////////////////////////////////////////////////////////
// Maximum value for windowBits in deflateInit2 and inflateInit2.
// WARNING: reducing MAX_WBITS makes minigzip unable to extract .gz files
// created by gzip. (Files created by minigzip can still be extracted by gzip.)
///////////////////////////////////////////////////////////////////////////////////////////////////
/* The memory requirements for deflate are (in bytes):(1 << (windowBits+2)) +  (1 << (memLevel+9))
 that is: 128K for windowBits=15  +  128K for memLevel = 8  (default values) plus a few kilobytes 
 for small objects. For example, if you want to reduce the default memory requirements from 256K 
 to 128K, compile with  make CFLAGS="-O -DMAX_WBITS=14 -DMAX_MEM_LEVEL=7"
 Of course this will generally degrade compression (there's no free lunch).

   The memory requirements for inflate are (in bytes) 1 << windowBits that is, 32K for windowBits=15 
   (default value) plus a few kilobytes for small objects.
*/
///////////////////////////////////////////////////////////////////////////////////////////////////
#define PRESET_DICT 0x20 /* preset dictionary flag in zlib header */

///////////////////////////////////////////////////////////////////////////////////////////////////
// Huffman code lookup table entry--this entry is four bytes for machines
// that have 16-bit pointers (e.g. PC's in the small or medium model). 
///////////////////////////////////////////////////////////////////////////////////////////////////
/* Maximum size of dynamic tree.  The maximum found in a Long but non-
   exhaustive search was 1004 huft structures (850 for length/literals
   and 154 for distances, the latter actually the result of an
   exhaustive search).  The actual maximum is not known, but the
   value below is more than safe. 
*/
///////////////////////////////////////////////////////////////////////////////////////////////////
#define HOWMANY_BRANCH   1440
#define MAX_MEM          2048           // MAX SIZE OF AVAILABLE MEMORY 2K FOR TREE.BITLENS BUFF

///////////////////////////////////////////////////////////////////////////////////////////////////
// LOCAL STRUCTURE 
///////////////////////////////////////////////////////////////////////////////////////////////////
typedef enum {
      BLK_TYPE,              // GET TYPE BITS (3, INCLUDING END BIT)
      BLK_LENS,              // GET LENGTHS FOR STORED
      BLK_STORED,            // PROCESSING STORED BLOCK
      BLK_TABLE,             // GET TABLE LENGTHS
      BLK_BTREE,             // GET BIT LENGTHS TREE FOR A DYNAMIC BLOCK
      BLK_DTREE,             // GET LENGTH, DISTANCE TREES FOR A DYNAMIC BLOCK
      BLK_CODES,             // PROCESSING FIXED OR DYNAMIC BLOCK
      BLK_DRY,               // OUTPUT REMAINING WINDOW BYTES
      BLK_DONE,              // FINISHED LAST BLOCK, DONE
      BLK_BAD}               // GOT A DATA ERROR--STUCK HERE
Inflate_Block_Mode;
///////////////////////////////////////////////////////////////////////////////////////////////////
typedef struct S_Inflate_Huft TInf_Huft;
struct S_Inflate_Huft {
    union {
      struct {
          Byte Ex_Op;        // NUMBER OF EXTRA BITS OR OPERATION
          Byte Bits;         // NUMBER OF BITS IN THIS CODE OR SUBCODE
      } What;
      PByte  Pad;            // PAD STRUCTURE TO A POWER OF 2 (4 BYTES FOR 16-BIT,
    } AsWord;                // 8 BYTES FOR 32-BIT MACHINES)
    union {
        UInt          Base;  // LITERAL, LENGTH BASE, OR DISTANCE BASE
        TInf_Huft    *Next; // POINTER TO NEXT LEVEL OF TABLE
    } More;
};
typedef struct S_Inflate_Huft TInflate_Huft;
////////////////////////////////////////////////////////////////////////////////////////////////////
typedef enum {           // WAITING FOR "I:"=INPUT, "O:"=OUTPUT, "X:"=NOTHING
    COD_START,           // X: SET UP FOR LEN
    COD_LEN,             // I: GET LENGTH/LITERAL/EOB NEXT
    COD_LENEXT,          // I: GETTING LENGTH EXTRA (HAVE BASE)
    COD_DIST,            // I: GET DISTANCE NEXT
    COD_DISTEXT,         // I: GETTING DISTANCE EXTRA
    COD_COPY,            // O: COPYING BYTES IN WINDOW, WAITING FOR SPACE
    COD_LIT,             // O: GOT LITERAL, WAITING FOR OUTPUT SPACE
    COD_WASH,            // O: GOT EOB, POSSIBLY STILL OUTPUT WAITING
    COD_END,             // X: GOT EOB AND ALL DATA FLUSHED
    COD_BAD              // X: GOT ERROR
} Inf_Code_Mode;         // CURRENT INFLATE_CODES MODE
////////////////////////////////////////////////////////////////////////////////////////////////////
struct Inflate_Codes_State { // INFLATE CODES PRIVATE STATE

    Inf_Code_Mode InfCodeMode;    // CURRENT INFLATE_CODES MODE

    // MODE DEPENDENT INFORMATION
    UInt ThisLenght;
    union {
      struct {
        TInflate_Huft *Tree;      // POINTER INTO TREE
        UInt BitsNeeded;          // BITS NEEDED
      } Code;                     // IF LEN OR DIST, WHERE IN TREE
      UInt Lit;                   // IF LIT, LITERAL
      struct {
        UInt BitsToGet;           // BITS TO GET FOR EXTRA
        UInt From;                // DISTANCE BACK TO COPY FROM
      } Copy;                     // IF EXT OR COPY, WHERE AND HOW MUCH
    } SubMode;

    // MODE INDEPENDENT INFORMATION
    Byte LitBits;                 // LITTREE BITS DECODED PER BRANCH
    Byte DistBits;                // DISTTREE BITS DECODER PER BRANCH
    TInflate_Huft *LitTree;       // LITERAL/LENGTH/EOB TREE
    TInflate_Huft *DistTree;      // DISTANCE TREE
};
typedef struct Inflate_Codes_State TInflate_Codes_State;
////////////////////////////////////////////////////////////////////////////////////////////////////
struct Inflate_Blocks_State { // INFLATE BLOCKS SEMI-PRIVATE STATE

    Inflate_Block_Mode  InfBlockMode; // CURRENT INFLATE_BLOCK MODE

    // MODE DEPENDENT INFORMATION
    union {
      UInt             BytesLeft;     // IF STORED, BYTES LEFT TO COPY
      struct {
        UInt           TblLen;        // TABLE LENGTHS (14 BITS)
        UInt           Index;         // INDEX INTO BITLEN (OR BORDER)
        UInt *         BitLens;       // BIT LENGTHS OF CODES
        UInt           BitDeep;       // BIT LENGTH TREE DEPTH
        TInflate_Huft *DecodeBit;     // BIT LENGTH DECODING TREE
      } Trees;                        // IF DTREE, DECODING INFO FOR TREES
      struct {
        TInflate_Codes_State *Codes;
      } Decode;                       // IF CODES, CURRENT STATE
    } SubMode;
    UInt LastBlk;                     // TRUE IF THIS BLOCK IS THE LAST BLOCK

    // MODE INDEPENDENT INFORMATION
    UInt            BitsLeft;         // BITS IN BIT BUFFER
    ULong           BitBuffer;        // BIT BUFFER
    TInflate_Huft * hufts;            // SINGLE MALLOC FOR TREE SPACE
    PByte           WinSlide;         // SLIDING WINDOW
    PByte           WinEnd;           // ONE BYTE AFTER SLIDING WINDOW
    PByte           WinRead;          // WINDOW READ POINTER
    PByte           WinWrite;         // WINDOW WRITE POINTER
    ULong           OutCheck;         // CHECK ON OUTPUT
};
typedef struct Inflate_Blocks_State TInflate_Blocks_State;
////////////////////////////////////////////////////////////////////////////////////////////////////
typedef enum {
    CUR_INF_METHOD,   // WAITING FOR METHOD BYTE
    CUR_INF_FLAG,     // WAITING FOR FLAG BYTE
    CUR_INF_DICT4,    // FOUR DICTIONARY CHECK BYTES TO GO
    CUR_INF_DICT3,    // THREE DICTIONARY CHECK BYTES TO GO
    CUR_INF_DICT2,    // TWO DICTIONARY CHECK BYTES TO GO
    CUR_INF_DICT1,    // ONE DICTIONARY CHECK BYTE TO GO
    CUR_INF_DICT0,    // WAITING FOR INFLATESETDICTIONARY
    CUR_INF_BLOCKS,   // DECOMPRESSING BLOCKS
    CUR_INF_CHECK4,   // FOUR CHECK BYTES TO GO
    CUR_INF_CHECK3,   // THREE CHECK BYTES TO GO
    CUR_INF_CHECK2,   // TWO CHECK BYTES TO GO
    CUR_INF_CHECK1,   // ONE CHECK BYTE TO GO
    CUR_INF_DONE,     // FINISHED CHECK, DONE
    CUR_INF_BAD       // GOT AN ERROR--STAY HERE
} Cur_Inflate_Mode;   // CURRENT INFLATE MODE
////////////////////////////////////////////////////////////////////////////////////////////////////
struct Internal_State {   // INFLATE PRIVATE STATE

    Cur_Inflate_Mode CurInflateMode;     // CURRENT INFLATE MODE

    // MODE DEPENDENT INFORMATION
    union {
      UInt Method;        // IF FLAGS, METHOD BYTE
      struct {
        ULong was;        // COMPUTED CHECK VALUE
        ULong need;       // STREAM CHECK VALUE
      } CheckVal;         // IF CHECK, CHECK VALUES TO COMPARE
      UInt    Marker;     // IF BAD, INFLATESYNC'S MARKER BYTES COUNT
    } SubMode;

    // MODE INDEPENDENT INFORMATION
    UInt WinBitsSize;                   // LOG2(WINDOW SIZE)  (8..15, DEFAULTS TO 15)
};
typedef struct Internal_State TInternal_State;
////////////////////////////////////////////////////////////////////////////////////////////////////
typedef struct Struct_Z_Stream {
    PByte           NextInByte;  // NEXT INPUT BYTE
    UInt            Avail_in;    // NUMBER OF BYTES AVAILABLE AT NEXTINBYTE
    ULong           Total_in;    // TOTAL NB OF INPUT BYTES READ SO FAR

    PByte           NextOutByte; // NEXT OUTPUT BYTE SHOULD BE PUT THERE
    UInt            Avail_out;   // REMAINING FREE SPACE AT NEXTOUTBYTE
    ULong           Total_out;   // TOTAL NB OF BYTES OUTPUT SO FAR

    TInternal_State InternState; // NOT VISIBLE BY APPLICATIONS

    ULong           CheckSum;    // CRC VALUE OF THE UNCOMPRESSED DATA
} Z_Stream;

typedef Z_Stream *PZ_Stream;
////////////////////////////////////////////////////////////////////////////////////////////////////
#if( ATMEL_PLATFORM == 1)
	//No Global variables in Register now
	//register unsigned Char  SRAMCounter   asm("r10");

    #define ProgMemmory  __attribute__ ((progmem))

    typedef const Byte ProgMemmory ROMBYTE;
    typedef ROMBYTE * LPROMBYTE;

    #define MAX_MEM_BUF           64   // BYTE SIZE OF TEMP INPUT AND OUTPUT BUFFERS, NEEDS TO BE WORD ALIGNED!
    #define PDA_MODE              8    // DEVICE MODE BOUND TO PHYSICAL MEM
    #define PIX_PER_BYTE          (1)  
    #define PIX_PER_WORD          (2   * PIX_PER_BYTE)
    #define PDA_VIR_MEM_WIDTH     (800 / PIX_PER_WORD) // VIRTUAL SCREEN WIDTH IN WORDS
    #define FAR

    #define CorrectPixLUT(TheBuff) LUT[ TheBuff ]

#else //////////////////////////////////////////////////////////////////////////////////////////

    extern  Int PDA_VIR_SCREEN_WIDTH;     // 
    #define ProgMemmory

    typedef const Byte ROMBYTE;
    typedef ROMBYTE * LPROMBYTE;

    ////////////////////////////////////////////////////////////////////////////////////////////////
    #define MAX_MEM_BUF           64    // BYTE SIZE OF TEMP INPUT AND OUTPUT BUFFERS, NEEDS TO BE WORD ALIGNED!
    #define PDA_MODE              8     // DEVICE MODE BOUND TO PHYSICAL MEM
    #define PIX_PER_BYTE          (1)
    #define PIX_PER_WORD          (2   * PIX_PER_BYTE)
    #define PDA_VIR_MEM_WIDTH     (PDA_VIR_SCREEN_WIDTH / PIX_PER_WORD) // VIRTUAL SCREEN WIDTH IN WORDS

    #ifndef WIN32
       #define FAR  __far
    #else 
       #ifndef _WINDOWS_
          #define FAR 
       #endif 
    #endif

    #define CorrectPixLUT(TheBuff) TheBuff

#endif
////////////////////////////////////////////////////////////////////////////////////////////////////
extern const TInflate_Huft  ProgMemmory _Fixed_LitTree   [(128 * 4)];
extern const TInflate_Huft  ProgMemmory _Fixed_DistTree  [(8   * 4)];
extern const UInt           ProgMemmory _CopyLens        [32];
extern const UInt           ProgMemmory _CopyLiteral     [32];
extern const UInt           ProgMemmory _CopyDist        [30];
extern const UInt           ProgMemmory _CopyExtraBits   [30];
////////////////////////////////////////////////////////////////////////////////////////////////////
// MASKS FOR LOWER BITS (SIZE GIVEN TO AVOID SILLY WARNINGS WITH VISUAL C++)
extern const UInt                       _Inflate_Mask    [18];

extern const UInt                       _DefBorder       [20];

extern PUInt                            Inflate_Mask;
extern PUInt                            DefBorder;

////////////////////////////////////////////////////////////////////////////////////////////////////
extern TInflate_Huft *                  Fixed_DistTree;
extern TInflate_Huft *                  Fixed_LitTree;

extern PUInt                            CopyLens;
extern PUInt                            CopyLiteral;
extern PUInt                            CopyDist;
extern PUInt                            CopyExtraBits;

////////////////////////////////////////////////////////////////////////////////////////////////////
#define SIZEOF_COPYLENS       ((UInt)(sizeof(_CopyLens)))
#define SIZEOF_COPYLITERAL    ((UInt)(sizeof(_CopyLiteral)))
#define SIZEOF_COPYDIST       ((UInt)(sizeof(_CopyDist)))
#define SIZEOF_COPYEXTRABITS  ((UInt)(sizeof(_CopyExtraBits)))

#define SIZEOF_FIXEDLITTREE   ((UInt)(sizeof(_Fixed_LitTree)))
#define SIZEOF_FIXEDDISTTREE  ((UInt)(sizeof(_Fixed_DistTree)))

#define SIZEOF_LZ77WINDOW     ((UInt)(0x8000U))
#define SIZEOF_ZSTREAM        ((UInt)(sizeof(Z_Stream)))
#define SIZEOF_BLOCKSTATE     ((UInt)(sizeof(TInflate_Blocks_State)))
#define SIZEOF_CODESTATE      ((UInt)(sizeof(TInflate_Codes_State)))
#define SIZEOF_INFLATEMASK    ((UInt)(sizeof(_Inflate_Mask)))
#define SIZEOF_DEFBORDER      ((UInt)(sizeof(_DefBorder)))
#define SIZEOF_HUFTMANTABLE   ((UInt)(sizeof(TInflate_Huft) * HOWMANY_BRANCH))
#define SIZEOF_TREEBITSBUFFER ((UInt)(sizeof(UInt) * 20     ))
#define SIZEOF_TREEDYNBUFFER  ((UInt)(sizeof(UInt) * 288    ))
#define SIZEOF_TREESBUFFER    ((UInt)(sizeof(Byte) * MAX_MEM))
#define SIZEOF_UNDEFBUFFER    ((UInt)(sizeof(Byte) * 32     ))

#define SIZEOF_MEMORYSIZE     (SIZEOF_LZ77WINDOW                                                   \
                              +SIZEOF_ZSTREAM        +SIZEOF_BLOCKSTATE     +SIZEOF_CODESTATE      \
                              +SIZEOF_INFLATEMASK    +SIZEOF_DEFBORDER                             \
                              +SIZEOF_HUFTMANTABLE   +SIZEOF_TREEBITSBUFFER                        \
                              +SIZEOF_COPYLENS       +SIZEOF_COPYLITERAL                           \
                              +SIZEOF_COPYDIST       +SIZEOF_COPYEXTRABITS                         \
                              +SIZEOF_FIXEDLITTREE   +SIZEOF_FIXEDDISTTREE                         \
                              +SIZEOF_TREEDYNBUFFER  +SIZEOF_TREESBUFFER                           \
                              +SIZEOF_UNDEFBUFFER)   // +0x2000)

extern UInt                   SizeofZStream;
extern UInt                   SizeofCopyLens;
extern UInt                   SizeofCopyLiteral;
extern UInt                   SizeofCopyDist;
extern UInt                   SizeofCopyExtraBits;

extern UInt                   SizeofLZ77Window;
extern UInt                   SizeofBlockState;
extern UInt                   SizeofCodeState;
extern UInt                   SizeofHuftmanTable;
extern UInt                   SizeofTreeBitsBuffer;
extern UInt                   SizeofTreeDynBuffer;

extern UInt                   SizeofDefBorder;
extern UInt                   SizeofInflateMask;

extern UInt                   SizeofFixLitTree;
extern UInt                   SizeofFixDistTree;

extern UInt                   SizeofTreesBuffer;
extern UInt                   SizeofUndefBuffer;

////////////////////////////////////////////////////////////////////////////////////////////////////
extern PZ_Stream              Zee;          // SIZEOF_ZSTREAM
extern PByte                  LZ77Window;   // SIZEOF_LZ77WINDOW
extern TInflate_Huft         *HuftmanTable; // SIZEOF_HUFTMANTABLE

extern TInflate_Codes_State  *ICState;      // SIZEOF_CODESTATE
extern TInflate_Blocks_State *IBState;      // SIZEOF_BLOCKSTATE

extern PUInt                  TreesBitsBuf; // SIZEOF_TREEBITSBUFFER // WORK AREA FOR HUFTBUILD IN INFLATETREESBITS()
extern PUInt                  TreesDynBuf;  // SIZEOF_TREEDYNBUFFER  // WORK AREA FOR HUFTBUILD IN INFLATETREESDYNAMIC()

extern PByte                  TreesBuffer;  // SIZEOF_TREESBUFFER

extern Int                    Access_Buf;

extern PByte                  MemoryWindow;
extern ULong                  MemoryWindowSize;

////////////////////////////////////////////////////////////////////////////////////////////////////
// TEST UNZIP LOOP
////////////////////////////////////////////////////////////////////////////////////////////////////
void   ZUnzip                 (void);
void   StackPoint             (void);

extern Byte FAR *             HoldImage;

////////////////////////////////////////////////////////////////////////////////////////////////////
extern  PChar                 Z_Current_Error_String;
extern  Byte                  Z_Current_Error_Num;
extern  Byte                  Z_Current_FastCode;

////////////////////////////////////////////////////////////////////////////////////////////////////
#define ReportErrorStr(ErrStr, ErrValue)  Z_Current_Error_Num = ErrValue;
////////////////////////////////////////////////////////////////////////////////////////////////////
/*
   The application must update NextInByte and Avail_in when Avail_in has
   dropped to zero. It must update NextOutByte and Avail_out when Avail_out
   has dropped to zero. The application must initialize zalloc, zfree and
   Object before calling the init function. All other fields are set by the
   compression library and must not be updated by the application.

   The Object value provided by the application will be passed as the first
   parameter for calls of zalloc and zfree. This can be useful for custom
   memory management. The compression library attaches no meaning to the
   Object value.

   zalloc must return Z_NULL if there is not enough memory for the object.
   On 16-bit systems, the functions zalloc and zfree must be able to allocate
   exactly 65536 bytes, but will not be required to allocate more than this
   if the symbol MAXSEG_64K is defined (see zconf.h). WARNING: On MSDOS,
   pointers returned by zalloc for objects of exactly 65536 bytes *must*
   have their offset normalized to zero. The default allocation function
   provided by this library ensures this (see zutil.c). To reduce memory
   requirements and avoid any allocation of 64K objects, at the expense of
   compression ratio, compile the library with -DMAX_WBITS=14 (see zconf.h).

   The fields Total_in and Total_out can be used for statistics or
   progress reports. After compression, Total_in holds the total size of
   the uncompressed data and may be saved for use in the decompressor
   (particularly if the decompressor wants to decompress everything in
   a single step).
*/
////////////////////////////////////////////////////////////////////////////////////////////////////
// DEFINES FOR INFLATE INPUT/OUTPUT  UPDATE POINTERS AND RETURN
#define UPDBITS {IBState->BitBuffer = BitBuffer; IBState->BitsLeft = BitsInBitBuffer;}
#define UPDIN {Zee->Avail_in = InputAvalaible; Zee->Total_in += InputDataPtr - Zee->NextInByte; Zee->NextInByte = InputDataPtr;}
#define UPDOUT {IBState->WinWrite = OutputWindowPtr;}
#define UPDATE {UPDBITS UPDIN UPDOUT}
#define LEAVE  {UPDATE return InflateFlush(RetCode);}
//   GET BYTES AND BITS
#define NEEDBYTE {if(InputAvalaible){ RetCode = Z_OK; } else { LEAVE } }
#define NEXTBYTE (InputAvalaible--, *InputDataPtr++)

#define LOADIN {InputDataPtr = Zee->NextInByte; InputAvalaible = Zee->Avail_in; BitBuffer = IBState->BitBuffer; BitsInBitBuffer = IBState->BitsLeft;}
#define NEEDBITS(j) {while(BitsInBitBuffer < (j)){NEEDBYTE; BitBuffer |= ((ULong)NEXTBYTE) << BitsInBitBuffer; BitsInBitBuffer += 8;}}
#define DUMPBITS(j) {BitBuffer >>= (j); BitsInBitBuffer -= (j);}
//   OUTPUT BYTES
#define WAVAIL (UInt)(OutputWindowPtr < IBState->WinRead ? IBState->WinRead - OutputWindowPtr - 1 : IBState->WinEnd - OutputWindowPtr)
#define LOADOUT {OutputWindowPtr  = IBState->WinWrite; WinEndCount = (UInt)WAVAIL;}
#define WRAP {if(OutputWindowPtr == IBState->WinEnd && IBState->WinRead != IBState->WinSlide){OutputWindowPtr = IBState->WinSlide; WinEndCount = (UInt)WAVAIL;} }
#define FLUSH {UPDOUT RetCode = InflateFlush(RetCode); LOADOUT}
#define NEEDOUT {if(WinEndCount == 0){WRAP if(WinEndCount == 0){FLUSH WRAP if(WinEndCount == 0) LEAVE}}RetCode = Z_OK;}
#define OUTBYTE(a) {*OutputWindowPtr++ = (Byte)(a); WinEndCount--;}
//   LOAD LOCAL POINTERS
#define LOAD {LOADIN LOADOUT}

// COPY AS MUCH AS POSSIBLE FROM THE SLIDING WINDOW TO THE OUTPUT AREA
Int InflateFlush (Int);
////////////////////////////////////////////////////////////////////////////////////////////////////
/*
     The 'zlib' compression library provides in-memory compression and
  decompression functions, including integrity checks of the uncompressed
  data.  This version of the library supports only one compression method
  (deflation) but other algorithms may be added later and will have the same
  stream interface.

     For compression the application must provide the output buffer and
  may optionally provide the input buffer for optimization. For decompression,
  the application must provide the input buffer and may optionally provide
  the output buffer for optimization.

     Compression can be done in a single step if the buffers are large
  enough (for example if an input file is mmap'ed), or can be done by
  repeated calls of the compression function.  In the latter case, the
  application must provide more input and/or consume the output
  (providing more output space) before each call.

     The library does not install any signal handler. It is recommended to
  add at least a handler for SIGSEGV when decompressing; the library checks
  the consistency of the input data whenever possible but may go nuts
  for some forms of corrupted input.
*/
////////////////////////////////////////////////////////////////////////////////////////////////////
Int InflateInit  (void);
/*
     Initializes the internal stream state for decompression. The fields
   zalloc, zfree and Object must be initialized before by the caller.  If
   zalloc and zfree are set to Z_NULL, inflateInit updates them to use default
   allocation functions.

     inflateInit returns Z_OK if success, Z_MEM_ERROR if there was not
   enough memory, Z_VERSION_ERROR if the zlib library version is incompatible
   with the version assumed by the caller.  msg is set to null if there is no
   error message. inflateInit does not perform any decompression: this will be
   done by inflate().
*/
////////////////////////////////////////////////////////////////////////////////////////////////////
Int Inflate (Int Flush);
/*
  Performs one or both of the following actions:

  - Decompress more input starting at NextInByte and update NextInByte and avail_in
    accordingly. If not all input can be processed (because there is not
    enough room in the output buffer), NextInByte is updated and processing
    will resume at this point for the next call of inflate().

  - Provide more output starting at NextOutByte and update NextOutByte and avail_out
    accordingly.  inflate() provides as much output as possible, until there
    is no more input data or no more space in the output buffer (see below
    about the flush parameter).

  Before the call of inflate(), the application should ensure that at least
  one of the actions is possible, by providing more input and/or consuming
  more output, and updating the next_* and avail_* values accordingly.
  The application can consume the uncompressed output when it wants, for
  example when the output buffer is full (avail_out == 0), or after each
  call of inflate(). If inflate returns Z_OK and with zero avail_out, it
  must be called again after making room in the output buffer because there
  might be more output pending.

    If the parameter flush is set to Z_PARTIAL_FLUSH, inflate flushes as much
  output as possible to the output buffer. The flushing behavior of inflate is
  not specified for values of the flush parameter other than Z_PARTIAL_FLUSH
  and Z_FINISH, but the current implementation actually flushes as much output
  as possible anyway.

    inflate() should normally be called until it returns Z_STREAM_END or an
  error. However if all decompression is to be performed in a single step
  (a single call of inflate), the parameter flush should be set to
  Z_FINISH. In this case all pending input is processed and all pending
  output is flushed; avail_out must be large enough to hold all the
  uncompressed data. (The size of the uncompressed data may have been saved
  by the compressor for this purpose.) The next operation on this stream must
  be inflateEnd to deallocate the decompression state. The use of Z_FINISH
  is never required, but can be used to inform inflate that a faster routine
  may be used for the single inflate() call.

  inflate() returns Z_OK if some progress has been made (more input processed or more output produced), 
  
  Z_STREAM_END    if the end of the  compressed data has been reached and all uncompressed output has been produced, 
  Z_NEED_DICT     if a preset dictionary is needed at this point (see inflateSetDictionary below), 
  Z_DATA_ERROR    if the input data was corrupted,
  Z_STREAM_ERROR  if the stream structure was inconsistent (for example if NextInByte or NextOutByte was NULL), 
  Z_MEM_ERROR     if there was not enough memory,
  Z_BUF_ERROR     if no progress is possible or if there was not enough room in the output buffer when Z_FINISH is used. 
  Z_DATA_ERROR    case, the  application may then call inflateSync to look for a good compression block.
  Z_NEED_DICT     case, strm->CheckSum is set to the CrC32 value of the dictionary chosen by the compressor.
*/
////////////////////////////////////////////////////////////////////////////////////////////////////
Int InflateEnd (void);
/*
     All dynamically allocated data structures for this stream are freed.
   This function discards any unprocessed input and does not flush any
   pending output.

     inflateEnd returns Z_OK if success, Z_STREAM_ERROR if the stream state
   was inconsistent. In the error case, msg may be set but then points to a
   static string (which must not be deallocated).
*/
////////////////////////////////////////////////////////////////////////////////////////////////////
Int InflateReset (void);
/*
     This function is equivalent to inflateEnd followed by inflateInit,
   but does not free and reallocate all the internal decompression state.
   The stream will keep attributes that may have been set by inflateInit2.

      inflateReset returns Z_OK if success, or Z_STREAM_ERROR if the source
   stream state was inconsistent (such as zalloc or state being NULL).

                         checksum functions

     These functions are not related to compression but are exported
   anyway because they might be useful in applications using the
   compression library.
*/
////////////////////////////////////////////////////////////////////////////////////////////////////
ULong CheckSum32 (ULong CheckSum, PByte buf, UInt len);
/*
     Update a running CheckSum-32 checksum with the bytes buf[0..len-1] and
   return the updated checksum. If buf is NULL, this function returns
   the required initial value for the checksum.
   An CheckSum-32 checksum is almost as reliable as a CRC32 but can be computed
   much faster. Usage example:

     ULong CheckSum = CheckSum32(0L, Z_NULL, 0);

     while (read_buffer(buffer, length) != EOF) {
       CheckSum = CheckSum32(CheckSum, buffer, length);
     }
     if(CheckSum != original_CheckSum) error();
*/
///////////////////////////////////////////////////////////////////////////////////////////////////
// INTERNALS
///////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////
// Z_Utils.h
///////////////////////////////////////////////////////////////////////////////////////////////////
void    UnZipMemCopy     (PByte dest,   PByte source, UInt len);
Int     UnZipMemCompare  (PByte Str1,   PByte Str2, UInt len);
void    UnZipMemClear    (PByte dest,   UInt  len);
void    UnZipMemFill     (PByte dest,   UInt  len,  Byte Fill);

PVoid   UnZipCalloc      (unsigned items, unsigned size);
void    UnZipFree        (PVoid BufPtr);

// #ifndef ZMemCopy       

   #define ZMemCopy      UnZipMemCopy      
   #define ZMemCompare   UnZipMemCompare      
   #define ZMemClear     UnZipMemClear      
   #define ZMemFill      UnZipMemFill      

// #endif


/* Diagnostic functions */
#  define Assert(cond,msg)
#  define Trace(x)
#  define Tracev(x)
#  define Tracevv(x)
#  define Tracec(c,x)
#  define Tracecv(c,x)


///////////////////////////////////////////////////////////////////////////////////////////////////
// InfCodes.h
///////////////////////////////////////////////////////////////////////////////////////////////////
void InflateCodesReset            (UInt, UInt,
                                   TInflate_Huft *, TInflate_Huft *);

Int InflateCodes                  (Int RetCode);

///////////////////////////////////////////////////////////////////////////////////////////////////
// InfBlocks.h
///////////////////////////////////////////////////////////////////////////////////////////////////
Int  InflateBlocks            (Int RetCode);              // initial return code 

void InflateBlocksNew         (void);

void InflateBlocksReset       (void);

Int  InflateBlocksFree        (void);

///////////////////////////////////////////////////////////////////////////////////////////////////
// InfTrees.h
///////////////////////////////////////////////////////////////////////////////////////////////////
Int InflateTreesBits (
    UInt *,                     // 19 code lengths 
    UInt *,                     // bits tree desired/actual depth 
    TInflate_Huft **,           // bits tree result 
    TInflate_Huft *);           // space for trees 

Int InflateTreesDynamic (
    UInt,                       // number of literal/length codes 
    UInt,                       // number of distance codes 
    UInt *,                     // that many (total) code lengths 
    UInt *,                     // literal desired/actual bit depth 
    UInt *,                     // distance desired/actual bit depth 
    TInflate_Huft **,           // literal/length tree result 
    TInflate_Huft **,           // distance tree result 
    TInflate_Huft *);           // space for trees 

///////////////////////////////////////////////////////////////////////////////////////////////////
// InfFixed.h
///////////////////////////////////////////////////////////////////////////////////////////////////
Int InflateTreesFixed    (
                          PUInt LitBit,               // literal desired/actual bit depth 
                          PUInt DistBit,              // distance desired/actual bit depth 
                          TInflate_Huft ** LitTree,   // literal/length tree result 
                          TInflate_Huft ** DistTree); // distance tree result 

///////////////////////////////////////////////////////////////////////////////////////////////////
// InfFast.h
///////////////////////////////////////////////////////////////////////////////////////////////////
Int InflateFast        (UInt,
                        UInt,
                        TInflate_Huft *,
                        TInflate_Huft *);

///////////////////////////////////////////////////////////////////////////////////////////////////
// DevUnzip.h
///////////////////////////////////////////////////////////////////////////////////////////////////
void InitImageFrame        (void);
Int  PrepareMemorys        (void);
Int  UnzipImageFrame       (void); 
Int	 PaintUnCompresedFrame (void);

///////////////////////////////////////////////////////////////////////////////////////////////////
#ifdef WIN32
#define Beep() ; // MessageBeep(0);
#else
#define Beep() ;
#endif
////////////////////////////////////////////////////////////////////////////////////////////////////
#ifdef __cplusplus
} 
#endif
////////////////////////////////////////////////////////////////////////////////////////////////////
#endif // _UnZipLib_H

