// DeviceHandle.h
///////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef __DEVICEHANDLE_H_
#define __DEVICEHANDLE_H_
///////////////////////////////////////////////////////////////////////////////////////////////////
extern "C" {
///////////////////////////////////////////////////////////////////////////////////////////////////
extern  void (__stdcall* OnPadStatus)   (DWORD CurrentStatus);
typedef void (__stdcall* PADSTATUSPROC) (DWORD CurrentStatus);
///////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef DLL_THREAD_MANAGEMENT
#define DLL_THREAD_MANAGEMENT()
#endif
///////////////////////////////////////////////////////////////////////////////////////////////////
// EXPORTABLE FUNCTIONS
///////////////////////////////////////////////////////////////////////////////////////////////////
BOOL    __stdcall InitScreen                    ();
BOOL    __stdcall StartDevice                   ();
BOOL    __stdcall StopDevice                    ();

BOOL    __stdcall OpenFileForPlayback           (LPCTSTR Filename);

BOOL    __stdcall IsPortOpen                    ();
BOOL    __stdcall OpenPort                      (LPCTSTR PortName, LPCTSTR PortValue);
BOOL    __stdcall ClosePort                     ();

BOOL    __stdcall SendData                      (const PBYTE Data, int Length);
BOOL    __stdcall RecvData                      (PBYTE RecvBuffer, int Length);

#if USE_ECHO
void    __stdcall TriggerEventStatus            ();
#else 
#define  TriggerEventStatus()
#endif

BOOL    __stdcall SendScrollAck                 (BYTE ScrollID);
BOOL    __stdcall SendFrameAck                  (BYTE SeqID,   BYTE FrameID, BYTE CurrentPhase, BYTE CurrentState);
BOOL    __stdcall SendAck                       (BYTE AckType, BYTE SeqID, BYTE FrameID, BYTE CurrentPhase, BYTE CurrentState);

BOOL    __stdcall PostABeacon                   ();
BOOL    __stdcall PostAGpsBeacon                ();

BOOL    __stdcall SendAScrollCommand            (short X, short Y, BYTE Status);
BOOL    __stdcall SendAnInit                    (CHAR ActionMode);
BOOL    __stdcall SendAConfig                   ();
BOOL    __stdcall SendAModeSwitch               ();

BOOL    __stdcall SendFocusText                 (DWORD  WId, DWORD  OId, DWORD  CId, LPCTSTR FocusText);
LPCTSTR __stdcall GetCurrentFocusText           (long * WId, long * OId, long * CId, long * Restricted);

void    __stdcall EmptyWriteBufferQueue         ();
BOOL    __stdcall WriteChannelBuffer            (LPCSTR BufferToWrite, DWORD SizeofBufferToWrite);

BOOL    __stdcall SendRawData                   (LPBYTE ByteToSend, long DataLen);
BOOL    __stdcall SendAString                   (LPCTSTR StringToSend);

BOOL    __stdcall SendATextString               (LPCTSTR StringToSend);
BOOL    __stdcall SendAFunctionString           (WORD FunctionKey, LPCTSTR StringToSend);
BOOL    __stdcall SendAFunctionKeyString        (WORD FunctionKey, WORD ExtraState, LPCTSTR StringToSend);


BOOL    __stdcall SendAMouseEvent               (long MouseState, long VirtualX, long VirtualY);

void    __stdcall PenDrawFreeLine               ();

///////////////////////////////////////////////////////////////////////////////////////////////////
//
///////////////////////////////////////////////////////////////////////////////////////////////////
#if USE_ECHO
BOOL    __stdcall ClearEcho                   ();
BOOL    __stdcall ClearStatus                 ();


long    __stdcall GetStatus                   ();

long    __stdcall GetStatusString             (LPTSTR StringBuffer, long ThisStrLen);
long    __stdcall GetTextEchoString           (LPTSTR StringBuffer, long ThisStrLen);
long    __stdcall GetRawEchoString            (LPTSTR StringBuffer, long ThisStrLen);

BOOL    __stdcall RegisterPadStatus           (PADSTATUSPROC PadHandler);
#else

#define ClearEcho() TRUE
#define ClearStatus() TRUE

#define GetStatus()  TRUE
#define GetStatusString(StringBuffer, ThisStrLen)    0
#define GetTextEchoString(StringBuffer, ThisStrLen)  0
#define GetRawEchoString(StringBuffer, ThisStrLen)   0
#define RegisterPadStatus(PadHandler) TRUE

#endif
///////////////////////////////////////////////////////////////////////////////////////////////////
typedef BYTE  PixColor, * PPixColor;  // FORMAT TO HOLD A SINGLE PIXSEL COLOR...
///////////////////////////////////////////////////////////////////////////////////////////////////
#define NOTIFY_ON_REGISTER_USER         0x1001
#define NOTIFY_ON_REGISTER_SERIAL       0x1002
#define NOTIFY_ON_NEW_FOCUS             0x1003
#define NOTIFY_ON_CONFIG_CHANGE         0x1004
#define NOTIFY_ON_DEVICE_STATUS         0x1005
///////////////////////////////////////////////////////////////////////////////////////////////////
typedef enum  {
    PDA_HOMEKEY       = 0x0080, PDA_STOPKEY      = 0x0081, PDA_FAVORITESKEY  = 0x0082,  //
    PDA_ADDRBARKEY    = 0x0083, PDA_REFRESHKEY   = 0x0084, PDA_GOKEY         = 0x0085,
    PDA_BACKWARDKEY   = 0x0086, PDA_FORWARDKEY   = 0x0087,
    PDA_F1KEY         = 0x0088, PDA_F2KEY        = 0x0089, PDA_F3KEY         = 0x008A
    //case HISTORYKEY:   PenClick (PDABROWSER_BUTHISTORY + 8, 4); break;
    //case SETTINGKEY:   PenClick (PDABROWSER_BUTSETTING + 8, 4); break;
    //case UNKNOWNKEY:   PenClick (PDABROWSER_BUTUNKNOWN + 8, 4); break;
} PdaFunctionKeys;


typedef enum {
    ACK_FRAME_SUCCESSFULL     = 0xD0,
    NACK_ERROR_RECEIVE        = 0xE1,
    NACK_ERROR_CACHECOMMAND   = 0xE2,
    NACK_ERROR_IMAGESIZE      = 0xE3,
    NACK_ERROR_PARITY         = 0xE4,
    NACK_ERROR_COMPRESSION    = 0xE5,
    NACK_ERROR_BUFFER         = 0xE6, // TEMPORAY BUFFER EXCEEDED
    NACK_ERROR_CACHEFAILED    = 0xE7, // FAILED TO RETREIVE FROM CACHE
    NACK_ERROR_FRAME          = 0xE8,
    NACK_ERROR_UNSUPPORTED    = 0xE9  // PROBABLY A COMPRESSION, COLOR OR COMMAND NOT SUPPORTED

} ACKSTATUS;
///////////////////////////////////////////////////////////////////////////////////////////////////
typedef enum PDA_USE_COMPRESS_METHOD {
    PD_COMPRESS_NONE       = 0,
    PD_COMPRESS_DEFAULT    = 3,
    PD_COMPRESS_G3         = 3,
    PD_COMPRESS_G4         = 4,
    PD_COMPRESS_N3         = 30,
    PD_COMPRESS_N4         = 40,
    PD_COMPRESS_JPEG       = 'J',
    PD_COMPRESS_ZIP        = 'Z'
} PDA_USE_COMPRESS_METHOD, *PPDA_USE_COMPRESS_METHOD;
/*
///////////////////////////////////////////////////////////////////////////////////////////////////
typedef enum PDA_USE_COLOR_MODE {
	PDA_MONO         = 0,
	PDA_2BITGRAY     = 1,
	PDA_4BITGRAY     = 2,
	PDA_3BITCOLOR    = 3,
	PDA_4BITCOLOR    = 4,
	PDA_8BITCOLOR    = 8
} PDA_USE_COLOR_MODE, *PPDA_USE_COLOR_MODE;
*/
///////////////////////////////////////////////////////////////////////////////////////////////////
typedef enum PDALCD_COLORS {
    PDALCD_NORMAL         = 0,
    PDALCD_BLUEBACKLIGHT  = 1,
    PDALCD_GREENBACKLIGHT = 2,
    PDALCD_SANDED         = 3,
    PDALCD_DESERT         = 4,
    PDALCD_GREYSANDED     = 5,
    PDALCD_GREYMETALIC    = 6,
    PDALCD_PINKYPOWERPUFF = 7,
    PDALCD_OCEAN          = 8,
    PDALCD_WHITEPAPER     = 9,
    PDALCD_USECOLORDEF    = -1
} PDALCD_COLORS, *PPDALCD_COLORS;
///////////////////////////////////////////////////////////////////////////////////////////////////
typedef enum PDA_ECHOSETTINGS {
    ECHONONE        = 0x0,
    ECHOALLMASK     = 0x3,
    ECHOONSELECTION = 0x1,
    ECHOONFILTER    = 0x2,
    ECHOALLAPPLYED  = 0x3,
    ECHORAW         = 0x4,
    ECHOTEXT        = 0x8,
    ECHOFRAME       = 0x10,
    ECHOFILL        = 0x20,
    ECHOBEACON      = 0x40,
    ECHOTASK        = 0x80,
    ECHOTIME        = 0x100,
    ECHOSTATUS      = 0x200,
    ECHOERRORS      = 0x400,
    ECHOSCROLL      = 0x1000
} PDA_ECHOSETTINGS, *PPDA_ECHOSETTINGS;
///////////////////////////////////////////////////////////////////////////////////////////////////
typedef enum PDA_PROCESSSETTINGS {
    PROCESSCHANNEL    = 0x10,
    PROCESSSENDBEACON = 0x20
} PDA_PROCESSSETTINGS, *PPDA_PROCESSSETTINGS;
///////////////////////////////////////////////////////////////////////////////////////////////////
typedef enum {
    PDA_FUNCTIONNALITY1        = 1,
    PDA_FUNCTIONNALITY2        = 2,
    PDA_FUNCTIONNALITY3        = 3,
    PDA_FUNCTIONNALITY4        = 4,
    PDA_FUNCTIONNALITY5        = 5,
    PDA_FUNCTIONNALITY6        = 6,
    PDA_FUNCTIONNALITY7        = 7,
    PDA_FUNCTIONNALITY8        = 8,

    PDA_PORPERTY               = 9

} PDAFEATURES, * PPDAFEATURES;
///////////////////////////////////////////////////////////////////////////////////////////////////
typedef enum PDA_FEATURES_FUNCTIONNALITY {
    PDA_SUPPORT_NONE           = 0,
    PDA_SUPPORT_SCROLLING      = 1,
    PDA_SUPPORT_PENUP          = 2,
    PDA_SUPPORT_PENMOVE        = 4,
    PDA_SUPPORT_PENCLICK       = 8,
    PDA_SUPPORT_PENDBCLICK     = 10
/*
    PDA_SUPPORT_UNSUPPORTED    = 0x20,
    PDA_SUPPORT_UNSUPPORTED    = 0x40,
    PDA_SUPPORT_UNSUPPORTED    = 0x80
*/
} PDA_FEATURES_FUNCTIONNALITY, * PPDA_FEATURES_FUNCTIONNALITY;
///////////////////////////////////////////////////////////////////////////////////////////////////
typedef enum PDA_FEATURES_FUNCTIONNALITY2 {
    //PDA_SUPPORT_NONE         = 0,
    PDA_SUPPORT_DYNPALCOMPRESS = 1,
    PDA_SUPPORT_DYNPAL         = 2,
/*
    PDA_SUPPORT_UNSUPPORTED    = 4,
    PDA_SUPPORT_UNSUPPORTED    = 8,
    PDA_SUPPORT_UNSUPPORTED    = 10,
*/
    PDA_SUPPORT_ROP            = 0x20,
    PDA_SUPPORT_PATFILL        = 0x40
/*
    PDA_SUPPORT_UNSUPPORTED    = 0x80,
*/
} PDA_FEATURES_FUNCTIONNALITY2, * PPDA_FEATURES_FUNCTIONNALITY2;
///////////////////////////////////////////////////////////////////////////////////////////////////
typedef enum PDA_FEATURES_FUNCTIONNALITY3 {
    //PDA_SUPPORT_NONE         = 0,
    PDA_SUPPORT_MULTIPHASE     = 1,
    PDA_SUPPORT_NOPADDING      = 2,
    PDA_SUPPORT_NOKEYBOARD     = 4,
    PDA_SUPPORT_NOKEYTALE      = 8,
    PDA_SUPPORT_NOMENU         = 0x10,
    PDA_SUPPORT_NOFONT         = 0x20,
    PDA_SUPPORT_NOCLEARMEM     = 0x40,
    PDA_SUPPORT_NOBEACON       = 0x80,

} PDA_FEATURES_FUNCTIONNALITY3, * PPDA_FEATURES_FUNCTIONNALITY3;
///////////////////////////////////////////////////////////////////////////////////////////////////
typedef enum PDA_FEATURES_FUNCTIONNALITY4 {
    //PDA_SUPPORT_NONE         = 0,
    PDA_SUPPORT_XORIMGREQUIRED = 1,
    PDA_SUPPORT_XORSTRREQUIRED = 2,
/*
    PDA_SUPPORT_UNSUPPORTED    = 4,
*/
    PDA_SUPPORT_PSWREQUIRED    = 8,
/*
    PDA_SUPPORT_UNSUPPORTED    = 10,
    PDA_SUPPORT_UNSUPPORTED    = 0x20,
    PDA_SUPPORT_UNSUPPORTED    = 0x40,
*/
    PDA_SUPPORT_ENHANCESTATUS  = 0x80

} PDA_FEATURES_FUNCTIONNALITY4, * PPDA_FEATURES_FUNCTIONNALITY4;
///////////////////////////////////////////////////////////////////////////////////////////////////
typedef enum PDA_FEATURES_FUNCTIONNALITY5 {
    //PDA_SUPPORT_NONE         = 0,
    PDA_SUPPORT_RECEIVELOGO    = 1,
/*
    PDA_SUPPORT_UNSUPPORTED    = 2,
    PDA_SUPPORT_UNSUPPORTED    = 4,
    PDA_SUPPORT_UNSUPPORTED    = 8,
    PDA_SUPPORT_UNSUPPORTED    = 10,
    PDA_SUPPORT_UNSUPPORTED    = 0x20,
    PDA_SUPPORT_UNSUPPORTED    = 0x40,
*/
    PDA_SUPPORT_LOGOINCOLOR    = 0x80

} PDA_FEATURES_FUNCTIONNALITY5, * PPDA_FEATURES_FUNCTIONNALITY5;
///////////////////////////////////////////////////////////////////////////////////////////////////
/*
typedef enum PDA_FEATURES_FUNCTIONNALITY6 {
    //PDA_SUPPORT_NONE         = 0,

    PDA_SUPPORT_UNSUPPORTED    = 1,
    PDA_SUPPORT_UNSUPPORTED    = 2,
    PDA_SUPPORT_UNSUPPORTED    = 4,
    PDA_SUPPORT_UNSUPPORTED    = 8,
    PDA_SUPPORT_UNSUPPORTED    = 10,
    PDA_SUPPORT_UNSUPPORTED    = 0x20,
    PDA_SUPPORT_UNSUPPORTED    = 0x40,
    PDA_SUPPORT_UNSUPPORTED    = 0x80

} PDA_FEATURES_FUNCTIONNALITY6, * PPDA_FEATURES_FUNCTIONNALITY6;
*/
///////////////////////////////////////////////////////////////////////////////////////////////////
typedef enum PDA_FEATURES_FUNCTIONNALITY7 {
//    PDA_SUPPORT_NONE         = 0,
/*
    PDA_SUPPORT_UNSUPPORTED    = 1,
    PDA_SUPPORT_UNSUPPORTED    = 2,
    PDA_SUPPORT_UNSUPPORTED    = 4,
    PDA_SUPPORT_UNSUPPORTED    = 8,
    PDA_SUPPORT_UNSUPPORTED    = 10,
*/
    PDA_SUPPORT_CACHESTORE     = 0x20,
    PDA_SUPPORT_BIOSTRANSFER   = 0x40,
    PDA_SUPPORT_FILETRANSFER   = 0x80

} PDA_FEATURES_FUNCTIONNALITY7, * PPDA_FEATURES_FUNCTIONNALITY7;
///////////////////////////////////////////////////////////////////////////////////////////////////
typedef enum PDA_FEATURES_FUNCTIONNALITY8 {
//    PDA_SUPPORT_NONE         = 0,
/*
    PDA_SUPPORT_UNSUPPORTED    = 1,
    PDA_SUPPORT_UNSUPPORTED    = 2,
    PDA_SUPPORT_UNSUPPORTED    = 4,
    PDA_SUPPORT_UNSUPPORTED    = 8,
    PDA_SUPPORT_UNSUPPORTED    = 10,
*/
    PDA_SUPPORT_SUPPORTJPEG    = 0x20,
    PDA_SUPPORT_USECOMPRESSBYT = 0x40,
    PDA_SUPPORT_ENHANCED_SEQID = 0x80

} PDA_FEATURES_FUNCTIONNALITY8, * PPDA_FEATURES_FUNCTIONNALITY8;
///////////////////////////////////////////////////////////////////////////////////////////////////
typedef enum PDACAPABILITYS{
    PDA_CAPMONO         = 0,
    PDA_CAP2BITGRAY     = 1,
    PDA_CAP4BITGRAY     = 2,
    PDA_CAP3BITCOLOR    = 3,
    PDA_CAP4BITCOLOR    = 4,
    PDA_CAP8BITCOLOR    = 8,

    PDA_CAPDYNAPALETTE  = 0x100,
    PDA_CAPMULTIPHASE   = 0x400,
    PDA_CAPDYNATABLE    = 0x800

} PDACAPABILITYS, * PPDACAPABILITYS;
///////////////////////////////////////////////////////////////////////////////////////////////////
typedef enum PDA_LANGUAGES{
    PDA_ENGLISH    = 0X409,
    PDA_FRENCH     = 0X40C,
    PDA_CHINESE    = 0X804,
    PDA_TURKISH    = 0X41F,
    PDA_SPANISH    = 0XC0A
} PDA_LANGUAGES, * PPDA_LANGUAGES;
///////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////
typedef enum PDASTATUS{
    STATUSNOTHING       = 0x0,
    STATUSCLOSED        = 0x0,
    STATUSCONNECTED     = 0x1,
    STATUSREADING       = 0x2,
    STATUSWRITING       = 0x4,

    STATUSCONNECTERROR  = 0x80000001,
    STATUSREADERROR     = 0x80000002,
    STATUSWRITEERROR    = 0x80000004,

    STATUSCOMMANDERROR  = 0x80001000,
    STATUSFRAMEERROR    = 0x80002000,

    STATUSIMAGEERROR    = 0x80010000,
    STATUSG3ERROR       = 0x80020000,
    STATUSG4ERROR       = 0x80040000,

    STATUSHARDERROR     = -1

} PDASTATUS, * PPDASTATUS;

///////////////////////////////////////////////////////////////////////////////////////////////////
enum   {PADREFRESH = 0,PADSETPOS = 1, PADADRESSDOWN = 2, WEBINTRANSITION = 4, PDAPAGEREADY = 0X10};
///////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////
typedef struct {
    long All; // TRISTATE 0 == NONE, 1 == FILTER INDIVIDUAL, 2 == ALL ON
    BOOL Raw,
         Text,
         Frame,
         Fill,
         Scroll,
         Beacon,
         Task,
         TimeStamp,
         Status,
         Errors;

    BOOL ProcessChannel,
         EnableSendBeacon,
         SendInitOnStart,
         ClearScreenOnStart;

int      HostStatus;
int      HostTimeOut;

BOOL     HardwareFlow;
BOOL     EnableRTS,
         EnableDTR,
         UseCarrier,
         DirectConnect;

BOOL     EnablePenMove,
         EnablePenUp,
         EnablePenClicks;

BOOL     EnableScreenPos;

BYTE     Features [8];

DWORD    ImageCapsRequested;

DWORD    LangID;

BOOL     EnableTransparency;

} PDASettings, * PPDASettings;
///////////////////////////////////////////////////////////////////////////////////////////////////
extern PDASettings PadSettings;
///////////////////////////////////////////////////////////////////////////////////////////////////
#pragma pack(push, 1)
///////////////////////////////////////////////////////////////////////////////////////////////////
typedef struct {
BYTE  Command;
BYTE  SubCommand;

union {
   short X;
   struct {
   BYTE  XLSB;
   BYTE  XMSB;
   };
};
union {
   short Y;
   struct {
   BYTE  YLSB;
   BYTE  YMSB;
   };
};
BYTE status;
BYTE Terminator;

} PADSCROLLINFO, * PPADSCROLLINFO;
///////////////////////////////////////////////////////////////////////////////////////////////////
typedef struct {
BYTE  Command;
BYTE  SubCommand;

union {
   short XSrc;
   struct {
   BYTE  XSrcLSB;
   BYTE  XSrcMSB;
   };
};
union {
   short YSrc;
   struct {
   BYTE  YSrcLSB;
   BYTE  YSrcMSB;
   };
};
BYTE ScrollID;
BYTE Terminator;

} SERVERSCROLLINFO, * PSERVERSCROLLINFO;
///////////////////////////////////////////////////////////////////////////////////////////////////
typedef struct {
union {
   short XDst;
   struct {
   BYTE  XDstLSB;
   BYTE  XDstMSB;
   };
};
union {
   short YDst;
   struct {
   BYTE  YDstLSB;
   BYTE  YDstMSB;
   };
};
union {
   short WDst;
   struct {
   BYTE  WDstLSB;
   BYTE  WDstMSB;
   };
};
union {
   short HDst;
   struct {
   BYTE  HDstLSB;
   BYTE  HDstMSB;
   };
};
} DESTSCROLLINFO, * PDESTSCROLLINFO;
///////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////
typedef struct {

BYTE    Command[8];

union {
   short X;
   struct {
   BYTE  XLSB;
   BYTE  XMSB;
   };
};
union {
   short Y;
   struct {
   BYTE  YLSB;
   BYTE  YMSB;
   };
};
union {
   short W;
   struct {
   BYTE  WLSB;
   BYTE  WMSB;
   };
};
union {
   short H;
   struct {
   BYTE  HLSB;
   BYTE  HMSB;
   };
};
union {
   WORD  SumData;
   struct {
   BYTE  ByteCountLSB;
   BYTE  ByteCountMSB;
   };
};

} FRAMEINFO, * PFRAMEINFO;
///////////////////////////////////////////////////////////////////////////////////////////////////
typedef struct {

BYTE  Command;
BYTE  SubCommand;
BYTE  Reserved1;
BYTE  Reserved2;
BYTE  Bitfields;
union {
   WORD ExtraLen;
   struct {
      BYTE  LenLSB;
      BYTE  LenMSB;
   };
};
BYTE  Terminator;

union {
   short X;
   struct {
   BYTE  XLSB;
   BYTE  XMSB;
   };
};
union {
   short Y;
   struct {
   BYTE  YLSB;
   BYTE  YMSB;
   };
};
union {
   short W;
   struct {
   BYTE  WLSB;
   BYTE  WMSB;
   };
};
union {
   short H;
   struct {
   BYTE  HLSB;
   BYTE  HMSB;
   };
};
union {
   DWORD WinId;
   struct {
   BYTE  WinIdByte [4];
   };
};
union {
   DWORD  ObjectId;
   struct {
   BYTE  ObjectIdBytes [4];
   };
};
union {
   DWORD  ChildId;
   struct {
   BYTE  ChildIdBytes [4];
   };
};
TCHAR       StringLen;
TCHAR       String[4096 + 2];

} FOCUSINFO, * PFOCUSINFO;
///////////////////////////////////////////////////////////////////////////////////////////////////
typedef struct {

BYTE    Command[8];

union {
   short X;
   struct {
   BYTE  XLSB;
   BYTE  XMSB;
   };
};
union {
   short Y;
   struct {
   BYTE  YLSB;
   BYTE  YMSB;
   };
};
union {
   short W;
   struct {
   BYTE  WLSB;
   BYTE  WMSB;
   };
};
union {
   short H;
   struct {
   BYTE  HLSB;
   BYTE  HMSB;
   };
};
   BYTE  BitPerPixel;
   BYTE  Features;

union {
   WORD SumData;
   struct {
   BYTE  ByteCountLSB;
   BYTE  ByteCountMSB;
   };
};

///////////////////////////////////////////
   BYTE  CacheAction;
   BYTE  XorCheck;    // When cache is retreive use to compare integrity.

union {
   long   MemAddress;

   struct{
   short  MemAddressLSW;
   short  MemAddressMSW;
   };
   struct{
   BYTE  MemAddressLSB0;
   BYTE  MemAddressMSB0;
   BYTE  MemAddressLSB1;
   BYTE  MemAddressMSB1;
   };
};
///////////////////////////////////////////

} COLORFRAMEINFO, * PCOLORFRAMEINFO;
///////////////////////////////////////////////////////////////////////////////////////////////////
typedef struct {

BYTE  Command;
BYTE  SubCommand;
BYTE  SequenceID;
BYTE  FrameID;
BYTE  Action;
union {
   WORD ExtraLen;
   struct {
      BYTE  LenLSB;
      BYTE  LenMSB;
   };
};
BYTE  Terminator;

} PALETTEINFO, * PPALETTEINFO;

///////////////////////////////////////////////////////////////////////////////////////////////////
typedef struct {

BYTE    Command[8];

union {
   short X;
   struct {
   BYTE  XLSB;
   BYTE  XMSB;
   };
};
union {
   short Y;
   struct {
   BYTE  YLSB;
   BYTE  YMSB;
   };
};
union {
   short W;
   struct {
   BYTE  WLSB;
   BYTE  WMSB;
   };
};
union {
   short H;
   struct {
   BYTE  HLSB;
   BYTE  HMSB;
   };
};
union {
   WORD  SumData;
   struct {
   BYTE  ByteCountLSB;
   BYTE  ByteCountMSB;
   };
};
union {
   WORD  FillWith;
   struct {
   BYTE  FillWithLSB;
   BYTE  FillWithMSB;
   };
};
} FILLINFO, * PFILLINFO;
///////////////////////////////////////////////////////////////////////////////////////////////////
typedef struct {
BYTE  Command;
BYTE  SubCommand;

union {
   short Version;
   struct {
   BYTE  MajorLSB;
   BYTE  MinorMSB;
   };
};
BYTE Action;
union {
   WORD  Len;
   struct {
   BYTE  LenLSB;
   BYTE  LenMSB;
   };
};
BYTE Terminator;

} INITINFO, * PINITINFO;
///////////////////////////////////////////////////////////////////////////////////////////////////
typedef struct {
    BYTE       Command;
    BYTE       SubCommand;

    BYTE       Action[3];
    union {
       WORD    Len;
       struct {
       BYTE    LenLSB;
       BYTE    LenMSB;
       };
    };
    BYTE       Terminator;

} REROUTEINFO, * PREROUTEINFO;

///////////////////////////////////////////////////////////////////////////////////////////////////
typedef struct {
BYTE  Command;               // MUST BE 'C'
BYTE  Len;                   // MUST BE  11 THE SIZE OF THIS STRUCTURE

BYTE  ColorBitDeep;           // BIT 0 - 2 MONO, 2BIT, 4BIT, 8BIT  BIT=PALETTE CHANGE SUPPORT

BYTE  ScreenWidth;            // MULTIPLE OF 32 PIXELS EX. 10 =  320
BYTE  ScreenHeight;           // MULTIPLE OF 32 PIXELS EX.  5 =  160
BYTE  VScreenWidth;           // MULTIPLE OF 32 PIXELS EX. 20 =  640
BYTE  VScreenHeight;          // MULTIPLE OF 32 PIXELS Ex. 32 = 1024  40 = 1280  255 = 8160

BYTE  IntegrationLevel;
BYTE  CompressMethodSupported;
BYTE  LineSpeedSupported;

BYTE  Terminator;            // MUST BE 0

} PADCAPSINFO, * PPADCAPSINFO;

///////////////////////////////////////////////////////////////////////////////////////////////////
typedef struct {
BYTE  Command;               // MUST BE 'F'
BYTE  Len;                   // MUST BE  11 THE SIZE OF THIS STRUCTURE

BYTE  Features[8];           // MULTIPLE BLOCK OF FEATURES

BYTE  Terminator;            // MUST BE 0

} PADFEATUREINFO, * PPADFEATUREINFO;
///////////////////////////////////////////////////////////////////////////////////////////////////
typedef struct {

    BYTE BlockType;
    BYTE BlockLen;

    union {
       WORD LangID;
       struct {
          BYTE  LangIDLSB;
          BYTE  LangIDMSB;
       };
    };
    union {
       WORD KeyboardID;
       struct {
          BYTE  KeyboardIDLSB;
          BYTE  KeyboardIDMSB;
       };
    };
    union {
       WORD ConversionType;
       struct {
          BYTE  ConversionTypeLSB;
          BYTE  ConversionTypeMSB;
       };
    };
    WORD  Reserved1;

    BYTE  Terminator;

} LANGIDINFO, * PLANGIDINFO;
///////////////////////////////////////////////////////////////////////////////////////////////////
#pragma pack(pop)
///////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////
} // extern "C"
///////////////////////////////////////////////////////////////////////////////////////////////////
#endif //__DEVICEHANDLE_H_

