// File name: G4Compress.cpp
// date: May 3,1999
//  
// This file contains support for encoding TIFF G4 compression algorithms.
// The algorithm is based on ITU-T Recommendation T.4 and T.6
#include "stdafx.h"

#include "G_Compress.h"

extern "C" {
PBYTE   EncodedDataBufCurrentPtr;   //current spot in raw buffer

DWORD   EncodedDataSize;            //bytes unread from raw buffer
DWORD   EncodedLines;               //number of lines encoded

BYTE    EncodingByte;               //current encoded byte;
BYTE    EncodingBit;                //current encoded bit in byte

PBYTE   EncodedDataBuffer;          //encoded data buffer
DWORD   EncodedBufferMaxSize;       //# of bytes in encoded data buffer

PBYTE   RefLine;                    //reference line
DWORD   BytesInLine;                //bytes in a scanline
DWORD   PixelsPerLine;              //pixels in a scanline
DWORD   BufferNeeded;               // TWICE THE SIZEOF LINE

PBYTE   WhiteReferenceData;

BOOL    FinishedEncoding;
/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
void  FlushBits();                   
//    write EncodingByte to buffer after all Pixel in EncodingByte has been 
//    encoded. 
/////////////////////////////////////////////////////////////////////////////
void FlushBits() // IN C
{                   
// write EncodingByte to buffer after all Pixel in EncodingByte has been encoded.

//    *EncodedDataBufCurrentPtr++ = EncodingByte;               
    EncodedDataBufCurrentPtr[EncodedDataSize] = EncodingByte;               
    EncodingByte                = 0; 
    EncodingBit                 = 8;                    
    EncodedDataSize++;          
}

/////////////////////////////////////////////////////////////////////////////////
//CallFlushBits: Save encoded data to EncodedDataBuffer
//Used register: 
//              EDI = always point to EncodedDataBuffer
//              AL  = EncodingByte (temporary variable 
#define CallFlushBits \
    __asm MOV  AL,    EncodingByte             \
    __asm MOV  [EDI], AL                       \
    __asm INC  EDI                             \
    __asm MOV  EncodingByte, 0                 \
    __asm MOV  EncodingBit,  8                 \
    __asm INC  EncodedDataSize                 

// Return the offset of the next bit in the range [StartPixel..PixelsPerLine] that is 
// different from the specified color. The PixelsPerLine is returned if no such bit
// exists.
#define FindNextColorOffset(I,S,C)                   \
                           (C ? FindBlackSpan(I,S) : \
                                FindWhiteSpan(I,S))


//////////////////////////////////////////////////////////////////////////////
//MacFindNextColorOffset: find offset of opposite Color
//Input parameter:
//                  ImgSrc      = reference or coding line
//                  StartPixel  = position of start pixel
//                  N           = used to distinguish jump number
//Used register:
//                  ESI = ImgSrc
//                  EAX = StartPixel
///////////////////////////////////////////////////////////////////////////////
#define MacFindNextColorOffset(ImgSrc,StartPixel,Color,N) \
    {                                                     \
    __asm MOV  ESI,    ImgSrc                             \
    __asm MOV  EAX,    StartPixel                         \
    __asm CMP  Color,  0                                  \
    __asm JNZ  BlackRun##N                                \
    __asm CALL SubFindWhiteSpan                           \
    __asm JMP  EndFindOffset##N                           \
    __asm BlackRun##N:                                    \
    __asm CALL SubFindBlackSpan                           \
    __asm EndFindOffset##N:                               \
    }


///////////////////////////////////////////////////////////////////////////////
//MacFindPixelColor: Find color of start pixel. The result is stored in DL
//Input parameter:
//                  ImgSrc = reference or coding line
//                  EAX    = position of start pixel
//Return:
//                  DL = color of the pixel
//Used register:
//                  ESI = ImgSrc
//                  EAX = StartPixel
//                  ECX = temporary variable
//                  EBX = effective address of BitMask
///////////////////////////////////////////////////////////////////////////////
//"C" Code: #define PixelColor(buf,ix) ((((buf)[(ix)>>3]) >> (7 - ((ix) & 7)))&1)

#define MacFindPixelColor(ImgSrc)\
    __asm MOV  ESI, ImgSrc               \
    __asm MOV  ECX, EAX                  \
    __asm SHR  ECX, 3                    \
    __asm ADD  ESI, ECX                  \
    __asm AND  EAX, 7                    \
    __asm LEA  EBX, [BitMask+EAX]        \
    __asm MOV  CL,  [EBX]                \
    __asm MOV  DL,  [ESI]                \
    __asm AND  DL,  CL             


////////////////////////////////////////////////////////////////////////////////
//  MacStoreCode: Write a variable-length code (bit value) to the output stream.
//  Input Parameter: 
//                  CodeLength
//                  Code
//                  N   (used to distinguish jump labels)
//  Used register:
//                  CL  = variable related to CodeLength
//                  AL  = variable related to Code
//                  ECX = temporiary variable
//                  EDX = _msbmask + ECX
//                  EDI = EncodedDataBufCurrentPtr (in CallFlushBits)
////////////////////////////////////////////////////////////////////////////////

#define MacStoreCode(CodeLength,Code,N)    \
    __asm XOR  ECX, ECX                    \
    __asm MOV  CL,  CodeLength             \
    __asm StoreByteCode##N:                \
    __asm CMP  CL,  EncodingBit            \
    __asm JLE  StoreBitCode##N             \
    __asm MOV  AL,  Code                   \
    __asm SUB  CL,  EncodingBit            \
    __asm SHR  AL,  CL                     \
    __asm OR   EncodingByte, AL            \
    CallFlushBits                          \
    __asm JMP  StoreByteCode##N            \
    __asm StoreBitCode##N:                 \
    __asm LEA  EDX, [_msbmask + ECX]       \
    __asm MOV  AL,  [EDX]                  \
    __asm AND  AL,  Code                   \
    __asm SUB  EncodingBit,  CL            \
    __asm MOV  CL,  EncodingBit            \
    __asm SHL  AL,  CL                     \
    __asm OR   EncodingByte, AL            \
    __asm OR   CL,  CL                     \
    __asm JNZ  StoringDone##N              \
    CallFlushBits                          \
    __asm StoringDone##N:


/////////////////////////////////////////////////////////////////////////////
static void  StoreCode  (BYTE Code, BYTE CodeLength);
static void  PutCode    (const TableEntry* CodeMode);
//          Write a variable-length code (bit value) to the output stream.
//          The value is assumed to be at most 16 bit.

static void  putspan(DWORD Runlength, const TableEntry* CodeWord);
//          For horizontal mode, write the sequence of codes that describes the 
//          specified runlength of zero's or one's. The appropriate table that holds 
//          the make-up and terminating codes is supplied.

static void  PutWhiteSpan(DWORD Runlength);
//          For horizontal mode, write the sequence of codes that describes the 
//          specified runlength of zero's or one's. The appropriate table that holds 
//          the make-up and terminating codes is supplied.

static void  PutBlackSpan(DWORD Runlength);
//          For horizontal mode, write the sequence of codes that describes the 
//          specified runlength of zero's or one's. The appropriate table that holds 
//          the make-up and terminating codes is supplied.

static int  FindWhiteSpan(PBYTE ImgLine, DWORD StartPixel);
//          Find a span of White using the supplied
//          table.  The ``base'' of the bit string is supplied
//          along with the start+end bit indices.

static int  FindBlackSpan(PBYTE ImgLine, DWORD StartPixel);
//          Find a span of Black using the supplied
//          table.  The ``base'' of the bit string is supplied
//          along with the start+end bit indices.
/////////////////////////////////////////////////////////////////////////////

} // extern "C"
/////////////////////////////////////////////////////////////////////////////
G4Compressor::G4Compressor()
{
    EncodedDataBuffer        = NULL;
    EncodedBufferMaxSize     = 0;
    EncodedDataBufCurrentPtr = NULL;
    EncodedDataSize          = 0;
    EncodedLines             = 0;

    RefLine                  = NULL;
    WhiteReferenceData       = NULL;
    BytesInLine              = 0;
    PixelsPerLine            = 0;
    EncodingByte             = 0;
    EncodingBit              = 8;
    
    FinishedEncoding         = 0;
}
/////////////////////////////////////////////////////////////////////////////
G4Compressor::~G4Compressor()
{
    if(WhiteReferenceData)  delete [] WhiteReferenceData;  WhiteReferenceData = NULL;
    if(EncodedDataBuffer)   GlobalFree(EncodedDataBuffer); EncodedDataBuffer  = NULL;
}
/////////////////////////////////////////////////////////////////////////////
BOOL G4Compressor::Initialize( DWORD SizeOfLine, DWORD BufferMaxSize)
{
    EncodedBufferMaxSize = BufferMaxSize;

    BytesInLine          = SizeOfLine;
    BufferNeeded         = BytesInLine << 2; // 4 TIMES THE SIZEOF LINE
    PixelsPerLine        = SizeOfLine  << 3; // * 8
    EncodingByte         = 0;
    EncodingBit          = 8;
    FinishedEncoding     = 1;

    if(Not WhiteReferenceData)
       WhiteReferenceData   = new BYTE[ BytesInLine + 4];
    else 
       return FALSE;

    EncodedDataBuffer     = (PBYTE)GlobalAlloc(GMEM_FIXED, EncodedBufferMaxSize);
    if(EncodedDataBuffer == NULL)
       return FALSE;

    //SET REFERENCE LINE TO WHITE
    ZeroMemory( WhiteReferenceData, BytesInLine + 4);
    RefLine   = WhiteReferenceData;
    return TRUE;
}
/////////////////////////////////////////////////////////////////////////////
DWORD G4Compressor::CompressG4 (DWORD &CompressedSize, 
                              PBYTE  SourceImageBuffer,
                              DWORD  LineToConvert,
                              BOOL   StillSaving)
{
    EncodedDataBufCurrentPtr  = EncodedDataBuffer;
    EncodedDataSize           = 0;
    EncodedLines              = 0;

    if (Not G4Encode(SourceImageBuffer, LineToConvert))
        return 0;

    if (Not StillSaving){
        if (FinishedEncoding){
           if (Not AddEofBits())
               return 0;
        }
    }
    CompressedSize = EncodedDataSize;
    return EncodedLines;
}
/////////////////////////////////////////////////////////////////////////////
BOOL G4Compressor::G4Encode(PBYTE SrcImageBuffer, DWORD LineToConvert)
{
    if (Not EncodingLine(SrcImageBuffer, LineToConvert))
            return (0);

    if(EncodedLines < LineToConvert){
       FinishedEncoding = 0;
    } else {
       FinishedEncoding = 1;
    }
    CopyMemory( WhiteReferenceData, RefLine, BytesInLine);
    RefLine   = WhiteReferenceData;
    return (TRUE);
}


/////////////////////////////////////////////////////////////////////////////
static const BYTE _msbmask[9] =
                  { 0x00, 0x01, 0x03, 0x07, 0x0f, 0x1f, 0x3f, 0x7f, 0xff };
/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
BOOL G4Compressor::EncodingLine(PBYTE ImgLine, DWORD LineToConvert)
{
//encode ImgLine based on RefLine
//
//                                RefStart   RefEnd
//                                   |       |                                  
//                                   v       v                                  
//            -----------------------------------------------
//RefLine     |B|B|B|B|B|B| | | | | |B|B|B|B| | | | | | | | |
//            -----------------------------------------------
//            -----------------------------------------------
//ImgLine     |B|B|B|B| | | | | |B|B|B|B|B|B|B| | | | | | | |
//            -----------------------------------------------
//                     ^         ^             ^  
//                     |         |             |  
//                     ImgStart  ImgChange     ImgRechange
//
//      PixelColor(buf,ix) is used to find color of a pixel at position ix in ImgLine buf
#define PixelColor(buf,ix)  ((( (buf)[ (ix) >> 3 ]) >> (7 - ((ix) & 7))) & 1)
BYTE ImgColReChange;
BYTE ImgColStart   ;
BYTE RefColStart   ;

/*
WORD ImgStart,        ImgChange, ImgRechange;
WORD RefStart,        RefEnd;

    while (EncodedLines < LineToConvert){

       ImgStart     = 0;
       ImgColStart  = ((*ImgLine & 0X80));
       RefColStart  = ((*RefLine & 0X80));
       ImgChange    = (ImgColStart?0:FindWhiteSpan(ImgLine, 0));
       RefStart     = (RefColStart?0:FindWhiteSpan(RefLine, 0));

       for (;;) {
           RefColStart = PixelColor(RefLine,RefStart);
           RefEnd      = FindNextColorOffset(RefLine, RefStart, RefColStart);

           if (RefEnd >= ImgChange) {

               WORD VHz = RefStart - ImgChange;
               if (Not(-3 <= VHz && VHz <= 3)) { // HORIZONTAL MODE LOOK FOR A DIF SMALLER THAN 3 

                
                   ImgColReChange = PixelColor(ImgLine,ImgChange);
                   ImgRechange    = FindNextColorOffset(ImgLine, ImgChange, ImgColReChange);
                
                   StoreCode(HorzCode, HorzCodeLenght);
            
                   if (ImgStart+ImgChange == 0 || ImgColStart == 0) { 
                       putspan(ImgChange  -ImgStart,  TIFFaxZeroCodes);
                       //putspan(ImgRechange-ImgChange, TIFFaxOneCodes);
                       PutBlackSpan(ImgRechange-ImgChange);
                   } else {
                       //putspan(ImgChange  -ImgStart,  TIFFaxOneCodes);
                       PutBlackSpan(ImgChange  -ImgStart);
                       putspan(ImgRechange-ImgChange, TIFFaxZeroCodes);
                   }
                   ImgStart = ImgRechange; 
               } else {            // VERTICAL MODE 
                   PutCode(&VerticalCode[VHz+3]);
                   ImgStart = ImgChange;   
               }
        
           } else {                // PASS MODE 

               StoreCode(PassCode, PassCodeLenght);
               ImgStart = RefEnd;
           }
           if (ImgStart >= PixelsPerLine)
               break;
        
           ImgColStart  = PixelColor(ImgLine,ImgStart);
           ImgChange    = FindNextColorOffset(ImgLine, ImgStart,     ImgColStart);
           RefStart     = FindNextColorOffset(RefLine, ImgStart, Not ImgColStart);
           RefStart     = FindNextColorOffset(RefLine, RefStart,     ImgColStart);
       }
       //SET REFERENCE LINE TO PREVIOUS CODING LINE
       RefLine         = ImgLine;
       ImgLine        += BytesInLine;
       EncodedLines++;

       if ((EncodedBufferMaxSize - EncodedDataSize) < BufferNeeded){
           break;
       }
    } 
*/

DWORD  ImgStart,         ImgChange,  ImgRechange;
DWORD  RefStart,         RefEnd;
BYTE   NotImgColStart;

      __asm MOV  EDI,   EncodedDataBufCurrentPtr   //EDI is always set to EncodedDataBuffer 

StartCoding:
        //while (EncodedLines < LineToConvert){
       _asm{
       MOV  EAX,  EncodedLines      //while (EncodedLines < LineToConvert){
       CMP  EAX,  LineToConvert
       JAE  EndOfCoding
      
       MOV  ImgStart,  0            //ImgStart     = 0;

       MOV  EAX,  [ImgLine]
       MOV  CL,   [EAX]       
       AND  ECX,  80h               // CL = ((*ImgLine & 0X80));

       MOV  ImgColStart,CL          //ImgColStart  = ((*ImgLine & 0X80));

       //ImgChange    = (ImgColStart?0:FindWhiteSpan(ImgLine, 0));
       XOR  EAX,  EAX                    
       TEST ImgColStart, 0

       JNZ  ZeroWhiteRun
    
       MOV  ESI,  ImgLine
       CALL SubFindWhiteSpan

ZeroWhiteRun:
       MOV  ImgChange, EAX
     
       MOV  EAX,  [RefLine]
       MOV  CL,   [EAX]
       AND  ECX,  80h               //CL = ((*RefLine & 0X80));
       MOV  RefColStart, CL         // RefColStart  = ((*RefLine & 0X80));

       //RefStart     = (RefColStart?0:FindWhiteSpan(RefLine, 0));
       XOR  EAX,  EAX               //EAX = StartPixel
       TEST RefColStart, 0
       JNZ  RefZeroWhiteRun
       MOV  ESI,  RefLine
       CALL SubFindWhiteSpan

RefZeroWhiteRun:
       MOV  RefStart, EAX   

StartOfOneCodingline:
        //RefColStart = PixelColor(RefLine,RefStart);
       MacFindPixelColor(RefLine);  //input:EAX = RefStart, Return:DL = RefColStart 

        //RefEnd      = FindNextColorOffset(RefLine, RefStart, RefColStart);
       MacFindNextColorOffset(RefLine, RefStart, DL, ForRefStart);

       MOV  RefEnd, EAX

       CMP  EAX, ImgChange          //EAX = RefEnd
       JB   PassMode

       MOV  EAX, RefStart
       SUB  EAX, ImgChange          //EAX = VHz = RefStart - ImgChange;

       //if (Not(-3 <= VHz && VHz <= 3)) { // HORIZONTAL MODE LOOK FOR A DIF SMALLER THAN 3 
       CMP  EAX, -3
       JL   HorizontalMode
       CMP  EAX, 3
       JLE  VerticalMode
         
HorizontalMode:               
        //ImgColReChange = PixelColor(ImgLine,ImgChange);
       MOV EAX, ImgChange
       MacFindPixelColor(ImgLine);  //input:EAX = ImgChange, Return:DL = ImgColReChange 

       //ImgRechange    = FindNextColorOffset(ImgLine, ImgChange, ImgColReChange);
       MacFindNextColorOffset(ImgLine, ImgChange, DL,ForImgChange);
       MOV  ImgRechange, EAX
                
       //StoreCode(HorzCode, HorzCodeLenght);
       MacStoreCode(HorzCodeLenght,HorzCode,0);

       //if (ImgStart+ImgChange == 0 || ImgColStart == 0) {         
       CMP  ImgColStart, 0
       JE   WhiteRunFirst
       MOV  EAX, ImgStart
       ADD  EAX, ImgChange
       CMP  EAX, 0
       JNE  BlackRunFirst
             
WhiteRunFirst:

        //putspan(ImgChange  -ImgStart,  TIFFaxZeroCodes);
       MOV  ECX, ImgChange
       SUB  ECX, ImgStart           //ECX = ImgChange - ImgStart
       CALL SubPutWhiteSpan
 
       //putspan(ImgRechange-ImgChange, TIFFaxOneCodes);
       MOV  ECX, ImgRechange
       SUB  ECX, ImgChange          //ECX = ImgRechange - ImgChange
       CALL SubPutBlackSpan
       JMP EndOfHorizontalMode

BlackRunFirst:                       
       //putspan(ImgChange  -ImgStart,  TIFFaxOneCodes);
       MOV  ECX, ImgChange
       SUB  ECX, ImgStart           //ECX = ImgChange - ImgStart
       CALL SubPutBlackSpan

       //putspan(ImgRechange-ImgChange, TIFFaxZeroCodes);
       MOV  ECX, ImgRechange
       SUB  ECX, ImgChange          //ECX = ImgRechange - ImgChange
       CALL SubPutWhiteSpan

EndOfHorizontalMode:                    
       MOV EAX,  ImgRechange
       MOV ImgStart, EAX            //ImgStart = ImgRechange;
       JMP EndOfCodingMode

VerticalMode:

        //PutCode(&VerticalCode[VHz+3]);
       LEA  EBX, [VerticalCode+EAX*2+6]
       MacStoreCode([EBX],[EBX+1],1);

       MOV EAX, ImgChange
       MOV ImgStart, EAX            //ImgStart = ImgChange; 
       JMP EndOfCodingMode  
   
PassMode:

       MacStoreCode(PassCodeLenght,PassCode,2);
       MOV EAX, RefEnd
       MOV ImgStart, EAX            //ImgStart = RefEnd;

EndOfCodingMode:


      CMP EAX, PixelsPerLine        //EAX = ImgStart  ( if (ImgStart >= PixelsPerLine) break;)
      JGE EndOfOneCodingLine
        
      //ImgColStart  = PixelColor(ImgLine,ImgStart);
      MacFindPixelColor(ImgLine);  //input:EAX = ImgStart, Return:DL = ImgColStart

      MOV  ImgColStart, DL
      AND DL, DL
      SETE NotImgColStart           //NotImgColStart= Not ImgColStart

      //ImgChange    = FindNextColorOffset(ImgLine, ImgStart,     ImgColStart);
      MacFindNextColorOffset(ImgLine, ImgStart, DL, ForImgStart);
      MOV   ImgChange, EAX

      //RefStart     = FindNextColorOffset(RefLine, ImgStart, Not ImgColStart);
      MacFindNextColorOffset(RefLine, ImgStart, NotImgColStart,ForImgStartWithOppositeColor);

      //RefStart     = FindNextColorOffset(RefLine, RefStart,     ImgColStart);
      MacFindNextColorOffset(RefLine, EAX, ImgColStart,ForNextRefStart);  //EAX = RefStart
      MOV   RefStart, EAX
      JMP   StartOfOneCodingline
       
EndOfOneCodingLine:
       //SET REFERENCE LINE TO PREVIOUS CODING LINE

      MOV ESI, ImgLine
      MOV RefLine, ESI              //RefLine = ImgLine;
      ADD ESI, BytesInLine        
      MOV ImgLine, ESI              //ImgLine += BytesInLine;
      INC EncodedLines              // EncodedLines++;

      //if ((EncodedBufferMaxSize - EncodedDataSize) < BufferNeeded), then, end of coding
      MOV EAX, EncodedBufferMaxSize
      SUB EAX, EncodedDataSize      //EAX = EncodedBufferMaxSize - EncodedDataSize
      CMP EAX, BufferNeeded
      JL  EndOfCoding
      JMP StartCoding
     }
EndOfCoding:
    return (1);


SubFindWhiteSpan:
//////////////////////////////////////////////////////////////////////////////////
// Input Parameter:
//                  ESI = ImgLine or RefLine
//                  EAX = StartPixel
// Return Parameter:
//                  EAX = offset of next black pixel ( = StartPixel + RunLenght )
// Used Registers:
//                  ECX = temporary variable 
//                  EDX = temporary variable 
//                  EBX = effective address of ZeroRuns
/////////////////////////////////////////////////////////////////////////////////

{
DWORD  RunLenght, StartPixel, Pixel;       BYTE   StartBitInByte, BitInByte, Mask, Span;  

        _asm{

        MOV  RunLenght, EAX       //set RunLength to StartPixel
        MOV  ECX, PixelsPerLine
        CMP  ECX, EAX
        JA   FindWhiteNow
        
        MOV  EAX, PixelsPerLine
        RET
 
FindWhiteNow:

        SUB  ECX,   EAX
        MOV  Pixel, ECX           //Pixel=PixelsPerLine - StartPixel

        MOV  ECX, EAX             //StartPixel >> 3; (StartPixel / 8)

        SHR  EAX, 3
        ADD  ESI, EAX             //ImgLine += StartPixel

        AND  CL,  7               //CL = StartBitInByte = (StartPixel & 7);
        JZ   MoreThanAPixel       //AL = StartBitInByte  

        MOV  DL,  8
        SUB  DL,  CL              //DL = BitInByte = 8 - StartBitInByte;    

       // FIND SPAN NOW.....................................................

        XOR  EAX, EAX
        MOV  AL, [ESI]            //AL  =(*ImgLine)
        SHL  AL, CL               //AL  = AL << StartBitInByte
        MOV  AL,  ZeroRuns[EAX]   //AL = ZeroRuns[*ImgLine]

        CMP  AL,  8               //if (Span == 8)
        JE   SpanIs8
//ELSE...................................
        CMP  AL, DL               //ELSE IF(SPAN < BITINBYTE) // DOESN'T EXTEND TO EDGE OF BYTE 
        JNB  SpanIsOk

        ADD  EAX, RunLenght             
        RET                       //return (EAX= RunLenght + Span);

SpanIs8:
        MOV  AL, DL               //Span = BitInByte;

SpanIsOk: 
        ADD  RunLenght, EAX       //RunLenght += Span;

        AND  EDX,   0XFF          
        SUB  Pixel, EDX           //Pixel  -= BitInByte;
        INC  ESI

MoreThanAPixel:      
        MOV  EDX,  Pixel

        //scan full 32 bits
LoopForPixel32:
        CMP  DX,    32
        JB   LoopForPixel8        //while (Pixel >= 32) {

        MOV  EAX,    [ESI]
        CMP  EAX,    0X00000000 
        JNE  LoopForPixel8       //if (*ImgLine != 0X00000000)// END OF RUN 

        ADD  RunLenght,  32      //RunLenght  += 32; 
        SUB  DX,         32      //Pixel      -= 32;
        ADD  ESI,         4      //ImgLine    +=  4;
        JMP  LoopForPixel32


        //scan a full byte
LoopForPixel8:
        CMP  DX,    8
        JB   ReturnRunLenght    //while (Pixel >= 8) {

        MOV  AL,    [ESI]
        CMP  AL,    0
        JE   ContinueNextSearch  //if (*ImgLine != 0X00)// END OF RUN 

        AND  EAX,   0XFF
        MOV  AL,  ZeroRuns[EAX]  //AL = ZeroRuns[*ImgLine]

        ADD  EAX, RunLenght
        RET                     //return (EAX = RunLenght + ZeroRuns[*ImgLine]);

ContinueNextSearch:        
        ADD  RunLenght,  8      //RunLenght  += 8; 
        SUB  DX,         8      //Pixel      -= 8;
        INC  ESI                //ImgLine++;
        JMP  LoopForPixel8
        
ReturnRunLenght:
        MOV EAX, RunLenght     //return ( EAX = RunLenght + ZeroRuns[*ImgLine])
        RET
       }

} //End of SubFindWhiteSpan


SubFindBlackSpan:
//////////////////////////////////////////////////////////////////////////////////
// Input Parameter:
//                  ESI = ImgLine or RefLine
//                  EAX = StartPixel
// Return Parameter:
//                  EAX = offset of next black pixel ( = StartPixel + RunLenght )
// Used Registers:
//                  ECX = temporary variable 
//                  EDX = temporary variable 
//                  EBX = effective address of OneRuns
/////////////////////////////////////////////////////////////////////////////////

{
DWORD  RunLenght, StartPixel, Pixel;       
BYTE   StartBitInByte, BitInByte, Mask, Span;  

        _asm{

        MOV  RunLenght, EAX             //set RunLength to StartPixel
        MOV  ECX, PixelsPerLine
        CMP  ECX, EAX
        JA   FindBlackNow
        
        MOV  EAX, PixelsPerLine
        RET
 
FindBlackNow:

        SUB  ECX,   EAX
        MOV  Pixel, ECX                 //Pixel=PixelsPerLine - StartPixel

        MOV  ECX, EAX                   //StartPixel >> 3;

        SHR  EAX, 3
        ADD  ESI, EAX                   //ImgLine += StartPixel>>3

        AND  CL,  7                     //StartBitInByte = (StartPixel & 7);
        JZ   MoreThanABlackPixel             //CL = StartBitInByte  

        XOR  EAX, EAX

        MOV  DL,  8                      
        SUB  DL,  CL                    //DL= BitInByte = 8 - StartBitInByte;
 
        // FIND SPAN NOW.....................................................
        MOV  AL, [ESI]                  //AL  =(*ImgLine)
        SHL  AL, CL                     //AL  = AL << StartBitInByte
        MOV  AL,  OneRuns[EAX]          //AL = OneRuns[*ImgLine]

        CMP  AL,    DL                  //if(Span < BitInByte) // DOESN'T EXTEND TO EDGE OF BYTE
        JNB  SearchNextByte
        ADD  EAX,  RunLenght
        RET                             //return ( EAX = RunLenght + OneRuns[*ImgLine])

SearchNextByte:

        ADD  RunLenght, EAX
        AND  EDX,   0XFF                
        SUB  Pixel, EDX                 //Pixel     -= BitInByte;
        INC  ESI

MoreThanABlackPixel:      
        MOV  EDX,  Pixel


        //scan a full 32 bits
LoopForBlackPixel32:
        CMP  DX,    32
        JB   LoopForBlackPixel8         //while (Pixel >= 32) {

        MOV  EAX,    [ESI]
        CMP  EAX,    0XFFFFFFFF 
        JNE  LoopForBlackPixel8         //if (*ImgLine != 0XFFFFFFFF)// END OF RUN 

        ADD  RunLenght,  32             //RunLenght  += 32; 
        SUB  DX,         32             //Pixel      -= 32;
        ADD  ESI,         4             //ImgLine    +=  4;
        JMP  LoopForBlackPixel32

        //scan a full byte
LoopForBlackPixel8:
        CMP  DX,    8
        JB   ReturnBlackRunLenght       //while (Pixel >= 8) {

        MOV  AL,    [ESI]
        CMP  AL,    0XFF 
        JE   ContinueNextSearchBlack    //if (*ImgLine != 0XFF)// END OF RUN 

        AND  EAX, 0XFF
        MOV  AL,  OneRuns[EAX]          // AL = OneRuns[*ImgLine]

        ADD  EAX, RunLenght  
        RET                             //return (RunLenght + OneRuns[*ImgLine]);

ContinueNextSearchBlack:        
        ADD  RunLenght,  8              //RunLenght  += 8; 
        SUB  DX,         8              //Pixel      -= 8;
        INC  ESI                        //ImgLine++;
        JMP  LoopForBlackPixel8

ReturnBlackRunLenght:
        MOV EAX, RunLenght
        RET
       }
} // end of SubFindBlackSpan



//++++++ SubStoreCode have been moved to macro. The following SubStoreCode will be removed later
SubStoreCode:
{
//  Write a variable-length code (bit value) to the output stream.
//  The value is assumed to be at most 16 Pixel.
// INPUT PARAMETERS/////////////////
//  BL = CodeLength
//  BH  = Code
//////////////////////////////
// USED //////////////////////
// ECX
// EDX = _msbmask + ECX
//////////////////////////////
// RETURN///////////////////// 
// Nothing

    _asm {

    XOR  EAX,    EAX
    XOR  ECX,    ECX
/*
    while (CodeLength > EncodingBit) {                  
        EncodingByte |= Code >> (CodeLength -= EncodingBit);         
        FlushBits();                
    }
*/

StoreByteCode:
    MOV  CL,     BL
    CMP  CL,     EncodingBit
    JLE  StoreBitCode

    MOV  AL,     BH
    SUB  CL,     EncodingBit
    MOV  BL, CL  
    SHR  AL,     CL 
    OR   EncodingByte, AL 

    //CALL FlushBits
    MOV  AL,    EncodingByte
    MOV  EDI,   EncodedDataBufCurrentPtr
    MOV  [EDI], AL
    INC  EncodedDataBufCurrentPtr              
    MOV  EncodingByte, 0  
    MOV  EncodingBit,  8
    INC  EncodedDataSize          

    JMP  StoreByteCode

StoreBitCode:

/*
    EncodingByte |= (Code & _msbmask[CodeLength]) << (EncodingBit - CodeLength);    
    EncodingBit  -= CodeLength;                      
    if (EncodingBit == 0)                       
        FlushBits();    
*/

    LEA  EDX,    [_msbmask + ECX]   // _msbmask[CodeLength]
    MOV  AL,     [EDX]        
    AND  AL,     BH               // (Code & _msbmask[CodeLength]) 

	SUB  EncodingBit,    CL
    MOV  CL,     EncodingBit        // CL IS EncodingBit
    SHL  AL,     CL 
    OR   EncodingByte, AL 

    OR   CL, CL                     // IS EncodingBit READY TO BE STORED == 0
    JNZ  StoringDone
     
    //CALLFlushBits
    MOV  AL,    EncodingByte
    MOV  EDI,   EncodedDataBufCurrentPtr
    MOV  [EDI], AL
    INC  EncodedDataBufCurrentPtr              
    MOV  EncodingByte, 0  
    MOV  EncodingBit,  8
    INC  EncodedDataSize          

StoringDone:
    RET 
    }
}    // End of SubStoreCode


    
SubPutWhiteSpan:       
{
/////////////////////////////////////////////////////////////////////////////////
//Input Parameters:
//                  ECX = Runlength
//Return Parameters:
//                  Nothing
//Used registers:
//                  EDX = Effective address of [FaxZeroRunLenght] or [ZeroCodes]
//                  BL  = CodeLength
//                  BH  = Code
//                  EAX = Temporary variable
//////////////////////////////////////////////////////////////////////////////////


DWORD   Runlength;
    _asm{

/*
    while (Runlength >= 2624) {
        const TableEntry* te = &CodeWord[63 + (2560 >> 6)];
        PutCode(te);
        Runlength -= te->runlen;
    }
*/
    MOV  Runlength, ECX                 //ECX comes from calling program

    CMP  ECX,  2624
    JB   RunLoWhite

    MOV  BX,  0x1f0c                    //BL = CodeLength = 12, BH = Code = 00011111b

RunHiWhite:

    MacStoreCode(BL,BH,ForWhiteSpanBigRun);

    SUB  Runlength, 2560                // 2560 = FaxOneRunLenght[103]

    CMP  Runlength,  2624
    JAE  RunHiWhite
     
RunLoWhite:    
/*
    if (Runlength >= 64) {
         int RunLen = FaxZeroRunLenght[63 + (Runlength >> 6)];
         StoreCode(FaxZeroCode[RunLen],FaxZeroLenght[RunLen]);

        Runlength -= RunLen;
    }
*/

    CMP  Runlength, 64
    JB   RunEndWhite

    MOV  EAX,  Runlength
    SHR  EAX,  6
    ADD  EAX,  63
    
    XOR  EBX,  EBX
    LEA  EDX,  [FaxZeroRunLenght+EAX * 2] // WORD ALIGN
    MOV  BX,   [EDX]
    SUB  Runlength, EBX                 // Runlength -= te->runlen;

    LEA  EDX,  [G3ZeroCode+EAX*2]
    MOV  BX,   [EDX]                    //BL = CodeLength, BH = Code

    MacStoreCode(BL,BH,ForWhiteSpanMidRun);

RunEndWhite:
    // if Runlength is less than 64

    MOV  EAX, Runlength
    LEA  EDX,  [G3ZeroCode+EAX*2]
    MOV  BX,   [EDX]

    MacStoreCode(BL,BH,ForWhiteSpanRun);
 
    RET
    }
}   //End of SubPutWhiteSpan


SubPutBlackSpan:
{
/////////////////////////////////////////////////////////////////////////////////
//Input Parameters:
//                  ECX = Runlength
//Return Parameters:
//                  Nothing
//Used registers:
//                  EDX = Effective address of [FaxOneRunLenght] or [OneCodes]
//                  BL  = CodeLength
//                  BH  = Code
//                  EAX = Temporary variable
//////////////////////////////////////////////////////////////////////////////////


DWORD  Runlength;

/*
    while (Runlength >= 2624) {
        const TableEntry* te = &CodeWord[63 + (2560 >> 6)];
        PutCode(te);
        Runlength -= te->runlen;
    }
*/
    _asm{

    MOV  Runlength, ECX

    CMP  ECX,  2624
    JB   RunLo

    MOV  BX,  0x1f0c                   //BL = CodeLength = 12, BH = Code = 00011111b

RunHi:
    MacStoreCode(BL,BH,ForBlackSpanBigRun);
    SUB  Runlength, 2560               // Runlength -= te->runlen; ( 2560 = FaxOneRunLenght[103]) 

    CMP  Runlength,  2624
    JAE  RunHi
     
RunLo:    
/*
    if (Runlength >= 64) {
         int RunLen = FaxOneRunLenght[63 + (Runlength >> 6)];
         StoreCode(FaxOneCode[RunLen],FaxOneLenght[RunLen]);

        Runlength -= RunLen;
    }
*/

    CMP  Runlength, 64
    JB   RunEnd

    MOV  EAX,  Runlength
    SHR  EAX,  6
    ADD  EAX,  63
    
    XOR  EBX,  EBX
    LEA  EDX,  [FaxOneRunLenght+EAX * 2] // WORD ALIGN
    MOV  BX,   [EDX]

    SUB  Runlength, EBX                 // Runlength -= te->runlen;

    LEA  EDX,  [G3OneCode+EAX*2]
    MOV  BX,   [EDX]                    //BL = CodeLength , BH = Code

    MacStoreCode(BL,BH,ForBlackSpanMidRun);

RunEnd:
    // if Runlength is less than 64

    MOV  EAX, Runlength

    LEA  EDX,  [G3OneCode+EAX*2]
    MOV  BX,   [EDX]                    //BL = CodeLength , BH = Code

    MacStoreCode(BL,BH,ForBlackSpan);

    RET
    }
 
}  // end of SubPutBlackSpan

#undef PixelColor
}
/////////////////////////////////////////////////////////////////////////////
BOOL  G4Compressor::AddEofBits()
{
    // terminate strip with EOFB 

    StoreCode(0x00, 8);
    StoreCode(0x10, 8);
    StoreCode(0x01, 8);
    StoreCode(0x00, 8);
    return (TRUE);
}
//********* Following codes are not used any more.
//********* Implementation in Standard C too slow
/////////////////////////////////////////////////////////////////////////////
extern "C"{
/////////////////////////////////////////////////////////////////////////////
static int  FindWhiteSpan(PBYTE ImgLine, DWORD StartPixel)
{
// Find a Span of zeros using the supplied
// table.  The ``base'' of the bit string is supplied
// along with the start+end bit indices.
//
//   0 1 2 3 4 5 6 7 8 9 0      1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6  
//  ----------------------.....---------------------------------
//  |*|*|*| | | |*|*|*| |      | | |*|*|*|*| | | | | | |*|*| | |
//  ----------------------.....---------------------------------
//                   |            |   ^
//                   |StartPixel= 12  ^
//                   |StartBitInByte=4|
#if 0 /*
long  Pixel     = PixelsPerLine - StartPixel;
WORD  RunLenght = StartPixel;
BYTE  StartBitInByte, BitInByte, Mask, Span;  

    if(Pixel > 0){ 

       ImgLine += StartPixel >> 3;
       // CHECK PARTIAL BYTE.

       StartBitInByte = (StartPixel & 7);
 
       if (StartBitInByte) {
           BitInByte = 8 - StartBitInByte;

           Mask = PixelToBit[ BitInByte ];
           Span = ZeroRuns
                  [((*ImgLine << StartBitInByte) & Mask)];
        
           if (Span == 8)
              Span = BitInByte;
           else if(Span < BitInByte) // DOESN'T EXTEND TO EDGE OF BYTE
              return (RunLenght + Span);

           RunLenght += Span;
           Pixel     -= BitInByte;
           ImgLine++;
       } 
       // SCAN FULL BYTES FOR ALL 0'S.
       while (Pixel >= 8) {
           if (*ImgLine != 0X00)// END OF RUN 
               return (RunLenght + ZeroRuns[*ImgLine]);
           RunLenght  += 8; 
           Pixel      -= 8;
           ImgLine++;
       }
       return (RunLenght);
    } else 
       return PixelsPerLine;
*/
#endif

long  Pixel     = PixelsPerLine - StartPixel;
WORD  RunLenght = StartPixel;
BYTE  StartBitInByte, BitInByte, Mask, Span;  

    if(Pixel > 0){ 
 
        _asm{

        MOV  EAX,  StartPixel
        SHR  EAX, 3
        MOV  ESI, ImgLine                //ImgLine += StartPixel >> 3;
        ADD  ESI, EAX

        MOV  ECX, StartPixel
        AND  CL,  7                      //StartBitInByte = (StartPixel & 7);
        JZ   MoreThanAPixel              //CL = StartBitInByte  

        MOV  AL,  8                      
        SUB  AL,  CL                     //BitInByte = 8 - StartBitInByte;
        MOV  DL,  AL                     //DL = BitInByte SAVE FOR FUTURE REF

        LEA  EBX, PixelToBit             //Get the Mask from PixelToBit
        XLAT  
        MOV  DH,  AL                     //DH = Mask

        // FIND SPAN NOW.....................................................
        MOV  AL, [ESI]                   //AL  =(*ImgLine)
        SHL  AL, CL                      //AL  = AL << StartBitInByte
        AND  AL, DH                      //AL  = AL & Mask; 
        LEA  EBX, ZeroRuns               //EBX = ZeroRuns
        XLAT                             //AL  =  Span = ZeroRuns [AL];

        CMP  AL,  8                      //if (Span == 8)
        JE   SpanIs8
//ELSE...................................
        CMP  AL, DL                      //ELSE IF(SPAN < BITINBYTE) // DOESN'T EXTEND TO EDGE OF BYTE 
        JNB  SpanIsOk

        AND  EAX, 0XFF
        ADD  RunLenght, AX              
        JMP  ReturnRunLenght             //return (RunLenght + Span);

SpanIs8:
        MOV  AL, DL                      //Span = BitInByte;

SpanIsOk: 
        AND  EAX,   0XFF                 //RunLenght += Span; 
        ADD  RunLenght, AX

        AND  EDX,   0XFF                 //Pixel     -= BitInByte;
        SUB  Pixel, EDX 
        
        INC  ESI
        }
        _asm{
MoreThanAPixel:      
        MOV  EDX,  Pixel

LoopForPixel32:
        CMP  DX,    32
        JB   LoopForPixel8              //while (Pixel >= 32) {

        MOV  EAX,    [ESI]
        CMP  EAX,    0X00000000 
        JNE  LoopForPixel8              //if (*ImgLine != 0X00000000)// END OF RUN 

        ADD  RunLenght,  32              //RunLenght  += 32; 
        SUB  DX,         32              //Pixel      -= 32;
        ADD  ESI,         4              //ImgLine    +=  4;
        JMP  LoopForPixel32

LoopForPixel8:
        CMP  DX,    8
        JB   ReturnRunLenght             //while (Pixel >= 8) {

        MOV  AL,    [ESI]
        CMP  AL,    0
        JE   ContinueNextSearch          //if (*ImgLine != 0X00)// END OF RUN 

        LEA  EBX, ZeroRuns
        XLAT                             //AL =  Span = ZeroRuns[*ImgLine]
        AND  EAX,   0XFF
        ADD  RunLenght,  AX
        JMP  ReturnRunLenght             //return (RunLenght + ZeroRuns[*ImgLine]);

ContinueNextSearch:        
        ADD  RunLenght,  8               //RunLenght  += 8; 
        SUB  DX,         8               //Pixel      -= 8;
        INC  ESI                         //ImgLine++;
        JMP  LoopForPixel8
        }
    } else 
       RunLenght = PixelsPerLine;

ReturnRunLenght:
       return (RunLenght);
}
/////////////////////////////////////////////////////////////////////////////
static int  FindBlackSpan(PBYTE ImgLine, DWORD StartPixel)
{
// Find a Span of ones using the supplied table. The ``base'' of the 
// bit string is supplied along with the start+end bit indices.
//
//   0 1 2 3 4 5 6 7 8 9 0    1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6  
//  -----------------------...--------------------------------
//  |*|*|*| | | |*|*|*| | |... | |*|*|*|*| | | | | | |*|*| | |
//  -----------------------...--------------------------------
//                   |            ^   ^
//                   |StartPixel = 13 ^
//                   |StartBitInByte=5|
#if 0    /*
long  Pixel     = PixelsPerLine - StartPixel;
WORD  RunLenght = StartPixel;
BYTE  StartBitInByte, BitInByte, Mask, Span;  

    if(Pixel > 0){ 

       ImgLine += StartPixel >> 3;

       // CHECK PARTIAL BYTE ON LHS.
       StartBitInByte = (StartPixel & 7);
       if( StartBitInByte ){

           BitInByte = 8 - StartBitInByte;

           Mask = PixelToBit[ BitInByte ];

           Span = OneRuns
                  [((*ImgLine << StartBitInByte) & Mask)];

           RunLenght += Span;
           Pixel     -= BitInByte;
           ImgLine++;

           if(Span < BitInByte) // DOESN'T EXTEND TO EDGE OF BYTE
              return (RunLenght);

       } 
       // SCAN FULL BYTES FOR ALL BLACK'S.
       while (Pixel >= 8) {
           if (*ImgLine != 0XFF)// END OF RUN
               return (RunLenght + OneRuns[*ImgLine]);
           RunLenght += 8;
           Pixel     -= 8;
           ImgLine++; 
       }
       return (RunLenght);
    } else
       return PixelsPerLine;
*/
#endif
long  Pixel     = PixelsPerLine - StartPixel;
WORD  RunLenght = StartPixel;
BYTE  StartBitInByte, BitInByte, Mask, Span;  

    if(Pixel > 0){ 
 
        _asm{

        MOV  EAX,  StartPixel
        SHR  EAX, 3
        MOV  ESI, ImgLine                //ImgLine += StartPixel >> 3;
        ADD  ESI, EAX

        MOV  ECX, StartPixel
        AND  CL,  7                      //StartBitInByte = (StartPixel & 7);
        JZ   MoreThanAPixel              //CL = StartBitInByte  

        MOV  AL,  8                      
        SUB  AL,  CL                     //BitInByte = 8 - StartBitInByte;
        MOV  DL,  AL                     //DL = BitInByte SAVE FOR FUTURE REF

        LEA  EBX, PixelToBit             //GET THE MASK FROM PIXELTOBIT
        XLAT  
        MOV  DH,  AL                     //DH = Mask

        // FIND SPAN NOW.....................................................
        MOV  AL, [ESI]                   //AL  =(*ImgLine)
        SHL  AL, CL                      //AL  = AL << StartBitInByte
        AND  AL, DH                      //AL  = AL & Mask; 
        LEA  EBX, OneRuns                //EBX = OneRuns
        XLAT                             //AL  =  Span = OneRuns [AL];

        AND  EAX,   0XFF                 //RunLenght += Span; 
        ADD  RunLenght, AX

        AND  EDX,   0XFF                 //Pixel     -= BitInByte;
        SUB  Pixel, EDX 
        
        INC  ESI
        
        
        CMP  AL,    DL                   //if(Span < BitInByte) // DOESN'T EXTEND TO EDGE OF BYTE
        JB   ReturnRunLenght             //   return (RunLenght);
        }
        _asm{
MoreThanAPixel:      
        MOV  EDX,  Pixel

LoopForPixel32:
        CMP  DX,    32
        JB   LoopForPixel8              //while (Pixel >= 32) {

        MOV  EAX,    [ESI]
        CMP  EAX,    0XFFFFFFFF 
        JNE  LoopForPixel8              //if (*ImgLine != 0XFFFFFFFF)// END OF RUN 

        ADD  RunLenght,  32              //RunLenght  += 32; 
        SUB  DX,         32              //Pixel      -= 32;
        ADD  ESI,         4              //ImgLine    +=  4;
        JMP  LoopForPixel32

LoopForPixel8:
        CMP  DX,    8
        JB   ReturnRunLenght             //while (Pixel >= 8) {

        MOV  AL,    [ESI]
        CMP  AL,    0XFF 
        JE   ContinueNextSearch          //if (*ImgLine != 0XFF)// END OF RUN 

        LEA  EBX, OneRuns
        XLAT                             //AL =  Span = OneRuns[*ImgLine]
        AND  EAX,   0XFF
        ADD  RunLenght,  AX
        JMP  ReturnRunLenght             //return (RunLenght + OneRuns[*ImgLine]);

ContinueNextSearch:        
        ADD  RunLenght,  8               //RunLenght  += 8; 
        SUB  DX,         8               //Pixel      -= 8;
        INC  ESI                         //ImgLine++;
        JMP  LoopForPixel8
        }
ReturnRunLenght:
       return (RunLenght);
    } else 
       return PixelsPerLine;
}
/////////////////////////////////////////////////////////////////////////////
static void  putspan(DWORD Runlength, const TableEntry* CodeWord)
{
// For horizontal mode, write the sequence of codes that describes the 
// specified runlength of zero's or one's. The appropriate table that holds 
// the make-up and terminating codes is supplied.
/*
    while (Runlength >= 2624) {
        const TableEntry* te = &CodeWord[63 + (2560 >> 6)];
        PutCode(te);
        Runlength -= te->runlen;
    }
*/
    _asm{
    MOV  ESI,  CodeWord
    ADD  ESI,  412           // 103 * sizeof(TableEntry)
    
RunHi:
    MOV  EAX,  Runlength
    CMP  EAX,  2624
    JB   RunLo
      
    PUSH ESI
    CALL PutCode
    ADD  ESP,4

    XOR  EAX, EAX
    MOV  AX,  [ESI+2]      
    SUB  Runlength, EAX      // Runlength -= te->runlen;

    JMP  RunHi
    }
     
RunLo:    
/*
    }
    if (Runlength >= 64) {
        const TableEntry* te = &CodeWord[63 + (Runlength >> 6)];
        PutCode(te);
        Runlength -= te->runlen;
    }
*/
    _asm{
    CMP  Runlength, 64
    JB   RunEnd

    MOV  ESI,  CodeWord
    MOV  EAX,  Runlength
    SHR  EAX,  6
    SHL  EAX,  2

    ADD  ESI,  252           // (63 * sizeof(TableEntry))
    ADD  ESI,  EAX

    XOR  EAX, EAX
    MOV  AX,  [ESI+2]      

    SUB  Runlength, EAX      // Runlength -= te->runlen;

    PUSH ESI
    CALL PutCode
    POP  ESI
    }

RunEnd:
    PutCode(&CodeWord[Runlength]);
}

static void  PutWhiteSpan(DWORD Runlength){

DWORD   WhiteRunLenght;
DWORD   Code, CodeLenght;
//#if 0
    _asm{

    XOR  EAX,  EAX
    LEA  EDX,  [FaxZeroRunLenght+103*2]
    MOV  AX,   [EDX]
    MOV  WhiteRunLenght,  EAX  

    XOR  EBX,  EBX
    LEA  EDX,  [FaxZeroLenght+103]
    MOV  BL,   [EDX]
    MOV  CodeLenght,   EBX

    LEA  EDX,  [FaxZeroCode+103]
    MOV  BL,   [EDX]
    MOV  Code, EBX

RunHi:
    MOV  EAX,  Runlength
    CMP  EAX,  2624
    JB   RunLo
      
    PUSH CodeLenght
    PUSH Code
    CALL StoreCode
    POP  EDX
    POP  EDX

    MOV  EAX,  WhiteRunLenght      
    SUB  Runlength, EAX      // Runlength -= te->runlen;

    JMP  RunHi
    }
     
RunLo:    
/*
    if (Runlength >= 64) {
         int RunLen = FaxOneRunLenght[63 + (Runlength >> 6)];
         StoreCode(FaxOneCode[RunLen],FaxOneLenght[RunLen]);

        Runlength -= RunLen;
    }
*/
    _asm{

    CMP  Runlength, 64
    JB   RunEnd

    MOV  EAX,  Runlength
    SHR  EAX,  6
    ADD  EAX,  63
    
    XOR  EBX,  EBX
    LEA  EDX,  [FaxZeroRunLenght+EAX * 2] // WORD ALIGN
    MOV  BX,   [EDX]
    SUB  Runlength, EBX      // Runlength -= te->runlen;

    LEA  EDX,  [FaxZeroLenght+EAX]
    MOV  BL,   [EDX]
    PUSH EBX

    LEA  EDX,  [FaxZeroCode+EAX]
    MOV  BL,   [EDX]       
    PUSH EBX

    CALL StoreCode
    POP  EDX
    POP  EDX
    }

RunEnd:
    StoreCode(FaxZeroCode[Runlength],FaxZeroLenght[Runlength]);
//#endif
}
/////////////////////////////////////////////////////////////////////////////
static void  PutBlackSpan(DWORD Runlength){
DWORD   BlackRunLenght;
BYTE   Code, CodeLength;
/*
    while (Runlength >= 2624) {
        const TableEntry* te = &CodeWord[63 + (2560 >> 6)];
        PutCode(te);
        Runlength -= te->runlen;
    }
*/
    _asm{

    XOR  EAX,  EAX
    LEA  EDX,  [FaxOneRunLenght+103*2]
    MOV  AX,   [EDX]
    MOV  BlackRunLenght,  EAX  

    XOR  EBX,  EBX
    LEA  EDX,  [FaxOneLenght+103]
    MOV  BL,   [EDX]
    MOV  CodeLength,   BL

    LEA  EDX,  [FaxOneCode+103]
    MOV  BL,   [EDX]
    MOV  Code, BL

RunHi:
    MOV  EAX,  Runlength
    CMP  EAX,  2624
    JB   RunLo
      
    PUSH CodeLength
    PUSH Code
    CALL StoreCode
    POP  DX
    POP  DX

    MOV  EAX,  BlackRunLenght      
    SUB  Runlength, EAX      // Runlength -= te->runlen;

    JMP  RunHi
    }
     
RunLo:    
/*
    if (Runlength >= 64) {
         int RunLen = FaxOneRunLenght[63 + (Runlength >> 6)];
         StoreCode(FaxOneCode[RunLen],FaxOneLenght[RunLen]);

        Runlength -= RunLen;
    }
*/
    _asm{

    CMP  Runlength, 64
    JB   RunEnd

    MOV  EAX,  Runlength
    SHR  EAX,  6
    ADD  EAX,  63
    
    XOR  EBX,  EBX
    LEA  EDX,  [FaxOneRunLenght+EAX * 2] // WORD ALIGN
    MOV  BX,   [EDX]
    SUB  Runlength, EBX      // Runlength -= te->runlen;

    LEA  EDX,  [FaxOneLenght+EAX]
    MOV  BL,   [EDX]
    PUSH EBX

    LEA  EDX,  [FaxOneCode+EAX]
    MOV  BL,   [EDX]       
    PUSH EBX

    CALL StoreCode
    POP  EDX
    POP  EDX

    }

RunEnd:
//    StoreCode(FaxOneCode[Runlength],FaxOneLenght[Runlength]);

_asm{

    MOV  EAX, Runlength
    LEA  EDX,  [FaxOneLenght+EAX]
    MOV  BL,   [EDX]
    PUSH EBX

    LEA  EDX,  [FaxOneCode+EAX]
    MOV  BL,   [EDX]       
    PUSH EBX

    CALL StoreCode
    POP  EDX
    POP  EDX

}  //end _asm
}


static void  PutCode(const TableEntry* CodeMode)
{
//  Write a variable-length code (bit value) to the output stream.
//  The value is assumed to be at most 16 Pixel.

BYTE Code       = CodeMode->code;
BYTE CodeLength = CodeMode->length;

    _asm {

    XOR  ECX,    ECX
/*
    while (CodeLength > EncodingBit) {                  
        EncodingByte |= Code >> (CodeLength -= EncodingBit);         
        FlushBits();                
    }
*/
StoreByteCode:
    MOV  CL,     CodeLength
    CMP  CL,     EncodingBit
    JLE  StoreBitCode

	MOV  AL,     Code
    SUB  CL,     EncodingBit
    MOV  CodeLength, CL  
    SHR  AL,     CL 
    OR   EncodingByte, AL 

    //CALL FlushBits
    MOV  AL,    EncodingByte
    MOV  EDI,   EncodedDataBufCurrentPtr
    MOV  [EDI], AL
    INC  EncodedDataBufCurrentPtr              
    MOV  EncodingByte, 0  
    MOV  EncodingBit,  8
    INC  EncodedDataSize          

    JMP  StoreByteCode

StoreBitCode:
/*
    EncodingByte |= (Code & _msbmask[CodeLength]) << (EncodingBit - CodeLength);    
    EncodingBit  -= CodeLength;                      
    if (EncodingBit == 0)                       
        FlushBits();    
*/
    LEA  ESI,    [_msbmask + ECX] // _msbmask[CodeLength]
    MOV  AL,     [ESI]        
    AND  AL,     Code             // (Code & _msbmask[CodeLength]) 

	SUB  EncodingBit,    CL
    MOV  CL,     EncodingBit      // CL IS EncodingBit
    SHL  AL,     CL 
    OR   EncodingByte, AL 

    OR   CL, CL                   // IS EncodingBit READY TO BE STORED == 0
    JNZ  StoringDone
     
    //CALLFlushBits
    MOV  AL,    EncodingByte
    MOV  EDI,   EncodedDataBufCurrentPtr
    MOV  [EDI], AL
    INC  EncodedDataBufCurrentPtr              
    MOV  EncodingByte, 0  
    MOV  EncodingBit,  8
    INC  EncodedDataSize          

StoringDone:
    }
}
/////////////////////////////////////////////////////////////////////////////
static void  StoreCode(BYTE Code, BYTE CodeLength)
{
//  Write a variable-length code (bit value) to the output stream.

    while (CodeLength > EncodingBit) {                  
        EncodingByte |= Code >> (CodeLength -= EncodingBit);         
        FlushBits();                
    }
    EncodingByte |= (Code & _msbmask[CodeLength]) << (EncodingBit - CodeLength);    
    EncodingBit  -= CodeLength;                      
    if (EncodingBit == 0)                       
        FlushBits();    
}
} // extern "C"
//////////////////////////////////////////////////////////////////////////////////////////
extern "C" {
//////////////////////////////////////////////////////////////////////////////////////////
long __stdcall G4Compress (PBYTE Buffer, DWORD BufLen, DWORD Width, DWORD Height)
{
DLL_THREAD_MANAGEMENT();

G4Compressor  ThisG4Compressor;
DWORD         LinesCompressed, CompressedSize;
DWORD         LinesToConvert = Height; 

   if( Not (ThisG4Compressor.Initialize(Width/8, 4*BufLen) )){
      return 0;
   } else {
      LinesCompressed = ThisG4Compressor.CompressG4( CompressedSize, Buffer, LinesToConvert, FALSE);

      if((LinesCompressed != LinesToConvert) ){
         EchoWarnString(_T("Warning: Missing lines in compressor using uncompressed buffer"));
         return 0;
      }
      if( (BufLen < ((CompressedSize * 8) / 10))){
         EchoWarnString(_T("Warning: Compression insufisant using uncompressed buffer"));
         return 0;
      }

      CopyMemory(Buffer, EncodedDataBuffer, CompressedSize);
      return CompressedSize;
   } 
   return 0;
}
////////////////////////////////////////////////////////////////////////
} // extern "C"
////////////////////////////////////////////////////////////////////////
