// Inflate.c -- zlib interface to Inflate modules
///////////////////////////////////////////////////////////////////////////////////////////////////
#include "DevUnzip.h"
///////////////////////////////////////////////////////////////////////////////////////////////////
/* And'ing with mask[n] masks the lower n bits */
const UInt ProgMemmory _Inflate_Mask [18]={
	 0x0000,
	 0x0001, 0x0003, 0x0007, 0x000f, 0x001f, 0x003f, 0x007f, 0x00ff,
	 0x01ff, 0x03ff, 0x07ff, 0x0fff, 0x1fff, 0x3fff, 0x7fff, 0xffff,
};

PUInt  Inflate_Mask = (PUInt) _Inflate_Mask;
///////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////
Int InflateReset(void)
{
    Zee->Total_in                   = Zee->Total_out = 0;
    ReportErrorStr(Z_NULL, Z_NULL);
    Zee->InternState.CurInflateMode = CUR_INF_METHOD;
    InflateBlocksReset();

    Tracev((stderr, "Inflate: reset\n"));
    return Z_OK;
}
///////////////////////////////////////////////////////////////////////////////////////////////////
Int InflateEnd(void)
{
    InflateBlocksFree();
    Tracev((stderr, "Inflate: WinEnd\n"));
    return Z_OK;
}
///////////////////////////////////////////////////////////////////////////////////////////////////
Int InflateInit(void)
{
Int WinBitsSize       = MAX_WBITS;
// Int WindowByteSize    = ((UInt)1 << WinBitsSize); //  SHOULD BE 32KB; (32768) 
  
    ZMemClear    ((PByte)Zee,  SIZEOF_ZSTREAM);   

    if(WinBitsSize   != 15)
    {
      return Z_STREAM_ERROR;
    }
    Zee->InternState.WinBitsSize             = (UInt)WinBitsSize;

    InflateBlocksNew();
    InflateReset();
    return Z_OK;
}
///////////////////////////////////////////////////////////////////////////////////////////////////
Int Inflate(Int Flush)
{
Int      RetCode;
UInt     CurrentByte;

#define ZNEEDBYTE { if(Zee->Avail_in == 0){ return RetCode; } else { RetCode = Flush; } }
#define ZNEXTBYTE (Zee->Avail_in--, Zee->Total_in++, *Zee->NextInByte++)

    if(Zee->NextInByte == Z_NULL){
       return Z_STREAM_ERROR;
    }
    Flush   = ((Flush == Z_FINISH) ? Z_BUF_ERROR : Z_OK);
    
    RetCode = Z_BUF_ERROR;
    while (1) switch (Zee->InternState.CurInflateMode)
    {
      case CUR_INF_METHOD:
           ZNEEDBYTE
           if(((Zee->InternState.SubMode.Method = ZNEXTBYTE) & 0xf) != Z_DEFLATED)
           {
              Zee->InternState.CurInflateMode = CUR_INF_BAD;
              ReportErrorStr("unknown compression method", Z_BAD_COMPMETHODE);
              Zee->InternState.SubMode.Marker = 5;       /* can't try InflateSync */
              break;
           }
           if((Zee->InternState.SubMode.Method >> 4) + 8 > Zee->InternState.WinBitsSize)
           {
              Zee->InternState.CurInflateMode = CUR_INF_BAD;
              ReportErrorStr("invalid WinSlide size", Z_BAD_WINDOWSIZE);
              Zee->InternState.SubMode.Marker = 5;       /* can't try InflateSync */
              break;
           }
           Zee->InternState.CurInflateMode = CUR_INF_FLAG;
      case CUR_INF_FLAG:
           ZNEEDBYTE
           CurrentByte = ZNEXTBYTE;
           if(((Zee->InternState.SubMode.Method << 8) + CurrentByte) % 31)
           {
              Zee->InternState.CurInflateMode = CUR_INF_BAD;
              ReportErrorStr("incorrect header check", Z_BAD_HEADERCHECK);
              Zee->InternState.SubMode.Marker = 5;       /* can't try InflateSync */
              break;
           }
           Tracev((stderr, "Inflate: zlib header ok\n"));
           if(! (CurrentByte & PRESET_DICT) )
           {
              Zee->InternState.CurInflateMode = CUR_INF_BLOCKS;
              break;
           }
           Zee->InternState.CurInflateMode = CUR_INF_DICT4;
      case CUR_INF_DICT4:
           ZNEEDBYTE
           Zee->InternState.SubMode.CheckVal.need  = (ULong)ZNEXTBYTE << 24;
           Zee->InternState.CurInflateMode = CUR_INF_DICT3;
      case CUR_INF_DICT3:
           ZNEEDBYTE
           Zee->InternState.SubMode.CheckVal.need += (ULong)ZNEXTBYTE << 16;
           Zee->InternState.CurInflateMode = CUR_INF_DICT2;
      case CUR_INF_DICT2:
           ZNEEDBYTE
           Zee->InternState.SubMode.CheckVal.need += (ULong)ZNEXTBYTE << 8;
           Zee->InternState.CurInflateMode = CUR_INF_DICT1;
      case CUR_INF_DICT1:
           ZNEEDBYTE
           Zee->InternState.SubMode.CheckVal.need += (ULong)ZNEXTBYTE;
           Zee->CheckSum = Zee->InternState.SubMode.CheckVal.need;
           Zee->InternState.CurInflateMode = CUR_INF_DICT0;
           return Z_NEED_DICT;
      case CUR_INF_DICT0:
           Zee->InternState.CurInflateMode = CUR_INF_BAD;
           ReportErrorStr("need dictionary", Z_BAD_DICTNEED);
           Zee->InternState.SubMode.Marker = 0;       /* can try InflateSync */
           return Z_STREAM_ERROR;
      case CUR_INF_BLOCKS:
           RetCode = InflateBlocks(RetCode);
           if(RetCode == Z_DATA_ERROR)
           {
              Zee->InternState.CurInflateMode = CUR_INF_BAD;
              Zee->InternState.SubMode.Marker = 0;       /* can try InflateSync */
              break;
           }
           if(RetCode == Z_OK){
              RetCode = Flush;
           }
           if(RetCode != Z_STREAM_END){
              return RetCode;
           }
           RetCode = Flush;
           Zee->InternState.SubMode.CheckVal.was = IBState->OutCheck;
           InflateBlocksReset();
           Zee->InternState.CurInflateMode       = CUR_INF_CHECK4;
      case CUR_INF_CHECK4:
           ZNEEDBYTE
           Zee->InternState.SubMode.CheckVal.need = (ULong)ZNEXTBYTE << 24;
           Zee->InternState.CurInflateMode = CUR_INF_CHECK3;
      case CUR_INF_CHECK3:
           ZNEEDBYTE
           Zee->InternState.SubMode.CheckVal.need += (ULong)ZNEXTBYTE << 16;
           Zee->InternState.CurInflateMode = CUR_INF_CHECK2;
      case CUR_INF_CHECK2:
           ZNEEDBYTE
           Zee->InternState.SubMode.CheckVal.need += (ULong)ZNEXTBYTE << 8;
           Zee->InternState.CurInflateMode = CUR_INF_CHECK1;
      case CUR_INF_CHECK1:
           ZNEEDBYTE
           Zee->InternState.SubMode.CheckVal.need += (ULong)ZNEXTBYTE;

           if(Zee->InternState.SubMode.CheckVal.was != Zee->InternState.SubMode.CheckVal.need)
           {
              Zee->InternState.CurInflateMode = CUR_INF_BAD;
              ReportErrorStr("incorrect data check", Z_BAD_DATACHECK);
              Zee->InternState.SubMode.Marker = 5;       /* can't try InflateSync */
              break;
           }
           Tracev((stderr, "Inflate: zlib check ok\n"));
           Zee->InternState.CurInflateMode = CUR_INF_DONE;
      case CUR_INF_DONE:
           return Z_STREAM_END;
      case CUR_INF_BAD:
           return Z_DATA_ERROR;
      default:
           return Z_STREAM_ERROR;
    }
    return Z_STREAM_ERROR;  
}
///////////////////////////////////////////////////////////////////////////////////////////////////
/* copy as much as possible from the sliding WinSlide to the output area */
Int InflateFlush(Int RetCode)
{
UInt    n;
PByte   PtrNextByte;
PByte   q;

    /* local copies of source and destination pointers */
    PtrNextByte = Zee->NextOutByte;
    q = IBState->WinRead;

    /* compute number of bytes to copy as far as WinEnd of WinSlide */
    n = (UInt)((q <= IBState->WinWrite ? IBState->WinWrite : IBState->WinEnd) - q);
    if(n > Zee->Avail_out) n = Zee->Avail_out;
    if(n && RetCode == Z_BUF_ERROR) RetCode = Z_OK;

    /* update counters */
    Zee->Avail_out -= n;
    Zee->Total_out += n;

    Zee->CheckSum = IBState->OutCheck = CheckSum32(IBState->OutCheck, q, n);

    /* copy as far as WinEnd of WinSlide */
    ZMemCopy(PtrNextByte, q, n);
    PtrNextByte += n;
    q += n;

    /* see if more to copy at beginning of WinSlide */
    if(q == IBState->WinEnd)
    {
       /* wrap pointers */
       q = IBState->WinSlide;
       if(IBState->WinWrite == IBState->WinEnd)
         IBState->WinWrite = IBState->WinSlide;

       /* compute bytes to copy */
       n = (UInt)(IBState->WinWrite - q);
       if(n > Zee->Avail_out) n = Zee->Avail_out;
       if(n && RetCode == Z_BUF_ERROR) RetCode = Z_OK;

       /* update counters */
       Zee->Avail_out -= n;
       Zee->Total_out += n;

       Zee->CheckSum = IBState->OutCheck = CheckSum32(IBState->OutCheck, q, n);

       /* copy */
       ZMemCopy(PtrNextByte, q, n);
       PtrNextByte += n;
       q += n;
    }
    /* update pointers */
    Zee->NextOutByte = PtrNextByte;
    IBState->WinRead = q;

    return RetCode; // DONE
}
///////////////////////////////////////////////////////////////////////////////////////////////////
