// infcodes.c -- process literals and length/distance pairs
///////////////////////////////////////////////////////////////////////////////////////////////////
#include "DevUnzip.h"
///////////////////////////////////////////////////////////////////////////////////////////////////
void InflateCodesReset (UInt LitBits, UInt DistBits, TInflate_Huft *LitTree, TInflate_Huft *DistTree)
{
    IBState->SubMode.Decode.Codes = ICState; 
    ZMemClear((PByte)ICState, SIZEOF_CODESTATE); 
 
    ICState->InfCodeMode  = COD_START;
    ICState->LitBits      = (Byte)LitBits;
    ICState->DistBits     = (Byte)DistBits;
    ICState->LitTree      = LitTree;
    ICState->DistTree     = DistTree;
}
///////////////////////////////////////////////////////////////////////////////////////////////////
Int InflateCodes(Int RetCode)
{
TInflate_Huft *TempHuft;       // TEMPORARY POINTER 
UInt   TempValue;              // TEMPORARY STORAGE 
UInt   Ex_Op;                  // EXTRA BITS OR OPERATION 
ULong  BitBuffer;              // BIT BUFFER 
UInt   BitsInBitBuffer;        // BITS IN BIT BUFFER 
PByte  InputDataPtr;           // INPUT DATA POINTER 
UInt   InputAvalaible;         // BYTES AVAILABLE THERE 
PByte  OutputWindowPtr;        // OUTPUT WINDOW WRITE POINTER 
UInt   WinEndCount;            // BYTES TO END OF WINDOW OR READ POINTER 
PByte  CopyBlk;                // POINTER TO COPY STRINGS FROM 

    LOAD // COPY INPUT/OUTPUT INFORMATION TO LOCALS (UPDATE MACRO RESTORES) 

    while (1) switch (ICState->InfCodeMode)// PROCESS INPUT AND OUTPUT BASED ON CURRENT STATE 
    {                                      // WAITING FOR "I:"=INPUT, "O:"=OUTPUT, "X:"=NOTHING 
    case COD_START:                        // X: SET UP FOR COD_LEN 
#ifndef SLOW
         if(WinEndCount >= 258 && InputAvalaible >= 10)
         {
           UPDATE
           RetCode       = InflateFast(ICState->LitBits, ICState->DistBits, ICState->LitTree, ICState->DistTree);
           LOAD
           if(RetCode   != Z_OK)
           {
             ICState->InfCodeMode = RetCode == Z_STREAM_END ? COD_WASH : COD_BAD;
             break;
           }
         }
#endif   // !SLOW 
         ICState->SubMode.Code.BitsNeeded = ICState->LitBits;
         ICState->SubMode.Code.Tree       = ICState->LitTree;
         ICState->InfCodeMode             = COD_LEN;
    case COD_LEN:  // I: GET LENGTH/LITERAL/EOB NEXT 
         TempValue = ICState->SubMode.Code.BitsNeeded;
         NEEDBITS(TempValue)
         TempHuft  = ICState->SubMode.Code.Tree + ((UInt)BitBuffer & Inflate_Mask[TempValue]);
         DUMPBITS(TempHuft->AsWord.What.Bits)
         Ex_Op     = (UInt)(TempHuft->AsWord.What.Ex_Op);
         if(Ex_Op == 0)                 // LITERAL 
         {
           ICState->SubMode.Lit = TempHuft->More.Base;
           Tracevv((stderr, TempHuft->More.Base >= 0x20 && TempHuft->More.Base < 0x7f ?
                    "inflate:         literal '%c'\n" :
                    "inflate:         literal 0x%02x\n", TempHuft->More.Base));
           ICState->InfCodeMode = COD_LIT;
           break;
         }
         if(Ex_Op & 16)                     // LENGTH 
         {
           ICState->SubMode.Copy.BitsToGet  = Ex_Op & 15;
           ICState->ThisLenght              = TempHuft->More.Base;
           ICState->InfCodeMode             = COD_LENEXT;
           break;
         }
         if((Ex_Op & 64) == 0)              // NEXT TABLE 
         {
           ICState->SubMode.Code.BitsNeeded = Ex_Op;
           ICState->SubMode.Code.Tree       = TempHuft + TempHuft->More.Base;
           break;
         }
         if(Ex_Op & 32)                     // END OF BLOCK 
         {
           Tracevv((stderr, "inflate:         end of block\n"));
           ICState->InfCodeMode = COD_WASH;
           break;
         }
         ICState->InfCodeMode = COD_BAD; // INVALID CODE 
         ReportErrorStr("invalid literal/length code", Z_BAD_DICLENCODE);
         RetCode = Z_DATA_ERROR;
         LEAVE
    case COD_LENEXT:    // I: GETTING LENGTH EXTRA (HAVE BASE) 
         TempValue = ICState->SubMode.Copy.BitsToGet;
         NEEDBITS(TempValue)
         ICState->ThisLenght += (UInt)BitBuffer & Inflate_Mask[TempValue];
         DUMPBITS(TempValue)
         ICState->SubMode.Code.BitsNeeded  = ICState->DistBits;
         ICState->SubMode.Code.Tree        = ICState->DistTree;
         Tracevv((stderr, "inflate:         length %u\n", ICState->ThisLenght));
         ICState->InfCodeMode             = COD_DIST;
    case COD_DIST:      // I: GET DISTANCE NEXT 
         TempValue = ICState->SubMode.Code.BitsNeeded;
         NEEDBITS(TempValue)
         TempHuft = ICState->SubMode.Code.Tree + ((UInt)BitBuffer & Inflate_Mask[TempValue]);
         DUMPBITS(TempHuft->AsWord.What.Bits)
         Ex_Op = (UInt)(TempHuft->AsWord.What.Ex_Op);
         if(Ex_Op & 16) // DISTANCE 
         {
           ICState->SubMode.Copy.BitsToGet = Ex_Op & 15;
           ICState->SubMode.Copy.From = TempHuft->More.Base;
           ICState->InfCodeMode = COD_DISTEXT;
           break;
         }
         if((Ex_Op & 64) == 0)          // NEXT TABLE 
         {
           ICState->SubMode.Code.BitsNeeded = Ex_Op;
           ICState->SubMode.Code.Tree = TempHuft + TempHuft->More.Base;
           break;
         }
         ICState->InfCodeMode = COD_BAD; // INVALID CODE 
         ReportErrorStr("invalid distance code", Z_BAD_DISTCODE);
         RetCode = Z_DATA_ERROR;
         LEAVE
    case COD_DISTEXT:       // I: GETTING DISTANCE EXTRA 
         TempValue = ICState->SubMode.Copy.BitsToGet;
         NEEDBITS(TempValue)
         ICState->SubMode.Copy.From += (UInt)BitBuffer & Inflate_Mask[TempValue];
         DUMPBITS(TempValue)
         Tracevv((stderr, "inflate:         distance %u\n", ICState->SubMode.Copy.From));
         ICState->InfCodeMode = COD_COPY;
    case COD_COPY:          // O: COPYING BYTES IN WINDOW, WAITING FOR SPACE 
         CopyBlk = OutputWindowPtr - ICState->SubMode.Copy.From;
         while (CopyBlk < IBState->WinSlide){                   // MODULO WINDOW SIZE-"WHILE" INSTEAD 
            CopyBlk += IBState->WinEnd - IBState->WinSlide;     // OF "IF" HANDLES INVALID DISTANCES 
         }
         while (ICState->ThisLenght)
         {
           NEEDOUT
           OUTBYTE(*CopyBlk++)
           if(CopyBlk == IBState->WinEnd){
              CopyBlk  = IBState->WinSlide;
           }
           ICState->ThisLenght--;
         }
         ICState->InfCodeMode = COD_START;
         break;
    case COD_LIT:           // O: GOT LITERAL, WAITING FOR OUTPUT SPACE 
         NEEDOUT
         OUTBYTE(ICState->SubMode.Lit)
         ICState->InfCodeMode = COD_START;
         break;
    case COD_WASH:                      // O: GOT EOB, POSSIBLY MORE OUTPUT 
         if(BitsInBitBuffer > 7)        // RETURN UNUSED BYTE, IF ANY 
         {
           Assert(BitsInBitBuffer < 16, "inflate_codes grabbed too many bytes")
           BitsInBitBuffer -= 8;
           InputAvalaible++;
           InputDataPtr--;              // CAN ALWAYS RETURN ONE 
         }
         FLUSH
         if(IBState->WinRead != IBState->WinWrite){
           LEAVE
         }
         ICState->InfCodeMode = COD_END;
    case COD_END:
         RetCode = Z_STREAM_END;
         LEAVE
    case COD_BAD:                       // X: GOT ERROR 
         RetCode = Z_DATA_ERROR;
         LEAVE
    default:
         RetCode = Z_STREAM_ERROR;
         LEAVE
    }
    return Z_STREAM_ERROR;  
}
///////////////////////////////////////////////////////////////////////////////////////////////////
