// inffast.c -- process literals and length/distance pairs fast
///////////////////////////////////////////////////////////////////////////////////////////////////
#include "DevUnzip.h"

// MACROS FOR BIT INPUT WITH NO CHECKING AND FOR RETURNING UNUSED BYTES 
#define GRABBITS(j) {while(BitsInBitBuffer<(j)){BitBuffer |= ((ULong)NEXTBYTE) << BitsInBitBuffer; BitsInBitBuffer += 8;}}
#define UNGRAB {BytesToCopy = Zee->Avail_in - InputAvalaible; BytesToCopy=(BitsInBitBuffer >> 3) < BytesToCopy ? BitsInBitBuffer >> 3 : BytesToCopy; InputAvalaible += BytesToCopy; InputDataPtr -= BytesToCopy; BitsInBitBuffer -= BytesToCopy << 3;}
  
///////////////////////////////////////////////////////////////////////////////////////////////////
//   Called with number of bytes left to write in window at least 258
//   (the maximum string length) and number of input bytes available
//   at least ten.  The ten bytes are six bytes for the longest length/
//   distance pair plus four bytes for overloading the bit buffer. 
///////////////////////////////////////////////////////////////////////////////////////////////////
Int InflateFast (
UInt           LitBit,  
UInt           DistBit, 
TInflate_Huft *LitTree, 
TInflate_Huft *DistTree )        // NEED SEPARATE DECLARATION FOR BORLAND C++ 
{
TInflate_Huft *TempHuftPtr;      // TEMPORARY POINTER 
UInt           Ex_Op;            // EXTRA BITS OR OPERATION 
ULong          BitBuffer;        // BIT BUFFER 
UInt           BitsInBitBuffer;  // BITS IN BIT BUFFER 
PByte          InputDataPtr;     // INPUT DATA POINTER 
UInt           InputAvalaible;   // BYTES AVAILABLE THERE 
PByte          OutputWindowPtr;  // OUTPUT WINDOW WRITE POINTER 
UInt           WinEndCount;      // BYTES TO END OF WINDOW OR READ POINTER 
UInt           LitTreeMask;      // MASK FOR LITERAL/LENGTH TREE
UInt           DistTreeMask;     // MASK FOR DISTANCE TREE 
UInt           BytesToCopy;      // BYTES TO COPY 
UInt           DistBackToCopy;   // DISTANCE BACK TO COPY FROM 
PByte          CopySrcPtr;       // COPY SOURCE POINTER 
   
   LOAD // LOAD INPUT, OUTPUT, BIT VALUES 

   LitTreeMask  = Inflate_Mask[LitBit];
   DistTreeMask = Inflate_Mask[DistBit];

                                     // DO UNTIL NOT ENOUGH INPUT OR OUTPUT SPACE FOR FAST LOOP 
   do {                              // ASSUME CALLED WITH WINENDCOUNT >= 258 && INPUTAVALAIBLE >= 10 
                                     // GET LITERAL/LENGTH CODE 
      GRABBITS(20)                   // MAX BITS FOR LITERAL/LENGTH CODE 
      if((Ex_Op = (TempHuftPtr = LitTree + ((UInt)BitBuffer & LitTreeMask))->AsWord.What.Ex_Op) == 0){
    
         DUMPBITS(TempHuftPtr->AsWord.What.Bits)
         Tracevv((stderr, TempHuftPtr->base >= 0x20 && TempHuftPtr->base < 0x7f ?
                   "inflate:         * literal '%BytesToCopy'\n" :
                   "inflate:         * literal 0x%02x\n", TempHuftPtr->base));
         *OutputWindowPtr++ = (Byte)TempHuftPtr->More.Base;
         WinEndCount--;
         continue;
      }
      do {
         DUMPBITS(TempHuftPtr->AsWord.What.Bits)
         if(Ex_Op & 16){
            // GET EXTRA ASWORD.WHAT.BITS FOR LENGTH 
            Ex_Op &= 15;
            BytesToCopy = TempHuftPtr->More.Base + ((UInt)BitBuffer & Inflate_Mask[Ex_Op]);
            DUMPBITS(Ex_Op)
            Tracevv((stderr, "inflate:         * length %u\n", BytesToCopy));

                                       // DECODE DISTANCE BASE OF BLOCK TO COPY 
            GRABBITS(15);              // MAX ASWORD.WHAT.BITS FOR DISTANCE CODE 
            Ex_Op = (TempHuftPtr = DistTree + ((UInt)BitBuffer & DistTreeMask))->AsWord.What.Ex_Op;
            do {
               DUMPBITS(TempHuftPtr->AsWord.What.Bits)
               if(Ex_Op & 16){
                  // GET EXTRA ASWORD.WHAT.BITS TO ADD TO DISTANCE BASE 
                  Ex_Op &= 15;
                  GRABBITS(Ex_Op)         // get extra AsWord.What.Bits (up to 13) 
                  DistBackToCopy = TempHuftPtr->More.Base + ((UInt)BitBuffer & Inflate_Mask[Ex_Op]);
                  DUMPBITS(Ex_Op)
                  Tracevv((stderr, "inflate:         * distance %u\n", DistBackToCopy));

                  // DO THE COPY 
                  WinEndCount   -= BytesToCopy;
                  CopySrcPtr     = OutputWindowPtr - DistBackToCopy;
                  if(CopySrcPtr  < IBState->WinSlide){                      // WRAP IF NEEDED 
                     do {
                        CopySrcPtr += IBState->WinEnd - IBState->WinSlide; // FORCE POINTER IN WINDOW 
                     } while (CopySrcPtr < IBState->WinSlide);             // COVERS INVALID DISTANCES 
                     Ex_Op = IBState->WinEnd - CopySrcPtr;
                     if(BytesToCopy > Ex_Op){
              
                        BytesToCopy -= Ex_Op;                              // WRAPPED COPY 
                        do {
                            *OutputWindowPtr++ = *CopySrcPtr++;
                        } while (--Ex_Op);
                        CopySrcPtr = IBState->WinSlide;
                        do {
                            *OutputWindowPtr++ = *CopySrcPtr++;
                        } while (--BytesToCopy);
                     } else {                                              // NORMAL COPY 
              
                        *OutputWindowPtr++ = *CopySrcPtr++;  BytesToCopy--;
                        *OutputWindowPtr++ = *CopySrcPtr++;  BytesToCopy--;
                        do {
                            *OutputWindowPtr++ = *CopySrcPtr++;
                        } while (--BytesToCopy);
                     }
                  } else {                                                 // NORMAL COPY 
                     *OutputWindowPtr++ = *CopySrcPtr++;  BytesToCopy--;
                     *OutputWindowPtr++ = *CopySrcPtr++;  BytesToCopy--;
                     do {
                       *OutputWindowPtr++ = *CopySrcPtr++;
                     } while (--BytesToCopy);
                  }
                 break;
               } else if((Ex_Op & 64) == 0){
          
                  TempHuftPtr += TempHuftPtr->More.Base;
                  Ex_Op = (TempHuftPtr += ((UInt)BitBuffer & Inflate_Mask[Ex_Op]))->AsWord.What.Ex_Op;
               } else {

                  ReportErrorStr("invalid distance code", (Z_Current_FastCode = Z_BAD_DISTCODE));
                  UNGRAB
                  UPDATE
                  return Z_DATA_ERROR;
               }
            } while (1);
            break;
         }
         if((Ex_Op & 64) == 0){
      
            TempHuftPtr += TempHuftPtr->More.Base;
            if((Ex_Op = (TempHuftPtr += ((UInt)BitBuffer & Inflate_Mask[Ex_Op]))->AsWord.What.Ex_Op) == 0){
        
               DUMPBITS(TempHuftPtr->AsWord.What.Bits)
               Tracevv((stderr, TempHuftPtr->base >= 0x20 && TempHuftPtr->base < 0x7f ?
                         "inflate:         * literal '%BytesToCopy'\n" :
                         "inflate:         * literal 0x%02x\n", TempHuftPtr->base));
               *OutputWindowPtr++ = (Byte)TempHuftPtr->More.Base;
               WinEndCount       --;
               break;
            }
         } else if(Ex_Op & 32){
            Tracevv((stderr, "inflate:         * end of block\n"));
            UNGRAB
            UPDATE
            return Z_STREAM_END;
         } else {
            ReportErrorStr("invalid literal/length code", (Z_Current_FastCode = Z_BAD_DICLENCODE));
            //Zee->ErrMsg = (Char*)"invalid literal/length code";
            UNGRAB
            UPDATE
            return Z_DATA_ERROR;
         }
      } while (1);
   } while (WinEndCount >= 258 && InputAvalaible >= 10);
   // NOT ENOUGH INPUT OR OUTPUT--RESTORE POINTERS AND RETURN 
   UNGRAB
   UPDATE
   return Z_OK;
}
///////////////////////////////////////////////////////////////////////////////////////////////////
