// G3Compress.cpp
#include "stdafx.h"


////////////////////////////////////////////////////////////////////////
static PBYTE    EncodedDataBufCurrentPtr    = NULL;
static BYTE     G3EncodingByte              = 0;
static BYTE     G3EncodingBit               = 8;
static DWORD    G3EncodedDataSize           = 0;
////////////////////////////////////////////////////////////////////////
// NEW OPTIMIZED FAX INTERPRETTATION...
////////////////////////////////////////////////////////////////////////
static TableEntry OptFaxOneCodes  [(sizeof(TIFFaxOneCodes) / sizeof(TableEntry))];
static TableEntry OptFaxZeroCodes [(sizeof(TIFFaxZeroCodes) / sizeof(TableEntry))];

static BYTE       RelocOne  [256];
static BYTE       RelocZero [256];

////////////////////////////////////////////////////////////////////////
BOOL     InitG3(DWORD BufLen)
{
     EncodedDataBufCurrentPtr    = (PBYTE) malloc ((2 * BufLen) + 32);
     G3EncodingByte              = 0;
     G3EncodingBit               = 8;
     G3EncodedDataSize           = 0;
     return (EncodedDataBufCurrentPtr != NULL);
}
////////////////////////////////////////////////////////////////////////
VOID     ReleaseG3  ()
{
     if(EncodedDataBufCurrentPtr){
        free( EncodedDataBufCurrentPtr );
        EncodedDataBufCurrentPtr = NULL;
     }
     G3EncodingByte              = 0;
     G3EncodingBit               = 8;
     G3EncodedDataSize           = 0;
}
////////////////////////////////////////////////////////////////////////
_inline long CalcOneRuns (PBYTE ImgLine, long  StartPixel,
                                         DWORD PixelsInLine){
long    Pixel         = PixelsInLine - StartPixel;
DWORD   RunLenght     = 0;
BYTE    StartBitInByte, BitInByte, Mask, Span;

    if(Pixel > 0){

       // CHECK PARTIAL BYTE ON LHS.
       StartBitInByte = (StartPixel & 7);
       if( StartBitInByte ){

           BitInByte = 8 - StartBitInByte;

           Mask = PixelToBit[ BitInByte ];
           Span = OneRuns
                  [((*ImgLine << StartBitInByte) & Mask)];

           if(Span < BitInByte){ // DOESN'T EXTEND TO EDGE OF BYTE
              return (RunLenght + Span);
           }
           RunLenght += Span;
           Pixel     -= BitInByte;
           ImgLine++;
       }
       // SCAN FULL BYTES FOR ALL ONE'S.
       while (Pixel > 0) {
           if (*ImgLine != 0XFF){// END OF RUN
               return (RunLenght + OneRuns[*ImgLine]);
           }
           RunLenght += 8;
           Pixel     -= 8;
           ImgLine++;
       }
       return (RunLenght);
    } else {
       return PixelsInLine;
    }
}
////////////////////////////////////////////////////////////////////////
_inline long CalcZeroRuns (PBYTE ImgLine, long  StartPixel,
                                          DWORD PixelsInLine){
long   Pixel     = PixelsInLine - StartPixel;
DWORD  RunLenght = 0;
BYTE   StartBitInByte, BitInByte, Mask, Span;

    if(Pixel > 0){

       StartBitInByte = (StartPixel & 7);

       if (StartBitInByte) {
           BitInByte = 8 - StartBitInByte;

           Mask = PixelToBit[ BitInByte ];
           Span = ZeroRuns
                  [((*ImgLine << StartBitInByte) & Mask)];

           if(Span < BitInByte){ // DOESN'T EXTEND TO EDGE OF BYTE
              return (RunLenght + Span);
           } else {
              Span = BitInByte;
           }
           RunLenght += Span;
           Pixel     -= BitInByte;
           ImgLine++;
       }
       // SCAN FULL BYTES FOR ALL 0'S.
       while (Pixel > 0) {
           if (*ImgLine != 0X00){// END OF RUN
               return (RunLenght + ZeroRuns[*ImgLine]);
           }
           RunLenght  += 8;
           Pixel      -= 8;
           ImgLine++;
       }
       return (RunLenght);
    } else {
       return PixelsInLine;
    }
}
////////////////////////////////////////////////////////////////////////
_inline void FlushBits() // IN C
{
// write G3EncodingByte to buffer after all Pixel in G3EncodingByte has been encoded.

    EncodedDataBufCurrentPtr[G3EncodedDataSize] = G3EncodingByte;
    G3EncodingByte              = 0;
    G3EncodingBit               = 8;
    G3EncodedDataSize++;
}

////////////////////////////////////////////////////////////////////////
_inline void  PutCode(const TableEntry* CodeMode)
//          Write a variable-length code (bit value) to the output stream.
//          The value is assumed to be at most 16 bit.
{
BYTE Code       = CodeMode->code;
BYTE CodeLength = CodeMode->length;

    while (CodeLength > G3EncodingBit) {
        G3EncodingByte |= Code >> (CodeLength -= G3EncodingBit);
        FlushBits();
    }
    G3EncodingByte |= (Code & MaskLeft[CodeLength]) << (G3EncodingBit - CodeLength);
    G3EncodingBit  -= CodeLength;
    if (G3EncodingBit == 0){
        FlushBits();
    }
}
////////////////////////////////////////////////////////////////////////
static void  StoreOneRun(DWORD Runlength)
{
// For horizontal mode, write the sequence of codes that describes the
// specified runlength of zero's or one's. The appropriate table that holds
// the make-up and terminating codes is supplied.

    // THE PALM DOES NOT SUPPORT MORE THAN 640
    /*
    while (Runlength >= 2624) {
        const TableEntry* te = &TIFFaxOneCodes[63 + (2560 >> 6)];
        PutCode(te);
        Runlength -= te->runlen;
    }
    */
    while (Runlength >= 640) {
        const TableEntry* te = &TIFFaxOneCodes[73];
        PutCode(te);
        Runlength -= te->runlen;
    }
    if (Runlength >= 64) {
        const TableEntry* te = &TIFFaxOneCodes[63 + (Runlength >> 6)];
        PutCode(te);
        Runlength -= te->runlen;
    }
    PutCode(&TIFFaxOneCodes[Runlength]);
}
////////////////////////////////////////////////////////////////////////
static void  StoreZeroRun(DWORD Runlength)
{
// For horizontal mode, write the sequence of codes that describes the
// specified runlength of zero's or one's. The appropriate table that holds
// the make-up and terminating codes is supplied.

    // THE PALM DOES NOT SUPPORT MORE THAN 640
    /*
    while (Runlength >= 640) { //  2624) {
        const TableEntry* te = &TIFFaxZeroCodes[63 + (2560 >> 6)];
        PutCode(te);
        Runlength -= te->runlen;
    }
    */
    while (Runlength >= 640) {
        const TableEntry* te = &TIFFaxZeroCodes[73];
        PutCode(te);
        Runlength -= te->runlen;
    }
    while(Runlength >= 64) {
        const TableEntry* te = &TIFFaxZeroCodes[63 + (Runlength >> 6)];
        PutCode(te);
        Runlength -= te->runlen;
    }
    PutCode(&TIFFaxZeroCodes[Runlength]);
}
//////////////////////////////////////////////////////////////////////////////////////////
extern "C" {
//////////////////////////////////////////////////////////////////////////////////////////
long  __stdcall G3Compress (PBYTE Buffer,  DWORD BufLen)
{
DLL_THREAD_MANAGEMENT();

long   WhiteRunPix,      BlackRunPix;
long   PixelsInLine    = BufLen * 8, StorePixelInLine;
long   CurrentPixel    = 0,          StoreCurrentPix;
long   StartPixel      = 0,          StoreStartPix;
PBYTE  ImgLine         = Buffer,     StoreImgLine;
DWORD  CodeLen         = BufLen;

    if( Not InitG3(BufLen)){
       AfxMessageBox(_T("Could not allocate Compression buffer"));
       return 0;
    }
    while((PixelsInLine > 0)&&(G3EncodedDataSize < BufLen)){
/*
#ifdef _DEBUG
       StoreImgLine     = ImgLine;
       StoreStartPix    = StartPixel;
       StoreCurrentPix  = CurrentPixel;
       StorePixelInLine = PixelsInLine;
// RESET:
       ImgLine          = StoreImgLine;
       StartPixel       = StoreStartPix;
       CurrentPixel     = StoreCurrentPix;
       PixelsInLine     = StorePixelInLine;
#endif
*/
       //////////////////////////////////////////////////////////////////////
       WhiteRunPix        = CalcZeroRuns (ImgLine, StartPixel, PixelsInLine);

       PixelsInLine      -= WhiteRunPix;
       CurrentPixel      += WhiteRunPix;
       StartPixel         = CurrentPixel % 8;

       ImgLine            = &Buffer[((CurrentPixel) / 8)];

       // THE PALM DOES NOT SUPPORT MORE THAN 640
       StoreZeroRun         (WhiteRunPix);
       //////////////////////////////////////////////////////////////////////
       BlackRunPix        = CalcOneRuns  (ImgLine, StartPixel, PixelsInLine);
       if(BlackRunPix    == 0){
          if(PixelsInLine > 0){
             AfxMessageBox(_T("Error in Compression buffer"));
          }
          break;
       }
       PixelsInLine      -= BlackRunPix;
       CurrentPixel      += BlackRunPix;
       StartPixel         = CurrentPixel % 8;

       ImgLine            = &Buffer[((CurrentPixel) / 8)];

       // THE PALM DOES NOT SUPPORT MORE THAN 640
       StoreOneRun          (BlackRunPix);
    }
    CodeLen = G3EncodedDataSize;
    if(CodeLen >= BufLen){
       EchoWarnString(_T("Warning: Compression insufisant using uncompressed buffer"));
       CodeLen = 0;
    } else {
       if (G3EncodingBit != 0){
           FlushBits(); CodeLen = G3EncodedDataSize;
       }
       CString Outform;
       Outform.Format( _T("Compression Percentage at: %f%% \r\nPacket Original Size:%d \r\nPacket Reduced Size:%d"),
                       (float)(((float)CodeLen/(float)BufLen)*100.0), BufLen, CodeLen );
       EchoDebugString( Outform);
       CopyMemory(Buffer, EncodedDataBufCurrentPtr,  CodeLen);
    }
    ReleaseG3();
    return CodeLen;
}
////////////////////////////////////////////////////////////////////////
} // extern "C"
////////////////////////////////////////////////////////////////////////
//
////////////////////////////////////////////////////////////////////////
static DWORD CalcZeroBits;
static DWORD CalcOneBits;
static DWORD ZeroBitsRunlenght[74];
static DWORD OneBitsRunlenght[74];

static DWORD FramesCalcZeroBits;
static DWORD FramesCalcOneBits;
static DWORD FramesZeroBitsRunlenght[74];
static DWORD FramesOneBitsRunlenght[74];

////////////////////////////////////////////////////////////////////////
static void  ReportOneRun(DWORD Runlength)
{
// For horizontal mode, write the sequence of codes that describes the
// specified runlength of zero's or one's. The appropriate table that holds
// the make-up and terminating codes is supplied.

    // THE PALM DOES NOT SUPPORT MORE THAN 640
    while (Runlength >= 640) {
        const TableEntry* te = &OptFaxOneCodes[RelocOne[73]];

        OneBitsRunlenght[73]++;
        CalcOneBits += te->length;
        Runlength   -= te->runlen;
    }
    if (Runlength >= 64) {
        const TableEntry* te = &OptFaxOneCodes[RelocOne[63 + (Runlength >> 6)]];

        OneBitsRunlenght[RelocOne[63 + (Runlength >> 6)]]++;
        CalcOneBits += te->length;
        Runlength   -= te->runlen;
    }
    CalcOneBits += OptFaxOneCodes[RelocOne[Runlength]].length;
    OneBitsRunlenght[RelocOne[Runlength]] += 1;
}
////////////////////////////////////////////////////////////////////////
static void  ReportZeroRun(DWORD Runlength)
{
// For horizontal mode, write the sequence of codes that describes the
// specified runlength of zero's or one's. The appropriate table that holds
// the make-up and terminating codes is supplied.
CString Outform;

    // THE PALM DOES NOT SUPPORT MORE THAN 640
    while (Runlength >= 640) {
        const TableEntry* te = &OptFaxZeroCodes[73];

        ZeroBitsRunlenght[RelocZero[73]]++;
        CalcZeroBits += te->length;
        Runlength    -= te->runlen;
    }
    while(Runlength >= 64) {
        const TableEntry* te = &OptFaxZeroCodes[RelocZero[63 + (Runlength >> 6)]];

        ZeroBitsRunlenght[RelocZero[63 + (Runlength >> 6)]]++;
        CalcZeroBits += te->length;
        Runlength    -= te->runlen;
    }
    CalcZeroBits += OptFaxZeroCodes[RelocZero[Runlength]].length;
    ZeroBitsRunlenght[RelocZero[Runlength]] += 1;
}
////////////////////////////////////////////////////////////////////////
//
////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////
void   ClearRLReport          () // RESET VALUE TO ZERO
{
    FramesCalcZeroBits        = 0;
    FramesCalcOneBits         = 0;
    ZeroMemory(FramesZeroBitsRunlenght, sizeof(FramesZeroBitsRunlenght));
    ZeroMemory(FramesOneBitsRunlenght,  sizeof(FramesOneBitsRunlenght));
}
static void InitRLReport ()
{
int     Looper;

    CalcZeroBits        = 0;
    CalcOneBits         = 0;

    ZeroMemory(ZeroBitsRunlenght, sizeof(ZeroBitsRunlenght));
    ZeroMemory(OneBitsRunlenght,  sizeof(OneBitsRunlenght));

    CopyMemory(OptFaxOneCodes,  TIFFaxOneCodes,  sizeof(OptFaxOneCodes));
    CopyMemory(OptFaxZeroCodes, TIFFaxZeroCodes, sizeof(OptFaxZeroCodes));

    for(Looper = 0; Looper < 256; Looper++){
       RelocOne  [Looper] = Looper;
       RelocZero [Looper] = Looper;
    }
}
////////////////////////////////////////////////////////////////////////
static void AddFramesStats ()
{
CString Outform;
int     Looper;

    FramesCalcZeroBits += CalcZeroBits;
    FramesCalcOneBits  += CalcOneBits;

    for(Looper = 0; Looper < 74; Looper++){
       FramesZeroBitsRunlenght[Looper] += ZeroBitsRunlenght[Looper];
    }
    for(Looper = 0; Looper < 74; Looper++){
       FramesOneBitsRunlenght[Looper]  += OneBitsRunlenght[Looper];
    }
}
////////////////////////////////////////////////////////////////////////
void ReportRunLen (PBYTE Buffer,  DWORD BufLen)
{
long    WhiteRunPix,      BlackRunPix;
long    PixelsInLine    = BufLen * 8;
long    CurrentPixel    = 0;
long    StartPixel      = 0;
PBYTE   ImgLine         = Buffer;
DWORD   CodeLen         = BufLen;

    InitRLReport ();

    while((PixelsInLine > 0)){
       //////////////////////////////////////////////////////////////////////
       WhiteRunPix        = CalcZeroRuns (ImgLine, StartPixel, PixelsInLine);

       PixelsInLine      -= WhiteRunPix;
       CurrentPixel      += WhiteRunPix;
       StartPixel         = CurrentPixel % 8;

       ImgLine            = &Buffer[((CurrentPixel) / 8)];

       // REPORT WHITE RUNS
       ReportZeroRun        (WhiteRunPix);

       //////////////////////////////////////////////////////////////////////
       BlackRunPix        = CalcOneRuns  (ImgLine, StartPixel, PixelsInLine);
       if(BlackRunPix    == 0){
          if(PixelsInLine > 0){
             AfxMessageBox(_T("Error in Compression buffer"));
          }
          break;
       }
       PixelsInLine      -= BlackRunPix;
       CurrentPixel      += BlackRunPix;
       StartPixel         = CurrentPixel % 8;

       ImgLine            = &Buffer[((CurrentPixel) / 8)];

       // REPORT BLACK RUNS
       ReportOneRun         (BlackRunPix);
    }
    AddFramesStats ();
    return ;
}
////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////
void ReportPageStats ()
{
CString Outform;
int     Looper;

    Outform.Format( _T("Zero Bits Count: \t%d\tPerFrames: \t%d\n"), CalcZeroBits, FramesCalcZeroBits);
    OutputDebugString( Outform);
    Outform.Format( _T("One Bits Count: \t%d\tPerFrames: \t%d\n"),  CalcOneBits,  FramesCalcOneBits);
    OutputDebugString( Outform);

    for(Looper = 0; Looper < 74; Looper++){
       Outform.Format    (_T("ZeroBits Runlenght \t%d \tCount: \t%d\tPerFrames: \t%d\n"),
                          Looper,
                          ZeroBitsRunlenght[Looper],
                          FramesZeroBitsRunlenght[Looper]);
       OutputDebugString (Outform);
    }
    for(Looper = 0; Looper < 74; Looper++){
       Outform.Format    (_T("OneBits Runlenght \t%d \tCount: \t%d\tPerFrames: \t%d\n"),
                          Looper,
                          OneBitsRunlenght[Looper],
                          FramesOneBitsRunlenght[Looper]);
       OutputDebugString (Outform);
    }
}
////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////
void ReportPageRunLen (PBYTE Buffer,  DWORD BufLen)
{
long    WhiteRunPix,      BlackRunPix;
long    PixelsInLine    = BufLen * 8;
long    CurrentPixel    = 0;
long    StartPixel      = 0;
PBYTE   ImgLine         = Buffer;
DWORD   CodeLen         = BufLen;

    InitRLReport ();

    while((PixelsInLine > 0)){
       //////////////////////////////////////////////////////////////////////
       WhiteRunPix        = CalcZeroRuns (ImgLine, StartPixel, PixelsInLine);

       PixelsInLine      -= WhiteRunPix;
       CurrentPixel      += WhiteRunPix;
       StartPixel         = CurrentPixel % 8;

       ImgLine            = &Buffer[((CurrentPixel) / 8)];

       // REPORT WHITE RUNS
       ReportZeroRun        (WhiteRunPix);

       //////////////////////////////////////////////////////////////////////
       BlackRunPix        = CalcOneRuns  (ImgLine, StartPixel, PixelsInLine);
       if(BlackRunPix    == 0){
          if(PixelsInLine > 0){
             AfxMessageBox(_T("Error in Compression buffer"));
          }
          break;
       }
       PixelsInLine      -= BlackRunPix;
       CurrentPixel      += BlackRunPix;
       StartPixel         = CurrentPixel % 8;

       ImgLine            = &Buffer[((CurrentPixel) / 8)];

       // REPORT BLACK RUNS
       ReportOneRun         (BlackRunPix);
    }
    ReportPageStats ();
    return;
}
