#include "stdafx.h"


#define MsgDiag(St, Mask) MessageBox( 0, St, _T("GCompress"), Mask)

CG3Decode::CG3Decode()
{
	m_WhiteRoot				= NULL;
	m_BlackRoot				= NULL;
	m_CurrentTreePointer	= NULL;
	m_IsWhite				= true;
	m_G4byteToStart			= 0;
	m_G4bitToStart			= 0;
	m_LineOffset			= 0;
	m_CurrentBinLine		= NULL;
	m_DestImgWidthInBytes	= 0;
    m_FirstEOL              = true;
}
bool CG3Decode::Initialize(DWORD DestImgWidthInBytes)
{
	if (! m_G3Tree.Initialize()){
		return FALSE;
	}
    m_WhiteRoot				= m_G3Tree.GetWhiteTree();
	m_BlackRoot				= m_G3Tree.GetBlackTree();
	m_CurrentTreePointer	= m_WhiteRoot;
	m_IsWhite				= true;
	m_G4byteToStart			= 0;
	m_G4bitToStart			= 0;
	m_LineOffset			= 0;
	m_DestImgWidthInBytes	= 0;
    m_FirstEOL              = true;

    if (m_CurrentBinLine){
       GlobalFree(m_CurrentBinLine);
       m_CurrentBinLine = NULL;
    }
/*
	if ((m_CurrentBinLine = (PBYTE)GlobalAlloc(GPTR, DestImgWidthInBytes*2))==NULL){
		return false;
    }
*/
	if ((m_CurrentBinLine = (PBYTE) new BYTE [DestImgWidthInBytes*2] ) == NULL){
		return false;
    }
	m_DestImgWidthInBytes = DestImgWidthInBytes;
	return true;
}
bool CG3Decode::Reset()
{
	m_CurrentTreePointer	= m_WhiteRoot;
	m_IsWhite				= true;
	m_G4byteToStart			= 0;
	m_G4bitToStart			= 0;
	m_LineOffset			= 0;
    m_FirstEOL              = true;

    if (Not m_CurrentBinLine){
       return FALSE;
    }
    ZeroMemory(m_CurrentBinLine, m_DestImgWidthInBytes);

	return true;
}
CG3Decode::~CG3Decode()
{
    if (m_CurrentBinLine) GlobalFree(m_CurrentBinLine);
    m_CurrentBinLine = NULL;
}
bool CG3Decode::UncompressG3ToBinary(PBYTE	SrcG3Buffer,
						             DWORD  SrcG3BufSize, //Must be the exact quantity of G4 data
						             PBYTE  DestBinBuffer,
						             DWORD  DestBinBufSize,
						             PDWORD LinesUncompressed,
						             PBOOL  CompressedBufferDone){

DWORD		bytelooper, DestImgHeight;
DWORD		ByteEndOffset, BitEndOffset, ByteStartOffset, BitStartOffset;
BYTE		bitlooper, CurrentByte;
long		RunLength, SizeToFill;

	if ((m_CurrentBinLine    == NULL)
    || (m_CurrentTreePointer == NULL)
    || (SrcG3BufSize         <= m_G4byteToStart)){
		return false;
    }
	DestImgHeight		 = DestBinBufSize / m_DestImgWidthInBytes;
	*LinesUncompressed   = 0;
	SrcG3Buffer         += m_G4byteToStart;

	for (bytelooper = m_G4byteToStart; bytelooper < SrcG3BufSize; bytelooper++){

        CurrentByte = *SrcG3Buffer++;

		for (bitlooper = m_G4bitToStart; bitlooper < 8; bitlooper++){
			if (CurrentByte & DecodeMask [bitlooper]){
				m_CurrentTreePointer = m_CurrentTreePointer->One;
			} else {
				m_CurrentTreePointer = m_CurrentTreePointer->Zero;
			}
			if (m_CurrentTreePointer == NULL){
				return false;
			}
            if (m_CurrentTreePointer->RunLength     != G4CODE_INVALID){

				if (m_CurrentTreePointer->RunLength == G4CODE_EOL){

					m_CurrentTreePointer	= m_WhiteRoot;
					m_IsWhite				= true;
					m_LineOffset			= 0;
                    if (m_FirstEOL){
                        m_FirstEOL = false;
                    } else {
					    CopyMemory(DestBinBuffer, m_CurrentBinLine, m_DestImgWidthInBytes);
					    DestBinBuffer += m_DestImgWidthInBytes;
					    ZeroMemory(m_CurrentBinLine, m_DestImgWidthInBytes);
					    (*LinesUncompressed)++;
					    if ((*LinesUncompressed) == DestImgHeight){
						    *CompressedBufferDone = 0;
						    m_G4bitToStart = bitlooper + 1;
						    if (m_G4bitToStart   == 8){
							    m_G4bitToStart    = 0;
							    m_G4byteToStart   = bytelooper + 1;
						    } else {
							    m_G4byteToStart   = bytelooper;
						    }
                            return true;
					    }
                    }
				} else {
					RunLength = m_CurrentTreePointer->RunLength;
					if ((RunLength) && (Not m_IsWhite)){
						BitStartOffset	= m_LineOffset                & 0x7;
						ByteStartOffset	= m_LineOffset               >>  3;
						BitEndOffset	= (m_LineOffset + RunLength)  & 0x7;
						ByteEndOffset	= (m_LineOffset + RunLength) >>  3;
						SizeToFill		= ByteEndOffset - ByteStartOffset - 1;
                        if (SizeToFill > 0){
                            /////////////////////////////////////////////////////////////
						    if((ByteStartOffset + 1 + SizeToFill) > m_DestImgWidthInBytes){
                               SizeToFill = m_DestImgWidthInBytes - (ByteStartOffset + 1);
                            }
                            /////////////////////////////////////////////////////////////
							FillMemory(m_CurrentBinLine + ByteStartOffset + 1, SizeToFill, 0xff);
                        }
						if (ByteStartOffset == ByteEndOffset){
							*(m_CurrentBinLine + ByteStartOffset) |= StartByteMask [BitStartOffset] & EndByteMask [BitEndOffset];
						} else {
							*(m_CurrentBinLine + ByteStartOffset) |= StartByteMask [BitStartOffset];
							*(m_CurrentBinLine + ByteEndOffset)   |= EndByteMask   [BitEndOffset];
						}
					}
					if (RunLength < 64) m_IsWhite = ! m_IsWhite;
					m_CurrentTreePointer = (m_IsWhite) ? m_WhiteRoot : m_BlackRoot;
					m_LineOffset += RunLength;
				}
			}
		}
		m_G4bitToStart    = 0;
	}
	*CompressedBufferDone = 1;
	m_G4byteToStart       = 0;
	m_G4bitToStart        = 0;
	return true;
}
//////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////
CG4Decode::CG4Decode()
{
	m_WhiteRoot				= NULL;
	m_BlackRoot				= NULL;
	m_CurrentTreePointer	= NULL;
	m_IsWhite				= true;
	m_G4byteToStart			= 0;
	m_G4bitToStart			= 0;
	m_LineOffset			= 0;
	m_CurrentBinLine		= NULL;
	m_DestImgWidthInBytes	= 0;
	m_ModeCodeRoot			= NULL;
	m_ReferenceRuns			= NULL;
	m_CodingRuns			= NULL;
}
bool CG4Decode::Initialize(DWORD DestImgWidthInBytes)
{
	if (Not m_G4Tree.Initialize()){
		return FALSE;
    }
/*
	if (m_CodingRuns){
       GlobalFree(m_CodingRuns);
	}
    if ((m_CodingRuns = (int*)GlobalAlloc(GPTR, DestImgWidthInBytes * 8 * sizeof(int) + 256)) == NULL){
	   return FALSE;
	}
*/
	if (m_CodingRuns){
       delete [] m_CodingRuns; m_CodingRuns = NULL;
	}
	if ((m_CodingRuns = (PINT) new BYTE [(DestImgWidthInBytes * 8 * sizeof(int) + 256)] ) == NULL){
		return false;
    }
/*
    if (m_ReferenceRuns){
       GlobalFree(m_ReferenceRuns);
	}
    if ((m_ReferenceRuns = (int*)GlobalAlloc(GPTR, DestImgWidthInBytes * 8 * sizeof(int) + 256 )) == NULL){
	   return FALSE;
	}
*/
	if (m_ReferenceRuns){
       delete [] m_ReferenceRuns; m_ReferenceRuns = NULL;
	}
	if ((m_ReferenceRuns = (PINT) new BYTE [(DestImgWidthInBytes * 8 * sizeof(int) + 256)] ) == NULL){
		return false;
    }

    m_WhiteRoot				= m_G4Tree.GetWhiteTree();
	m_BlackRoot				= m_G4Tree.GetBlackTree();
	m_ModeCodeRoot			= m_G4Tree.GetModeCodeTree();

	m_CurrentTreePointer	= m_ModeCodeRoot;
	m_IsWhite				= true;
	m_G4byteToStart			= 0;
	m_G4bitToStart			= 0;
	m_LineOffset			= 0;
	m_DestImgWidthInBytes	= 0;
/*
	if (m_CurrentBinLine){
       GlobalFree(m_CurrentBinLine);
       m_CurrentBinLine     = NULL;
	}
    if ((m_CurrentBinLine = (PBYTE)GlobalAlloc(GPTR, DestImgWidthInBytes * 2 + 256 ))==NULL){
	   return false;
    }
*/
	if (m_CurrentBinLine){
       delete [] m_CurrentBinLine; m_CurrentBinLine     = NULL;
	}
	if ((m_CurrentBinLine = (PBYTE) new BYTE [(DestImgWidthInBytes * 2 + 256)] ) == NULL){
		return false;
    }

	m_DestImgWidthInBytes = DestImgWidthInBytes;
	m_LookForCode			= true;
	m_IsFirstRun			= true;
	m_CodingIndex			= 1;
	m_ReferenceIndex		= 1;
	m_CodingRuns[0]			= -1;
	m_ReferenceRuns[0]		= -1;
	m_ReferenceRuns[1]		= m_DestImgWidthInBytes * 8;
	m_ReferenceRuns[2]		= m_DestImgWidthInBytes * 8;
	return true;
}
bool CG4Decode::Reset()
{
	if (Not m_CodingRuns) {
       return FALSE;
    }
    ZeroMemory(m_CodingRuns, m_DestImgWidthInBytes * 8 * sizeof(int) + 256);

	if (Not m_ReferenceRuns){
       return FALSE;
    }
    ZeroMemory(m_ReferenceRuns, m_DestImgWidthInBytes * 8 * sizeof(int) + 256);

	if (Not m_CurrentBinLine){
       return FALSE;
    }
    ZeroMemory(m_CurrentBinLine, m_DestImgWidthInBytes);

	m_CurrentTreePointer	= m_ModeCodeRoot;
	m_IsWhite				= true;
	m_G4byteToStart			= 0;
	m_G4bitToStart			= 0;
	m_LineOffset			= 0;
	m_LookForCode			= true;
	m_IsFirstRun			= true;
	m_CodingIndex			= 1;
	m_ReferenceIndex		= 1;
	m_CodingRuns[0]			= -1;
	m_ReferenceRuns[0]		= -1;
	m_ReferenceRuns[1]		= m_DestImgWidthInBytes * 8;
	m_ReferenceRuns[2]		= m_DestImgWidthInBytes * 8;
	return true;
}
CG4Decode::~CG4Decode()
{
	if (m_CodingRuns){
       delete [] m_CodingRuns; m_CodingRuns = NULL;
	}
	if (m_ReferenceRuns){
       delete [] m_ReferenceRuns; m_ReferenceRuns = NULL;
	}
	if (m_CurrentBinLine){
       delete [] m_CurrentBinLine; m_CurrentBinLine     = NULL;
	}
/*
    if (m_CurrentBinLine){
       GlobalFree(m_CurrentBinLine);
    }
	m_CurrentBinLine = NULL;

	if (m_CodingRuns){
       GlobalFree(m_CodingRuns);
	}
    m_CodingRuns = NULL;
	if (m_ReferenceRuns){
       GlobalFree(m_ReferenceRuns);
    }
	m_ReferenceRuns = NULL;
*/
}
bool CG4Decode::UncompressG4ToBinary(PBYTE	SrcG4Buffer,
					                 DWORD	SrcG4BufSize,
					                 PBYTE  DestBinBuffer,
					                 DWORD  DestBinBufSize,
					                 PDWORD LinesUncompressed,
					                 PBOOL  CompressedBufferDone){
DWORD		bytelooper, DestImgHeight;
BYTE		bitlooper, CurrentByte;
long		RunLength;
short		VerticalOffset;
int*		pTemp;
DWORD		DestImgSizeInPixels = m_DestImgWidthInBytes * 8;

    if ((m_CurrentBinLine    == NULL)
    || (m_CurrentTreePointer == NULL)
    || (SrcG4BufSize         <= m_G4byteToStart)){
		return false;
	}
    DestImgHeight		= DestBinBufSize / m_DestImgWidthInBytes;
	*LinesUncompressed	= 0;
	SrcG4Buffer        += m_G4byteToStart;

	for (bytelooper = m_G4byteToStart; bytelooper < SrcG4BufSize; bytelooper++){
		CurrentByte = *SrcG4Buffer++;

		for (bitlooper = m_G4bitToStart; bitlooper < 8; bitlooper++){

            if (CurrentByte & DecodeMask [bitlooper]){
				m_CurrentTreePointer = m_CurrentTreePointer->One;
			} else {
				m_CurrentTreePointer = m_CurrentTreePointer->Zero;
			}
			if (m_CurrentTreePointer == NULL){
				return true;
			}
            if (m_CurrentTreePointer->RunLength != G4CODE_INVALID){

                if (m_LookForCode){

                	if (m_LineOffset >= DestImgSizeInPixels){

						m_CodingRuns [m_CodingIndex]     = m_CodingRuns [m_CodingIndex - 1];
						m_CodingRuns [m_CodingIndex + 1] = m_CodingRuns [m_CodingIndex];

						m_LineOffset			= 0;

						pTemp					= m_ReferenceRuns;
						m_ReferenceRuns			= m_CodingRuns;
						m_CodingRuns			= pTemp;

                        m_CodingIndex			= 1;
						m_ReferenceIndex		= 1;

						m_IsWhite				= true;
						m_LookForCode			= true;
                        m_IsFirstRun			= true;

                        CopyMemory(DestBinBuffer, m_CurrentBinLine, m_DestImgWidthInBytes);

                        DestBinBuffer += m_DestImgWidthInBytes;

                        ZeroMemory(m_CurrentBinLine, m_DestImgWidthInBytes);

                        (*LinesUncompressed)++;

						if ((*LinesUncompressed) == DestImgHeight){
							*CompressedBufferDone = 0;
							m_G4bitToStart        = bitlooper + 1;
							if (m_G4bitToStart   == 8){
								m_G4bitToStart    = 0;
								m_G4byteToStart   = bytelooper + 1;
							} else {
								m_G4byteToStart   = bytelooper;
							}
                            return true;
						}
					}
					switch(m_CurrentTreePointer->RunLength){
					case G4MODE_PASS:
						 if (Not m_IsWhite){
							 FillCurrentBinLine(m_LineOffset, m_ReferenceRuns [m_ReferenceIndex + 1]);
						 }
						 m_LineOffset         = m_ReferenceRuns [m_ReferenceIndex + 1];
						 m_ReferenceIndex    += 2;
						 m_CurrentTreePointer = m_ModeCodeRoot;
						 break;
					case G4MODE_V0:
					case G4MODE_VR1:
					case G4MODE_VR2:
					case G4MODE_VR3:
						 VerticalOffset = m_CurrentTreePointer->RunLength - G4MODE_V0;
						 if (Not m_IsWhite){
							 FillCurrentBinLine(m_LineOffset, m_ReferenceRuns [m_ReferenceIndex] + VerticalOffset);
						 }
						 m_IsWhite      = ! m_IsWhite;
						 m_LineOffset   = m_ReferenceRuns [m_ReferenceIndex] + VerticalOffset;
						 m_CodingRuns [m_CodingIndex] = m_LineOffset;
						 m_CodingIndex    ++;
						 m_ReferenceIndex ++;
						 if (m_LineOffset     == m_ReferenceRuns [m_ReferenceIndex]){
							 m_ReferenceIndex += 2;
						 }
                         m_CurrentTreePointer  =  m_ModeCodeRoot;
						 break;
					case G4MODE_VL1:
					case G4MODE_VL2:
					case G4MODE_VL3:
						 VerticalOffset = m_CurrentTreePointer->RunLength - G4_VL_OFFSET;
						 if (Not m_IsWhite){
							 FillCurrentBinLine(m_LineOffset, m_ReferenceRuns [m_ReferenceIndex] - VerticalOffset);
						 }
						 m_IsWhite    = ! m_IsWhite;
						 m_LineOffset = m_ReferenceRuns [m_ReferenceIndex] - VerticalOffset;
						 if ((int)m_LineOffset < 0){
							 m_LineOffset = 0;
                         }
						 m_CodingRuns [m_CodingIndex] = m_LineOffset;
						 m_CodingIndex++;
						 if ((VerticalOffset   > 1)
                         && ((int)m_LineOffset < m_ReferenceRuns[m_ReferenceIndex - 1])){
							 m_ReferenceIndex--;
						 } else {
							 m_ReferenceIndex++;
						 }
                         m_CurrentTreePointer = m_ModeCodeRoot;
						 break;
					case G4MODE_HORIZ:
						 m_LookForCode        = false;
						 m_CurrentTreePointer = (m_IsWhite) ? m_WhiteRoot : m_BlackRoot;
						 break;
					case G4CODE_EOL:
                         int whatever = 0;
						 break;
					}
				} else {
					RunLength = m_CurrentTreePointer->RunLength;
					if ((RunLength) && (Not m_IsWhite)){
						FillCurrentBinLine(m_LineOffset, m_LineOffset + RunLength);
					}
					m_LineOffset += RunLength;
					if (RunLength    < 64){
						m_IsWhite    = ! m_IsWhite;
						m_IsFirstRun = ! m_IsFirstRun;
						m_CodingRuns [m_CodingIndex] = m_LineOffset;
						m_CodingIndex++;
					}
					if ((RunLength < 64)
                    && (m_IsFirstRun)){
						m_LookForCode = true;
						m_CurrentTreePointer = m_ModeCodeRoot;
						while (m_ReferenceRuns [m_ReferenceIndex] <= m_LineOffset){
							m_ReferenceIndex += 2;
						}
					} else {
						m_CurrentTreePointer = (m_IsWhite) ? m_WhiteRoot : m_BlackRoot;
					}
				}
			}
		}
		m_G4bitToStart    = 0;
	}
	*CompressedBufferDone = 1;
	m_G4byteToStart       = 0;
	m_G4bitToStart        = 0;
	return true;
}
////////////////////////////////////////////////////////////////////////////////////////////////////////
_inline void CG4Decode::FillCurrentBinLine(DWORD Begin, DWORD End){
DWORD		ByteEndOffset, BitEndOffset, ByteStartOffset, BitStartOffset;
long		SizeToFill;

    if (/*(End<=Begin)||*/
		((int)End < 0)){
		return;
	}
    BitStartOffset	= Begin  & 0x7;
	ByteStartOffset	= Begin >>   3;
	BitEndOffset	= End    & 0x7;
	ByteEndOffset	= End   >>   3;
	SizeToFill		= ByteEndOffset - ByteStartOffset - 1;

    if (SizeToFill >= 0){ // IF THE SIZE IS GREATER THAN A BYTE...
		FillMemory(m_CurrentBinLine + ByteStartOffset + 1, SizeToFill, 0xff);
		*(m_CurrentBinLine + ByteStartOffset) |= StartByteMask [BitStartOffset];
		*(m_CurrentBinLine + ByteEndOffset)   |= EndByteMask   [BitEndOffset];
	//if (ByteStartOffset == ByteEndOffset)
	} else {
		*(m_CurrentBinLine + ByteStartOffset) |= StartByteMask [BitStartOffset] & EndByteMask [BitEndOffset];
	}
}
////////////////////////////////////////////////////////////////////////////////////////////////////////
// BRIGE TO CLASS...
////////////////////////////////////////////////////////////////////////////////////////////////////////
extern "C" {
////////////////////////////////////////////////////////////////////////////////////////////////////////

HANDLE __stdcall G3StartReaderEngine (){
DLL_THREAD_MANAGEMENT();
   return (HANDLE)(new CG3Decode);
}
HANDLE __stdcall G4StartReaderEngine (){
DLL_THREAD_MANAGEMENT();
   return (HANDLE)(new CG4Decode);
}
////////////////////////////////////////////////////////////////////////////////////////////////////////
void __stdcall G3ReleaseReaderEngine (HANDLE G3){
DLL_THREAD_MANAGEMENT();
   if(G3){
      if(Not IsBadWritePtr(G3, sizeof(CG3Decode))){
         delete (CG3Decode *)G3;
         return;
      }
   }
   MsgDiag(_T("Fail to Free G3 Engine"), MB_ICONSTOP|MB_TOPMOST);
}
void __stdcall G4ReleaseReaderEngine (HANDLE G4){
DLL_THREAD_MANAGEMENT();
   if(G4){
      if(Not IsBadWritePtr(G4, sizeof(CG4Decode))){
         delete (CG4Decode *)G4;
         return;
      }
   }
   MsgDiag(_T("Fail to Free G4 Engine"), MB_ICONSTOP|MB_TOPMOST);
}
////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////
BOOL __stdcall G3ReaderInitialize (HANDLE G3, DWORD DestImgWidthInBytes){
DLL_THREAD_MANAGEMENT();
   if(G3){
      if(Not IsBadWritePtr(G3, sizeof(CG3Decode))){
         return ((CG3Decode *)G3)->Initialize(DestImgWidthInBytes);
      }
   }
   MsgDiag(_T("Bad G3 Engine"), MB_ICONSTOP|MB_TOPMOST);
   return FALSE;
}
BOOL __stdcall G4ReaderInitialize (HANDLE G4, DWORD DestImgWidthInBytes){
DLL_THREAD_MANAGEMENT();
   if(G4){
      if(Not IsBadWritePtr(G4, sizeof(CG4Decode))){
         return ((CG4Decode *)G4)->Initialize(DestImgWidthInBytes);
      }
   }
   MsgDiag(_T("Bad G4 Engine"), MB_ICONSTOP|MB_TOPMOST);
   return FALSE;
}
////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////
BOOL __stdcall G3ReaderReset (HANDLE G3){
DLL_THREAD_MANAGEMENT();
   if(G3){
      if(Not IsBadWritePtr(G3, sizeof(CG3Decode))){
         return ((CG3Decode *)G3)->Reset();
      }
   }
   MsgDiag(_T("Bad G3 Engine"), MB_ICONSTOP|MB_TOPMOST);
   return FALSE;
}
BOOL __stdcall G4ReaderReset (HANDLE G4){
DLL_THREAD_MANAGEMENT();
   if(G4){
      if(Not IsBadWritePtr(G4, sizeof(CG4Decode))){
         return ((CG4Decode *)G4)->Reset();
      }
   }
   MsgDiag(_T("Bad G4 Engine"), MB_ICONSTOP|MB_TOPMOST);
   return FALSE;
}
////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////
BOOL __stdcall G3ReaderUncompress (HANDLE G3, PBYTE  SrcG3Buffer,
					                       DWORD  SrcG3BufSize,
					                       PBYTE  DestBinBuffer,
					                       DWORD  DestBinBufSize,
					                       PDWORD LinesUncompressed,
					                       PBOOL  CompressedBufferDone){
DLL_THREAD_MANAGEMENT();
   if(G3){
      if(Not IsBadWritePtr(G3, sizeof(CG3Decode))){
         return ((CG3Decode *)G3)->UncompressG3ToBinary(SrcG3Buffer,
					                                    SrcG3BufSize,
					                                    DestBinBuffer,
					                                    DestBinBufSize,
					                                    LinesUncompressed,
					                                    CompressedBufferDone);
      }
   }
   MsgDiag(_T("Bad G3 Engine"), MB_ICONSTOP|MB_TOPMOST);
   return FALSE;
}
BOOL __stdcall G4ReaderUncompress (HANDLE G4, PBYTE  SrcG4Buffer,
					                       DWORD  SrcG4BufSize,
					                       PBYTE  DestBinBuffer,
					                       DWORD  DestBinBufSize,
					                       PDWORD LinesUncompressed,
					                       PBOOL  CompressedBufferDone){
DLL_THREAD_MANAGEMENT();
   if(G4){
      if(Not IsBadWritePtr(G4, sizeof(CG4Decode))){
         return ((CG4Decode *)G4)->UncompressG4ToBinary(SrcG4Buffer,
					                                    SrcG4BufSize,
					                                    DestBinBuffer,
					                                    DestBinBufSize,
					                                    LinesUncompressed,
					                                    CompressedBufferDone);
      }
   }
   MsgDiag(_T("Bad G4 Engine"), MB_ICONSTOP|MB_TOPMOST);
   return FALSE;
}
#if 0 // SLOW VERSION
//////////////////////////////////////////////////////////////////////////////////////////////
BOOL __stdcall G4Uncompress (PBYTE DataDst,  LONG DstBufferSize,
                             LONG  ImgWidth, LONG ImgHeight, // IN PIXEL
                             PBYTE RcvSrc,   LONG RcvCount)
{
DLL_THREAD_MANAGEMENT();
DWORD  DstWidthInBytes      = ImgWidth / 8;
DWORD  LineUnCompressed     = 0 ;
BOOL   CompressedBufferDone = 0;
HANDLE G4Handle = G4StartReaderEngine();
BOOL   Ret;

	Ret = G4ReaderInitialize(G4Handle, DstWidthInBytes);
	Ret = G4ReaderUncompress(G4Handle, RcvSrc, RcvCount,     DataDst, DstBufferSize,
		                               &LineUnCompressed,    &CompressedBufferDone);
	G4ReleaseReaderEngine(G4Handle);
/*
CG4Decode G4Decoder;

    if(G4Decoder.Initialize(DstWidthInBytes)){
       G4Decoder.UncompressG4ToBinary (RcvSrc,
			                           RcvCount,
					                   DataDst,
					                   DstBufferSize,
					                   &LineUnCompressed,
					                   &CompressedBufferDone);
    }
*/
    return (LineUnCompressed == ImgHeight);
}
#endif
////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////
} // extern "C" {
////////////////////////////////////////////////////////////////////////////////////////////////////////


