// G_Compress.h
// date: Jul,2000
//
// Header file for G3 and G4 encoding
// The algorithm is based on ITU-T Recommendation T.4 and T.6
#ifndef __G_COMPRESS_H_
#define __G_COMPRESS_H_





#ifndef  EchoWarnString
#define  OutputMessageString(A)  OutputDebugString (A)
#define  OutputFrameString(A)    OutputDebugString (A)
#define  OutputStateString(A)    OutputDebugString (A)

#define  EchoInfoString(A)
#define  EchoDebugString(A)      OutputDebugString (A)
#define  EchoWarnString(A)       OutputDebugString (A)
#define  EchoErrorString(A)      OutputDebugString (A)
#endif

#ifndef _G4_Compress_
#define _G4_Compress_

// Note that these tables are ordered such that the
// index into the table is known to be either the
// run length, or (run length / 64) + a fixed offset.
//
// NB: The G3CODE_INVALID entries are only used
//     during state generation .
#ifndef _NEWG3G4TABLE
#pragma message (">>>> Using Old G tables support.")

const TableEntry TIFFaxZeroCodes[] = {
    { 8,  0x35, 0 },                   // 0011 0101
    { 6,  0x7,  1 },                   // 0001 11
    { 4,  0x7,  2 },                   // 0111
    { 4,  0x8,  3 },                   // 1000
    { 4,  0xB,  4 },                   // 1011
    { 4,  0xC,  5 },                   // 1100
    { 4,  0xE,  6 },                   // 1110
    { 4,  0xF,  7 },                   // 1111
    { 5,  0x13, 8 },                   // 1001 1
    { 5,  0x14, 9 },                   // 1010 0
    { 5,  0x7,  10 },                  // 0011 1
    { 5,  0x8,  11 },                  // 0100 0
    { 6,  0x8,  12 },                  // 0010 00
    { 6,  0x3,  13 },                  // 0000 11
    { 6,  0x34, 14 },                  // 1101 00
    { 6,  0x35, 15 },                  // 1101 01
    { 6,  0x2A, 16 },                  // 1010 10
    { 6,  0x2B, 17 },                  // 1010 11
    { 7,  0x27, 18 },                  // 0100 111
    { 7,  0xC,  19 },                  // 0001 100
    { 7,  0x8,  20 },                  // 0001 000
    { 7,  0x17, 21 },                  // 0010 111
    { 7,  0x3,  22 },                  // 0000 011
    { 7,  0x4,  23 },                  // 0000 100
    { 7,  0x28, 24 },                  // 0101 000
    { 7,  0x2B, 25 },                  // 0101 011
    { 7,  0x13, 26 },                  // 0010 011
    { 7,  0x24, 27 },                  // 0100 100
    { 7,  0x18, 28 },                  // 0011 000
    { 8,  0x2,  29 },                  // 0000 0010
    { 8,  0x3,  30 },                  // 0000 0011
    { 8,  0x1A, 31 },                  // 0001 1010
    { 8,  0x1B, 32 },                  // 0001 1011
    { 8,  0x12, 33 },                  // 0001 0010
    { 8,  0x13, 34 },                  // 0001 0011
    { 8,  0x14, 35 },                  // 0001 0100
    { 8,  0x15, 36 },                  // 0001 0101
    { 8,  0x16, 37 },                  // 0001 0110
    { 8,  0x17, 38 },                  // 0001 0111
    { 8,  0x28, 39 },                  // 0010 1000
    { 8,  0x29, 40 },                  // 0010 1001
    { 8,  0x2A, 41 },                  // 0010 1010
    { 8,  0x2B, 42 },                  // 0010 1011
    { 8,  0x2C, 43 },                  // 0010 1100
    { 8,  0x2D, 44 },                  // 0010 1101
    { 8,  0x4,  45 },                  // 0000 0100
    { 8,  0x5,  46 },                  // 0000 0101
    { 8,  0xA,  47 },                  // 0000 1010
    { 8,  0xB,  48 },                  // 0000 1011
    { 8,  0x52, 49 },                  // 0101 0010
    { 8,  0x53, 50 },                  // 0101 0011
    { 8,  0x54, 51 },                  // 0101 0100
    { 8,  0x55, 52 },                  // 0101 0101
    { 8,  0x24, 53 },                  // 0010 0100
    { 8,  0x25, 54 },                  // 0010 0101
    { 8,  0x58, 55 },                  // 0101 1000
    { 8,  0x59, 56 },                  // 0101 1001
    { 8,  0x5A, 57 },                  // 0101 1010
    { 8,  0x5B, 58 },                  // 0101 1011
    { 8,  0x4A, 59 },                  // 0100 1010
    { 8,  0x4B, 60 },                  // 0100 1011
    { 8,  0x32, 61 },                  // 0011 0010
    { 8,  0x33, 62 },                  // 0011 0011
    { 8,  0x34, 63 },                  // 0011 0100
    { 5,  0x1B, 64 },                  // 1101 1
    { 5,  0x12, 128 },                 // 1001 0
    { 6,  0x17, 192 },                 // 0101 11
    { 7,  0x37, 256 },                 // 0110 111
    { 8,  0x36, 320 },                 // 0011 0110
    { 8,  0x37, 384 },                 // 0011 0111
    { 8,  0x64, 448 },                 // 0110 0100
    { 8,  0x65, 512 },                 // 0110 0101
    { 8,  0x68, 576 },                 // 0110 1000
    { 8,  0x67, 640 },                 // 0110 0111
    { 9,  0xCC, 704 },                 // 0110 0110 0
    { 9,  0xCD, 768 },                 // 0110 0110 1
    { 9,  0xD2, 832 },                 // 0110 1001 0
    { 9,  0xD3, 896 },                 // 0110 1001 1
    { 9,  0xD4, 960 },                 // 0110 1010 0
    { 9,  0xD5, 1024 },                // 0110 1010 1
    { 9,  0xD6, 1088 },                // 0110 1011 0
    { 9,  0xD7, 1152 },                // 0110 1011 1
    { 9,  0xD8, 1216 },                // 0110 1100 0
    { 9,  0xD9, 1280 },                // 0110 1100 1
    { 9,  0xDA, 1344 },                // 0110 1101 0
    { 9,  0xDB, 1408 },                // 0110 1101 1
    { 9,  0x98, 1472 },                // 0100 1100 0
    { 9,  0x99, 1536 },                // 0100 1100 1
    { 9,  0x9A, 1600 },                // 0100 1101 0
    { 6,  0x18, 1664 },                // 0110 00
    { 9,  0x9B, 1728 },                // 0100 1101 1
    { 11, 0x8,  1792 },                // 0000 0001 000
    { 11, 0xC,  1856 },                // 0000 0001 100
    { 11, 0xD,  1920 },                // 0000 0001 101
    { 12, 0x12, 1984 },                // 0000 0001 0010
    { 12, 0x13, 2048 },                // 0000 0001 0011
    { 12, 0x14, 2112 },                // 0000 0001 0100
    { 12, 0x15, 2176 },                // 0000 0001 0101
    { 12, 0x16, 2240 },                // 0000 0001 0110
    { 12, 0x17, 2304 },                // 0000 0001 0111
    { 12, 0x1C, 2368 },                // 0000 0001 1100
    { 12, 0x1D, 2432 },                // 0000 0001 1101
    { 12, 0x1E, 2496 },                // 0000 0001 1110
    { 12, 0x1F, 2560 },                // 0000 0001 1111
    { 12, 0x1,  G4CODE_EOL },          // 0000 0000 0001
    { 9,  0x1,  G4CODE_INVALID },      // 0000 0000 1
    { 10, 0x1,  G4CODE_INVALID },      // 0000 0000 01
    { 11, 0x1,  G4CODE_INVALID },      // 0000 0000 001
    { 12, 0x0,  G4CODE_INVALID },      // 0000 0000 0000
};
//////////////////////////////////////////////////////////////////
// BREAKED DOWN FOR G4
//////////////////////////////////////////////////////////////////
const BYTE G3ZeroCode[][2] = {
    { 8,  0x35 },
    { 6,  0x7  },
    { 4,  0x7  },
    { 4,  0x8  },
    { 4,  0xB  },
    { 4,  0xC  },
    { 4,  0xE  },
    { 4,  0xF  },
    { 5,  0x13 },
    { 5,  0x14 },
    { 5,  0x7  },
    { 5,  0x8  },
    { 6,  0x8  },
    { 6,  0x3  },
    { 6,  0x34 },
    { 6,  0x35 },
    { 6,  0x2A },
    { 6,  0x2B },
    { 7,  0x27 },
    { 7,  0xC  },
    { 7,  0x8  },
    { 7,  0x17 },
    { 7,  0x3  },
    { 7,  0x4  },
    { 7,  0x28 },
    { 7,  0x2B },
    { 7,  0x13 },
    { 7,  0x24 },
    { 7,  0x18 },
    { 8,  0x2  },
    { 8,  0x3  },
    { 8,  0x1A },
    { 8,  0x1B },
    { 8,  0x12 },
    { 8,  0x13 },
    { 8,  0x14 },
    { 8,  0x15 },
    { 8,  0x16 },
    { 8,  0x17 },
    { 8,  0x28 },
    { 8,  0x29 },
    { 8,  0x2A },
    { 8,  0x2B },
    { 8,  0x2C },
    { 8,  0x2D },
    { 8,  0x4  },
    { 8,  0x5  },
    { 8,  0xA  },
    { 8,  0xB  },
    { 8,  0x52 },
    { 8,  0x53 },
    { 8,  0x54 },
    { 8,  0x55 },
    { 8,  0x24 },
    { 8,  0x25 },
    { 8,  0x58 },
    { 8,  0x59 },
    { 8,  0x5A },
    { 8,  0x5B },
    { 8,  0x4A },
    { 8,  0x4B },
    { 8,  0x32 },
    { 8,  0x33 },
    { 8,  0x34 },
    { 5,  0x1B },
    { 5,  0x12 },
    { 6,  0x17 },
    { 7,  0x37 },
    { 8,  0x36 },
    { 8,  0x37 },
    { 8,  0x64 },
    { 8,  0x65 },
    { 8,  0x68 },
    { 8,  0x67 },
    { 9,  0xCC },
    { 9,  0xCD },
    { 9,  0xD2 },
    { 9,  0xD3 },
    { 9,  0xD4 },
    { 9,  0xD5 },
    { 9,  0xD6 },
    { 9,  0xD7 },
    { 9,  0xD8 },
    { 9,  0xD9 },
    { 9,  0xDA },
    { 9,  0xDB },
    { 9,  0x98 },
    { 9,  0x99 },
    { 9,  0x9A },
    { 6,  0x18 },
    { 9,  0x9B },
    { 11, 0x8  },
    { 11, 0xC  },
    { 11, 0xD  },
    { 12, 0x12 },
    { 12, 0x13 },
    { 12, 0x14 },
    { 12, 0x15 },
    { 12, 0x16 },
    { 12, 0x17 },
    { 12, 0x1C },
    { 12, 0x1D },
    { 12, 0x1E },
    { 12, 0x1F },
    { 12, 0x1  },
    { 9,  0x1  },
    { 10, 0x1  },
    { 11, 0x1  },
    { 12, 0x0  },
};
const BYTE FaxZeroLenght[] = {
     8,
     6,
     4,
     4,
     4,
     4,
     4,
     4,
     5,
     5,
     5,
     5,
     6,
     6,
     6,
     6,
     6,
     6,
     7,
     7,
     7,
     7,
     7,
     7,
     7,
     7,
     7,
     7,
     7,
     8,
     8,
     8,
     8,
     8,
     8,
     8,
     8,
     8,
     8,
     8,
     8,
     8,
     8,
     8,
     8,
     8,
     8,
     8,
     8,
     8,
     8,
     8,
     8,
     8,
     8,
     8,
     8,
     8,
     8,
     8,
     8,
     8,
     8,
     8,
     5,
     5,
     6,
     7,
     8,
     8,
     8,
     8,
     8,
     8,
     9,
     9,
     9,
     9,
     9,
     9,
     9,
     9,
     9,
     9,
     9,
     9,
     9,
     9,
     9,
     6,
     9,
     11,
     11,
     11,
     12,
     12,
     12,
     12,
     12,
     12,
     12,
     12,
     12,
     12,
     12,
     9,
     10,
     11,
     12
};
const BYTE FaxZeroCode[] = {
    0x35,
    0x7,
    0x7,
    0x8,
    0xB,
    0xC,
    0xE,
    0xF,
    0x13,
    0x14,
    0x7,
    0x8,
    0x8,
    0x3,
    0x34,
    0x35,
    0x2A,
    0x2B,
    0x27,
    0xC,
    0x8,
    0x17,
    0x3,
    0x4,
    0x28,
    0x2B,
    0x13,
    0x24,
    0x18,
    0x2,
    0x3,
    0x1A,
    0x1B,
    0x12,
    0x13,
    0x14,
    0x15,
    0x16,
    0x17,
    0x28,
    0x29,
    0x2A,
    0x2B,
    0x2C,
    0x2D,
    0x4,
    0x5,
    0xA,
    0xB,
    0x52,
    0x53,
    0x54,
    0x55,
    0x24,
    0x25,
    0x58,
    0x59,
    0x5A,
    0x5B,
    0x4A,
    0x4B,
    0x32,
    0x33,
    0x34,
    0x1B,
    0x12,
    0x17,
    0x37,
    0x36,
    0x37,
    0x64,
    0x65,
    0x68,
    0x67,
    0xCC,
    0xCD,
    0xD2,
    0xD3,
    0xD4,
    0xD5,
    0xD6,
    0xD7,
    0xD8,
    0xD9,
    0xDA,
    0xDB,
    0x98,
    0x99,
    0x9A,
    0x18,
    0x9B,
    0x8,
    0xC,
    0xD,
    0x12,
    0x13,
    0x14,
    0x15,
    0x16,
    0x17,
    0x1C,
    0x1D,
    0x1E,
    0x1F,
    0x1,
    0x1,
    0x1,
    0x1,
    0x0
};
const WORD FaxZeroRunLenght[] = {
    0   ,
    1   ,
    2   ,
    3   ,
    4   ,
    5   ,
    6   ,
    7   ,
    8   ,
    9   ,
    10  ,
    11  ,
    12  ,
    13  ,
    14  ,
    15  ,
    16  ,
    17  ,
    18  ,
    19  ,
    20  ,
    21  ,
    22  ,
    23  ,
    24  ,
    25  ,
    26  ,
    27  ,
    28  ,
    29  ,
    30  ,
    31  ,
    32  ,
    33  ,
    34  ,
    35  ,
    36  ,
    37  ,
    38  ,
    39  ,
    40  ,
    41  ,
    42  ,
    43  ,
    44  ,
    45  ,
    46  ,
    47  ,
    48  ,
    49  ,
    50  ,
    51  ,
    52  ,
    53  ,
    54  ,
    55  ,
    56  ,
    57  ,
    58  ,
    59  ,
    60  ,
    61  ,
    62  ,
    63  ,
    64  ,
    128 ,
    192 ,
    256 ,
    320 ,
    384 ,
    448 ,
    512 ,
    576 ,
    640 ,
    704 ,
    768 ,
    832 ,
    896 ,
    960 ,
    1024 ,
    1088 ,
    1152 ,
    1216 ,
    1280 ,
    1344 ,
    1408 ,
    1472 ,
    1536 ,
    1600 ,
    1664 ,
    1728 ,
    1792 ,
    1856 ,
    1920 ,
    1984 ,
    2048 ,
    2112 ,
    2176 ,
    2240 ,
    2304 ,
    2368 ,
    2432 ,
    2496 ,
    2560 ,
    G4CODE_EOL ,
    G4CODE_INVALID ,
    G4CODE_INVALID ,
    G4CODE_INVALID ,
    G4CODE_INVALID
};
//////////////////////////////////////////////////////////////////
const TableEntry TIFFaxOneCodes[] = {
    { 10, 0x37, 0 },                   // 0000 1101 11
    { 3,  0x2,  1 },                   // 010
    { 2,  0x3,  2 },                   // 11
    { 2,  0x2,  3 },                   // 10
    { 3,  0x3,  4 },                   // 011
    { 4,  0x3,  5 },                   // 0011
    { 4,  0x2,  6 },                   // 0010
    { 5,  0x3,  7 },                   // 0001 1
    { 6,  0x5,  8 },                   // 0001 01
    { 6,  0x4,  9 },                   // 0001 00
    { 7,  0x4,  10 },                  // 0000 100
    { 7,  0x5,  11 },                  // 0000 101
    { 7,  0x7,  12 },                  // 0000 111
    { 8,  0x4,  13 },                  // 0000 0100
    { 8,  0x7,  14 },                  // 0000 0111
    { 9,  0x18, 15 },                  // 0000 1100 0
    { 10, 0x17, 16 },                  // 0000 0101 11
    { 10, 0x18, 17 },                  // 0000 0110 00
    { 10, 0x8,  18 },                  // 0000 0010 00
    { 11, 0x67, 19 },                  // 0000 1100 111
    { 11, 0x68, 20 },                  // 0000 1101 000
    { 11, 0x6C, 21 },                  // 0000 1101 100
    { 11, 0x37, 22 },                  // 0000 0110 111
    { 11, 0x28, 23 },                  // 0000 0101 000
    { 11, 0x17, 24 },                  // 0000 0010 111
    { 11, 0x18, 25 },                  // 0000 0011 000
    { 12, 0xCA, 26 },                  // 0000 1100 1010
    { 12, 0xCB, 27 },                  // 0000 1100 1011
    { 12, 0xCC, 28 },                  // 0000 1100 1100
    { 12, 0xCD, 29 },                  // 0000 1100 1101
    { 12, 0x68, 30 },                  // 0000 0110 1000
    { 12, 0x69, 31 },                  // 0000 0110 1001
    { 12, 0x6A, 32 },                  // 0000 0110 1010
    { 12, 0x6B, 33 },                  // 0000 0110 1011
    { 12, 0xD2, 34 },                  // 0000 1101 0010
    { 12, 0xD3, 35 },                  // 0000 1101 0011
    { 12, 0xD4, 36 },                  // 0000 1101 0100
    { 12, 0xD5, 37 },                  // 0000 1101 0101
    { 12, 0xD6, 38 },                  // 0000 1101 0110
    { 12, 0xD7, 39 },                  // 0000 1101 0111
    { 12, 0x6C, 40 },                  // 0000 0110 1100
    { 12, 0x6D, 41 },                  // 0000 0110 1101
    { 12, 0xDA, 42 },                  // 0000 1101 1010
    { 12, 0xDB, 43 },                  // 0000 1101 1011
    { 12, 0x54, 44 },                  // 0000 0101 0100
    { 12, 0x55, 45 },                  // 0000 0101 0101
    { 12, 0x56, 46 },                  // 0000 0101 0110
    { 12, 0x57, 47 },                  // 0000 0101 0111
    { 12, 0x64, 48 },                  // 0000 0110 0100
    { 12, 0x65, 49 },                  // 0000 0110 0101
    { 12, 0x52, 50 },                  // 0000 0101 0010
    { 12, 0x53, 51 },                  // 0000 0101 0011
    { 12, 0x24, 52 },                  // 0000 0010 0100
    { 12, 0x37, 53 },                  // 0000 0011 0111
    { 12, 0x38, 54 },                  // 0000 0011 1000
    { 12, 0x27, 55 },                  // 0000 0010 0111
    { 12, 0x28, 56 },                  // 0000 0010 1000
    { 12, 0x58, 57 },                  // 0000 0101 1000
    { 12, 0x59, 58 },                  // 0000 0101 1001
    { 12, 0x2B, 59 },                  // 0000 0010 1011
    { 12, 0x2C, 60 },                  // 0000 0010 1100
    { 12, 0x5A, 61 },                  // 0000 0101 1010
    { 12, 0x66, 62 },                  // 0000 0110 0110
    { 12, 0x67, 63 },                  // 0000 0110 0111
    { 10, 0xF,  64 },                  // 0000 0011 11
    { 12, 0xC8, 128  },                // 0000 1100 1000
    { 12, 0xC9, 192  },                // 0000 1100 1001
    { 12, 0x5B, 256  },                // 0000 0101 1011
    { 12, 0x33, 320  },                // 0000 0011 0011
    { 12, 0x34, 384  },                // 0000 0011 0100
    { 12, 0x35, 448  },                // 0000 0011 0101
    { 13, 0x6C, 512  },                // 0000 0011 0110 0
    { 13, 0x6D, 576  },                // 0000 0011 0110 1
    { 13, 0x4A, 640  },                // 0000 0010 0101 0
    { 13, 0x4B, 704  },                // 0000 0010 0101 1
    { 13, 0x4C, 768  },                // 0000 0010 0110 0
    { 13, 0x4D, 832  },                // 0000 0010 0110 1
    { 13, 0x72, 896  },                // 0000 0011 1001 0
    { 13, 0x73, 960  },                // 0000 0011 1001 1
    { 13, 0x74, 1024 },                // 0000 0011 1010 0
    { 13, 0x75, 1088 },                // 0000 0011 1010 1
    { 13, 0x76, 1152 },                // 0000 0011 1011 0
    { 13, 0x77, 1216 },                // 0000 0011 1011 1
    { 13, 0x52, 1280 },                // 0000 0010 1001 0
    { 13, 0x53, 1344 },                // 0000 0010 1001 1
    { 13, 0x54, 1408 },                // 0000 0010 1010 0
    { 13, 0x55, 1472 },                // 0000 0010 1010 1
    { 13, 0x5A, 1536 },                // 0000 0010 1101 0
    { 13, 0x5B, 1600 },                // 0000 0010 1101 1
    { 13, 0x64, 1664 },                // 0000 0011 0010 0
    { 13, 0x65, 1728 },                // 0000 0011 0010 1
    { 11, 0x8,  1792 },                // 0000 0001 000
    { 11, 0xC,  1856 },                // 0000 0001 100
    { 11, 0xD,  1920 },                // 0000 0001 101
    { 12, 0x12, 1984 },                // 0000 0001 0010
    { 12, 0x13, 2048 },                // 0000 0001 0011
    { 12, 0x14, 2112 },                // 0000 0001 0100
    { 12, 0x15, 2176 },                // 0000 0001 0101
    { 12, 0x16, 2240 },                // 0000 0001 0110
    { 12, 0x17, 2304 },                // 0000 0001 0111
    { 12, 0x1C, 2368 },                // 0000 0001 1100
    { 12, 0x1D, 2432 },                // 0000 0001 1101
    { 12, 0x1E, 2496 },                // 0000 0001 1110
    { 12, 0x1F, 2560 },                // 0000 0001 1111
    { 12, 0x1,  G4CODE_EOL },          // 0000 0000 0001
    { 9,  0x1,  G4CODE_INVALID },      // 0000 0000 1
    { 10, 0x1,  G4CODE_INVALID },      // 0000 0000 01
    { 11, 0x1,  G4CODE_INVALID },      // 0000 0000 001
    { 12, 0x0,  G4CODE_INVALID },      // 0000 0000 0000
};
//////////////////////////////////////////////////////////////////
// BREAKED DOWN FOR G4
//////////////////////////////////////////////////////////////////
const BYTE G3OneCode[][2] = {
    { 10, 0x37 },
    { 3,  0x2  },
    { 2,  0x3  },
    { 2,  0x2  },
    { 3,  0x3  },
    { 4,  0x3  },
    { 4,  0x2  },
    { 5,  0x3  },
    { 6,  0x5  },
    { 6,  0x4  },
    { 7,  0x4  },
    { 7,  0x5  },
    { 7,  0x7  },
    { 8,  0x4  },
    { 8,  0x7  },
    { 9,  0x18 },
    { 10, 0x17 },
    { 10, 0x18 },
    { 10, 0x8  },
    { 11, 0x67 },
    { 11, 0x68 },
    { 11, 0x6C },
    { 11, 0x37 },
    { 11, 0x28 },
    { 11, 0x17 },
    { 11, 0x18 },
    { 12, 0xCA },
    { 12, 0xCB },
    { 12, 0xCC },
    { 12, 0xCD },
    { 12, 0x68 },
    { 12, 0x69 },
    { 12, 0x6A },
    { 12, 0x6B },
    { 12, 0xD2 },
    { 12, 0xD3 },
    { 12, 0xD4 },
    { 12, 0xD5 },
    { 12, 0xD6 },
    { 12, 0xD7 },
    { 12, 0x6C },
    { 12, 0x6D },
    { 12, 0xDA },
    { 12, 0xDB },
    { 12, 0x54 },
    { 12, 0x55 },
    { 12, 0x56 },
    { 12, 0x57 },
    { 12, 0x64 },
    { 12, 0x65 },
    { 12, 0x52 },
    { 12, 0x53 },
    { 12, 0x24 },
    { 12, 0x37 },
    { 12, 0x38 },
    { 12, 0x27 },
    { 12, 0x28 },
    { 12, 0x58 },
    { 12, 0x59 },
    { 12, 0x2B },
    { 12, 0x2C },
    { 12, 0x5A },
    { 12, 0x66 },
    { 12, 0x67 },
    { 10, 0xF  },
    { 12, 0xC8 },
    { 12, 0xC9 },
    { 12, 0x5B },
    { 12, 0x33 },
    { 12, 0x34 },
    { 12, 0x35 },
    { 13, 0x6C },
    { 13, 0x6D },
    { 13, 0x4A },
    { 13, 0x4B },
    { 13, 0x4C },
    { 13, 0x4D },
    { 13, 0x72 },
    { 13, 0x73 },
    { 13, 0x74 },
    { 13, 0x75 },
    { 13, 0x76 },
    { 13, 0x77 },
    { 13, 0x52 },
    { 13, 0x53 },
    { 13, 0x54 },
    { 13, 0x55 },
    { 13, 0x5A },
    { 13, 0x5B },
    { 13, 0x64 },
    { 13, 0x65 },
    { 11, 0x8  },
    { 11, 0xC  },
    { 11, 0xD  },
    { 12, 0x12 },
    { 12, 0x13 },
    { 12, 0x14 },
    { 12, 0x15 },
    { 12, 0x16 },
    { 12, 0x17 },
    { 12, 0x1C },
    { 12, 0x1D },
    { 12, 0x1E },
    { 12, 0x1F },
    { 12, 0x1  },
    { 9,  0x1  },
    { 10, 0x1  },
    { 11, 0x1  },
    { 12, 0x0  },
};
const BYTE FaxOneLenght[] = {
     10,
     3,
     2,
     2,
     3,
     4,
     4,
     5,
     6,
     6,
     7,
     7,
     7,
     8,
     8,
     9,
     10,
     10,
     10,
     11,
     11,
     11,
     11,
     11,
     11,
     11,
     12,
     12,
     12,
     12,
     12,
     12,
     12,
     12,
     12,
     12,
     12,
     12,
     12,
     12,
     12,
     12,
     12,
     12,
     12,
     12,
     12,
     12,
     12,
     12,
     12,
     12,
     12,
     12,
     12,
     12,
     12,
     12,
     12,
     12,
     12,
     12,
     12,
     12,
     10,
     12,
     12,
     12,
     12,
     12,
     12,
     13,
     13,
     13,
     13,
     13,
     13,
     13,
     13,
     13,
     13,
     13,
     13,
     13,
     13,
     13,
     13,
     13,
     13,
     13,
     13,
     11,
     11,
     11,
     12,
     12,
     12,
     12,
     12,
     12,
     12,
     12,
     12,
     12,
     12,
     9,
     10,
     11,
     12
};
const BYTE FaxOneCode[] = {
    0x37,
    0x2,
    0x3,
    0x2,
    0x3,
    0x3,
    0x2,
    0x3,
    0x5,
    0x4,
    0x4,
    0x5,
    0x7,
    0x4,
    0x7,
    0x18,
    0x17,
    0x18,
    0x8,
    0x67,
    0x68,
    0x6C,
    0x37,
    0x28,
    0x17,
    0x18,
    0xCA,
    0xCB,
    0xCC,
    0xCD,
    0x68,
    0x69,
    0x6A,
    0x6B,
    0xD2,
    0xD3,
    0xD4,
    0xD5,
    0xD6,
    0xD7,
    0x6C,
    0x6D,
    0xDA,
    0xDB,
    0x54,
    0x55,
    0x56,
    0x57,
    0x64,
    0x65,
    0x52,
    0x53,
    0x24,
    0x37,
    0x38,
    0x27,
    0x28,
    0x58,
    0x59,
    0x2B,
    0x2C,
    0x5A,
    0x66,
    0x67,
    0xF,
    0xC8,
    0xC9,
    0x5B,
    0x33,
    0x34,
    0x35,
    0x6C,
    0x6D,
    0x4A,
    0x4B,
    0x4C,
    0x4D,
    0x72,
    0x73,
    0x74,
    0x75,
    0x76,
    0x77,
    0x52,
    0x53,
    0x54,
    0x55,
    0x5A,
    0x5B,
    0x64,
    0x65,
    0x8,
    0xC,
    0xD,
    0x12,
    0x13,
    0x14,
    0x15,
    0x16,
    0x17,
    0x1C,
    0x1D,
    0x1E,
    0x1F,
    0x1,
    0x1,
    0x1,
    0x1,
    0x0
};
const WORD FaxOneRunLenght[] = {
    0    ,
    1    ,
    2    ,
    3    ,
    4    ,
    5    ,
    6    ,
    7    ,
    8    ,
    9    ,
    10   ,
    11   ,
    12   ,
    13   ,
    14   ,
    15   ,
    16   ,
    17   ,
    18   ,
    19   ,
    20   ,
    21   ,
    22   ,
    23   ,
    24   ,
    25   ,
    26   ,
    27   ,
    28   ,
    29   ,
    30   ,
    31   ,
    32   ,
    33   ,
    34   ,
    35   ,
    36   ,
    37   ,
    38   ,
    39   ,
    40   ,
    41   ,
    42   ,
    43   ,
    44   ,
    45   ,
    46   ,
    47   ,
    48   ,
    49   ,
    50   ,
    51   ,
    52   ,
    53   ,
    54   ,
    55   ,
    56   ,
    57   ,
    58   ,
    59   ,
    60   ,
    61   ,
    62   ,
    63   ,
    64   ,
    128  ,
    192  ,
    256  ,
    320  ,
    384  ,
    448  ,
    512  ,
    576  ,
    640  ,
    704  ,
    768  ,
    832  ,
    896  ,
    960  ,
    1024 ,
    1088 ,
    1152 ,
    1216 ,
    1280 ,
    1344 ,
    1408 ,
    1472 ,
    1536 ,
    1600 ,
    1664 ,
    1728 ,
    1792 ,
    1856 ,
    1920 ,
    1984 ,
    2048 ,
    2112 ,
    2176 ,
    2240 ,
    2304 ,
    2368 ,
    2432 ,
    2496 ,
    2560 ,
    G4CODE_EOL ,
    G4CODE_INVALID ,
    G4CODE_INVALID ,
    G4CODE_INVALID ,
    G4CODE_INVALID
};
////////////////////////////
#else //#ifdef _NEWG3G4TABLE
////////////////////////////
#pragma message (">>>> Using New G tables support.")
#pragma message (">>>> New G tables Undefine.")

#endif
///////////////////////////////////////////////////////////////////
// Array ZeroRuns is used to determine zero runlength based on value of 8bits
// e.g if p=10000000b=0x80, zero runlength is zero, if p=00000001b=0x01,
// zeros is 7.
static const BYTE ZeroRuns[256] = {
    8, 7, 6, 6, 5, 5, 5, 5, 4, 4, 4, 4, 4, 4, 4, 4, // 0x00 - 0x0f
    3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, // 0x10 - 0x1f
    2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, // 0x20 - 0x2f
    2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, // 0x30 - 0x3f
    1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, // 0x40 - 0x4f
    1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, // 0x50 - 0x5f
    1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, // 0x60 - 0x6f
    1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, // 0x70 - 0x7f
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, // 0x80 - 0x8f
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, // 0x90 - 0x9f
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, // 0xa0 - 0xaf
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, // 0xb0 - 0xbf
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, // 0xc0 - 0xcf
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, // 0xd0 - 0xdf
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, // 0xe0 - 0xef
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, // 0xf0 - 0xff
};
///////////////////////////////////////////////////////////////////
// Array OneRuns is used to determine one runlength based on value of 8bits
// e.g if p=10000000b=0x80, one runlength is 1, if p=00000001b=0x01,
// one runlength is 7.
static const BYTE OneRuns[256] = {
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, // 0x00 - 0x0f
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, // 0x10 - 0x1f
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, // 0x20 - 0x2f
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, // 0x30 - 0x3f
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, // 0x40 - 0x4f
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, // 0x50 - 0x5f
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, // 0x60 - 0x6f
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, // 0x70 - 0x7f
    1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, // 0x80 - 0x8f
    1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, // 0x90 - 0x9f
    1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, // 0xa0 - 0xaf
    1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, // 0xb0 - 0xbf
    2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, // 0xc0 - 0xcf
    2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, // 0xd0 - 0xdf
    3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, // 0xe0 - 0xef
    4, 4, 4, 4, 4, 4, 4, 4, 5, 5, 5, 5, 6, 6, 7, 8, // 0xf0 - 0xff
};
////////////////////////////////////////////////////////////////////////
//                         0,1,   2,   3,   4,   5,   6,   7,   8
const BYTE PixelToBit []  = {0,0x80,0xC0,0XE0,0XF0,0XF8,0XFC,0XFE,0XFF};
// THIS TABLE CONVERT FROM PIXEL TO Pixel......................
////////////////////////////////////////////////////////////////////////
// BitMask[] is used in MacFindPixelColor to find color of a pixel
const BYTE BitMask    []  = {0x80,0x40,0X20,0X10,0X08,0X04,0X02,0X01};
////////////////////////////////////////////////////////////////////////
const BYTE MaskLeft  [9] = {0,0X01,0X03,0X07,0X0F,0X1F,0X3F,0X7F,0XFF};
// MaskLeft MASK THE LEFTSIDE OF THE BYTE BASE ON ITS VALUE.
////////////////////////////////////////////////////////////////////////
// FLAG CODE FOR PASS MODE
static const TableEntry passcode =
    { 4, 0x1 };     // 0001
/////////////////////////////////////////////
#define PassCodeLenght 4
#define PassCode      0x1
// FLAG CODE FOR HORIZONTAL MODE
static const TableEntry horizcode =
    { 3, 0x1 };     // 001
#define HorzCodeLenght 3
#define HorzCode       0x1
/////////////////////////////////////////////
// FLAG CODE FOR VERTICAL MODE
static const TableEntry VerticalCodes[7] = {
    { 7, 0x03 },    // 0000 011
    { 6, 0x03 },    // 0000 11
    { 3, 0x03 },    // 011
    { 1, 0x1 },     // 1
    { 3, 0x2 },     // 010
    { 6, 0x02 },    // 0000 10
    { 7, 0x02 }     // 0000 010
};
static const BYTE VerticalCode[][2] = {
    { 7, 0x03 },    // 0000 011
    { 6, 0x03 },    // 0000 11
    { 3, 0x03 },    // 011
    { 1, 0x1 },     // 1
    { 3, 0x2 },     // 010
    { 6, 0x02 },    // 0000 10
    { 7, 0x02 }     // 0000 010
};
// CODE FOR END OF FAX BLOCK
static const DWORD EOFL =  0x001001; // 24 BIT..............................
                          // EOFB code value = 0000 0000 0001 0000 0000 0001
//////////////////////////////////////////////////////////////////////////////////////////
extern "C" {
//////////////////////////////////////////////////////////////////////////////////////////

BOOL __stdcall G3Uncompress  (PBYTE DataDst, LONG DataCount, PBYTE RcvSrc, LONG RcvCount);
long __stdcall G3Compress    (PBYTE ImgLine, DWORD BufLen);

} // extern "C"

void   ClearRLReport          (); // RESET VALUE TO ZERO

// FOR STD TABLE
void   ReportRunLen           (PBYTE Buffer,  DWORD BufLen);
void   ReportPageRunLen       (PBYTE Buffer,  DWORD BufLen);

// FOR RELOCATED TABLE
void   ReportNewPageRunLen    (PBYTE Buffer,  DWORD BufLen);
void   ReportNewPageRunLen    (PBYTE Buffer,  DWORD BufLen);

void   ReportRunLen   (PBYTE Buffer,  DWORD BufLen);
void   ReportNewRunLen(PBYTE Buffer,  DWORD BufLen);
#endif

#endif // G_COMPRESS
