/*
 * ringbuffer.cpp
 *
 *  Created on: Dec 11, 2008
 *      Author: root
 */
#include "StdAfx.h"
P_RB rb_send;
P_RB rb_recv;

bool rb_create(P_RB rb){
	bool ret;
	ret = true;
	rb->container = (unsigned char*)malloc(RECV_BUFFER_SIZE);
	rb = (P_RB)malloc(sizeof(RB));
	if (!rb)
		ret = false;
	return ret;
}

void rb_init(P_RB rb){
	memset((PVOID)rb->container,RECV_BUFFER_SIZE,0);
	rb->header =rb->container;
	rb->wp = rb->header;
	rb->rp = rb->header;
	rb->tail = rb->header;
	rb->available_to_write = RECV_BUFFER_SIZE;
	rb->available_to_read = 0;
	rb->ready_to_write = true ;
	rb->ready_to_read = false;
}
void rb_update(P_RB rb,int action){
	switch (action)
	{
	case RB_WRITE:
		if(rb->wp > rb->rp){
			rb->available_to_read = rb->wp - rb->rp;
			rb->available_to_write = RECV_BUFFER_SIZE - rb->available_to_read;
			rb->ready_to_read = true;
			rb->ready_to_write = true;
		}
		else if (rb->wp < rb->rp){
			rb->available_to_write = rb->rp - rb->wp;
			rb->available_to_read = RECV_BUFFER_SIZE - rb->available_to_write;
			rb->ready_to_read = true;
			rb->ready_to_write = true;
		}
		else{
			rb->available_to_write = 0;
			rb->available_to_read = RECV_BUFFER_SIZE;
			rb->ready_to_read = true;
			rb->ready_to_write = false;
		}

		break;
	case RB_READ:
		if(rb->wp > rb->rp){
				rb->available_to_read = rb->wp - rb->rp;
				rb->available_to_write = RECV_BUFFER_SIZE - rb->available_to_read;
				rb->ready_to_read = true;
				rb->ready_to_write = true;
		}
		else if (rb->wp < rb->rp){
				rb->available_to_write = rb->rp - rb->wp;
				rb->available_to_read = RECV_BUFFER_SIZE - rb->available_to_write;
				rb->ready_to_read = true;
				rb->ready_to_write = true;
		}
		else{
				rb->available_to_read = 0;
				rb->available_to_write = RECV_BUFFER_SIZE;
				rb->ready_to_write = true;
				rb->ready_to_read = false;
		}
		break;
	default:
		break;
	}
}
size_t rb_write(P_RB rb,LPCSTR data, size_t len){
	size_t byteswritten;
	size_t bytesleft;
	size_t wptotail;

	if (len > rb->available_to_write){
		return 0;
	}
	wptotail = rb->tail -rb->wp + 1;
	if( wptotail >= len){
		memcpy(rb->wp,data,len);
		rb->wp = rb->wp + len;
		byteswritten += len;
	}
	else{
		memcpy(rb->wp,data,wptotail);
		rb->wp = rb->header;
		byteswritten = wptotail;
		bytesleft = len - byteswritten;
		memcpy(rb->wp,(const void *)data[byteswritten],bytesleft);
		rb->wp = rb->wp + bytesleft;
		byteswritten += bytesleft;
		bytesleft = 0;
	}

	rb_update(rb,RB_WRITE);

	return byteswritten;
}
size_t rb_read(P_RB rb,const unsigned char* buf, size_t len){
	size_t bytesread;
	size_t bytesleft;
	size_t rptotail;

	rptotail = rb->tail -rb->rp + 1;

	if (len > rb->available_to_read){
		if(rptotail >= len){
			bytesread=rb->available_to_read;
			memcpy((PVOID)buf,rb->rp,bytesread);
			rb->rp = rb->rp + bytesread;
			//return bytesread;
		}
	else{

			memcpy((PVOID)buf,rb->rp,rptotail);
			bytesread=rptotail;
			bytesleft = rb->available_to_read - bytesread;
			rb->rp = rb->header;
			memcpy((PVOID)buf[bytesread],rb->rp,bytesleft);
			rb->rp = rb->rp + bytesleft;
			bytesread += bytesleft;
			bytesleft = 0;

		}
	}
	else{

		if(rptotail >= len){
				bytesread=len;
				memcpy((PVOID)buf,rb->rp,bytesread);
				rb->rp = rb->rp + bytesread;
				//return bytesread;
		}
		else{

				memcpy((PVOID)buf,rb->rp,rptotail);
				bytesread=rptotail;
				bytesleft = len - bytesread;
				rb->rp = rb->header;
				memcpy((PVOID)buf[bytesread],rb->rp,bytesleft);
				rb->rp = rb->rp + bytesleft;
				bytesread += bytesleft;
				bytesleft = 0;

		}
	}

	rb_update(rb,RB_WRITE);
	return bytesread;

}
void rb_destroy(P_RB rb){
	free(rb);
}
