// UnZipLib.h
///////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef _UnZipLib_H
#define _UnZipLib_H
////////////////////////////////////////////////////////////////////////////////////////////////////
#ifdef __cplusplus
extern "C" {
#endif

    ///////////////////////////////////////////////////////////////////////////////////////////////
    #define Z_OK              0
    #define Z_STREAM_END      1
    #define Z_NEED_DICT       2
    #define Z_ERRNO         (-1)
    #define Z_STREAM_ERROR  (-2)
    #define Z_DATA_ERROR    (-3)
    #define Z_MEM_ERROR     (-4)
    #define Z_BUF_ERROR     (-5)
    #define Z_VERSION_ERROR (-6)

    #define Z_INIT_ERROR    (0xE0) // 224
    #define Z_DISPLAY_ERROR (0xE1) // 225

    ///////////////////////////////////////////////////////////////////////////////////////////////
    #define Z_BAD_BLOCKTYPE       (0X80)  // 128 ReportErrorStr("invalid block type");
    #define Z_BAD_BLOCKLEN        (0X81)  // 129 ReportErrorStr("invalid stored block lengths");
    #define Z_BAD_TOOMANY         (0X82)  // 130 ReportErrorStr("too many length or distance symbols");
    #define Z_BAD_BITLENREPEAT    (0X83)  // 131 ReportErrorStr("invalid bit length repeat");
    #define Z_BAD_DICLENCODE      (0X84)  // 132 ReportErrorStr("invalid literal/length code");
    #define Z_BAD_DISTCODE        (0X85)  // 133 ReportErrorStr("invalid distance code");
    #define Z_BAD_COMPMETHODE     (0X86)  // 134 ReportErrorStr("unknown compression Method");
    #define Z_BAD_WINDOWSIZE      (0X87)  // 135 ReportErrorStr("invalid WinSlide size");
    #define Z_BAD_HEADERCHECK     (0X88)  // 136 ReportErrorStr("incorrect header check");
    #define Z_BAD_DICTNEED        (0X89)  // 137 ReportErrorStr("need dictionary");
    #define Z_BAD_DATACHECK       (0X8A)  // 138 ReportErrorStr("incorrect data check");
    #define Z_BAD_OVERSUBDYNAMIC  (0X8B)  // 139 ReportErrorStr("oversubscribed dynamic bit lengths tree");
    #define Z_BAD_DYNAMICLEN      (0X8C)  // 140 ReportErrorStr("incomplete dynamic bit lengths tree");
    #define Z_BAD_OVERLITLEN      (0X8D)  // 141 ReportErrorStr("oversubscribed literal/length tree")
    #define Z_BAD_COMPLETELIT     (0X8E)  // 142 ReportErrorStr("incomplete literal/length tree");
    #define Z_BAD_OVERDIST        (0X8F)  // 143 ReportErrorStr("oversubscribed distance tree");
    #define Z_BAD_COMPLETEDIST    (0X90)  // 144 ReportErrorStr("incomplete distance tree");
    #define Z_BAD_EMPTYDIST       (0X91)  // 145 ReportErrorStr("empty distance tree with lengths");
    #define Z_BAD_BUF_ERROR       (0X92)  // 146 ReportErrorStr("Buffer insufisant");

    #define Z_BAD_XORCHECK        (0X93)  // 147 ReportErrorStr("incorrect XOR check");
    // RETURN CODES FOR THE COMPRESSION/DECOMPRESSION FUNCTIONS. NEGATIVE
    // VALUES ARE ERRORS, POSITIVE VALUES ARE USED FOR SPECIAL BUT NORMAL EVENTS.

    ///////////////////////////////////////////////////////////////////////////////////////////////
    #define Z_NO_FLUSH      0
    #define Z_PARTIAL_FLUSH 1 // WILL BE REMOVED, USE Z_SYNC_FLUSH INSTEAD 
    #define Z_SYNC_FLUSH    2
    #define Z_FULL_FLUSH    3
    #define Z_FINISH        4

    ///////////////////////////////////////////////////////////////////////////////////////////////
    #define Z_NO_COMPRESSION         0
    #define Z_BEST_SPEED             1
    #define Z_BEST_COMPRESSION       9
    #define Z_DEFAULT_COMPRESSION  (-1)

    ///////////////////////////////////////////////////////////////////////////////////////////////
    // COMPRESSION STRATEGY; SEE DEFLATEINIT2() BELOW FOR DETAILS 
    #define Z_FILTERED            1
    #define Z_HUFFMAN_ONLY        2
    #define Z_DEFAULT_STRATEGY    0

    ///////////////////////////////////////////////////////////////////////////////////////////////
    // POSSIBLE VALUES OF THE DATA_TYPE FIELD
    #define Z_BINARY   0
    #define Z_ASCII    1
    #define Z_UNKNOWN  2

    ///////////////////////////////////////////////////////////////////////////////////////////////
    // THE DEFLATE COMPRESSION METHOD (THE ONLY ONE SUPPORTED IN THIS VERSION) 
    #define Z_DEFLATED   8

    #define Z_NULL       0 

    #define Z_TRUE       1
    #define Z_FALSE      0

    ///////////////////////////////////////////////////////////////////////////////////////////////
    #ifndef FAR
        #define FAR
    #endif 
    #define CFAR FAR 
    ///////////////////////////////////////////////////////////////////////////////////////////////
    typedef       signed   char         Char;    // 8 bits 
    typedef       signed   short        Short;   // 16 bits only
    typedef       signed   int          Int;     // 16 bits or more 
    typedef       signed   long         Long;    // 32 bits or more 

    typedef       unsigned char         Byte;    // 8 bits 
    typedef       unsigned char         BYTE;    // 8 bits 
    typedef       unsigned short        UShort;  // 16 bits only
    typedef       unsigned int          UInt;    // 16 bits or more 
    typedef       unsigned int          UINT;    // 16 bits or more 
    typedef       unsigned long         ULong;   // 32 bits or more 
    typedef       unsigned long         ULONG;   // 32 bits or more 

    typedef       signed   char  CFAR * PChar;   // 8 bits 
    typedef       signed   short CFAR * PShort;  // 16 bits only
    typedef       signed   int   CFAR * PInt;    // 16 bits or more 
    typedef       signed   long  CFAR * PLong;   // 32 bits or more 

    typedef       unsigned char  CFAR * PByte;   // 8 bits 
    typedef       unsigned char  CFAR * PBYTE;   // 8 bits 
    typedef       unsigned short CFAR * PUShort; // 16 bits only
    typedef       unsigned int   CFAR * PUInt;   // 16 bits or more 
    typedef       unsigned long  CFAR * PULong;  // 32 bits or more 

    typedef const signed   char  CFAR * CPChar;  // 8 bits 
    typedef const signed   short CFAR * CPShort; // 16 bits only
    typedef const signed   int   CFAR * CPInt;   // 16 bits or more 
    typedef const signed   long  CFAR * CPLong;  // 32 bits or more 

    typedef const unsigned char  CFAR * CPByte;  // 8 bits 
    typedef const unsigned short CFAR * CPUShort;// 16 bits only
    typedef const unsigned int   CFAR * CPUInt;  // 16 bits or more 
    typedef const unsigned long  CFAR * CPULong; // 32 bits or more 

    ///////////////////////////////////////////////////////////////////////////////////////////////
    typedef                void  CFAR * PVoid;
    ///////////////////////////////////////////////////////////////////////////////////////////////
    #ifndef OF
    #  ifdef STDC
    #    define OF(args)  args
    #  else
    #    define OF(args)  ()
    #  endif
    #endif
    #ifndef Local
    #  define Local static
    #endif


////////////////////////////////////////////////////////////////////////////////////////////////////
Int  Z_Uncompress (PByte dest, PULong destLen, PByte source, UInt sourceLen);
////////////////////////////////////////////////////////////////////////////////////////////////////
#ifdef __cplusplus
}
#endif
////////////////////////////////////////////////////////////////////////////////////////////////////
#endif // _UnZipLib_H

